// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/asset/v1/asset_service.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_ASSET_ASSET_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_ASSET_ASSET_CLIENT_H

#include "google/cloud/asset/asset_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace asset {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Asset service definition.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class AssetServiceClient {
 public:
  explicit AssetServiceClient(
      std::shared_ptr<AssetServiceConnection> connection, Options opts = {});
  ~AssetServiceClient();

  ///@{
  // @name Copy and move support
  AssetServiceClient(AssetServiceClient const&) = default;
  AssetServiceClient& operator=(AssetServiceClient const&) = default;
  AssetServiceClient(AssetServiceClient&&) = default;
  AssetServiceClient& operator=(AssetServiceClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(AssetServiceClient const& a,
                         AssetServiceClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(AssetServiceClient const& a,
                         AssetServiceClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Exports assets with time and resource types to a given Cloud Storage
  /// location/BigQuery table. For Cloud Storage location destinations, the
  /// output format is newline-delimited JSON. Each line represents a
  /// [google.cloud.asset.v1.Asset][google.cloud.asset.v1.Asset] in the JSON
  /// format; for BigQuery table destinations, the output table stores the
  /// fields in asset Protobuf as columns. This API implements the
  /// [google.longrunning.Operation][google.longrunning.Operation] API, which
  /// allows you to keep track of the export. We recommend intervals of at least
  /// 2 seconds with exponential retry to poll the export operation result. For
  /// regular-size resource parent, the export operation usually finishes within
  /// 5 minutes.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::ExportAssetsRequest,google/cloud/asset/v1/asset_service.proto#L272}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::ExportAssetsResponse,google/cloud/asset/v1/asset_service.proto#L339}
  ///
  /// [google.cloud.asset.v1.ExportAssetsRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L272}
  /// [google.cloud.asset.v1.ExportAssetsResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L339}
  ///
  future<StatusOr<google::cloud::asset::v1::ExportAssetsResponse>> ExportAssets(
      google::cloud::asset::v1::ExportAssetsRequest const& request,
      Options opts = {});

  ///
  /// Lists assets with time and resource types and returns paged results in
  /// response.
  ///
  /// @param parent  Required. Name of the organization, folder, or project the
  /// assets belong to. Format:
  ///  "organizations/[organization-number]" (such as "organizations/123"),
  ///  "projects/[project-id]" (such as "projects/my-project-id"),
  ///  "projects/[project-number]" (such as "projects/12345"), or
  ///  "folders/[folder-number]" (such as "folders/12345").
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::Asset,google/cloud/asset/v1/assets.proto#L96}
  ///
  /// [google.cloud.asset.v1.ListAssetsRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L355}
  /// [google.cloud.asset.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/asset/v1/assets.proto#L96}
  ///
  StreamRange<google::cloud::asset::v1::Asset> ListAssets(
      std::string const& parent, Options opts = {});

  ///
  /// Lists assets with time and resource types and returns paged results in
  /// response.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::ListAssetsRequest,google/cloud/asset/v1/asset_service.proto#L355}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::Asset,google/cloud/asset/v1/assets.proto#L96}
  ///
  /// [google.cloud.asset.v1.ListAssetsRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L355}
  /// [google.cloud.asset.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/asset/v1/assets.proto#L96}
  ///
  StreamRange<google::cloud::asset::v1::Asset> ListAssets(
      google::cloud::asset::v1::ListAssetsRequest request, Options opts = {});

  ///
  /// Batch gets the update history of assets that overlap a time window.
  /// For IAM_POLICY content, this API outputs history when the asset and its
  /// attached IAM POLICY both exist. This can create gaps in the output
  /// history. Otherwise, this API outputs history with asset in both non-delete
  /// or deleted status. If a specified asset does not exist, this API returns
  /// an INVALID_ARGUMENT error.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::BatchGetAssetsHistoryRequest,google/cloud/asset/v1/asset_service.proto#L441}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::BatchGetAssetsHistoryResponse,google/cloud/asset/v1/asset_service.proto#L493}
  ///
  /// [google.cloud.asset.v1.BatchGetAssetsHistoryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L441}
  /// [google.cloud.asset.v1.BatchGetAssetsHistoryResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L493}
  ///
  StatusOr<google::cloud::asset::v1::BatchGetAssetsHistoryResponse>
  BatchGetAssetsHistory(
      google::cloud::asset::v1::BatchGetAssetsHistoryRequest const& request,
      Options opts = {});

  ///
  /// Creates a feed in a parent project/folder/organization to listen to its
  /// asset updates.
  ///
  /// @param parent  Required. The name of the project/folder/organization where
  /// this feed
  ///  should be created in. It can only be an organization number (such as
  ///  "organizations/123"), a folder number (such as "folders/123"), a project
  ///  ID (such as "projects/my-project-id")", or a project number (such as
  ///  "projects/12345").
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::Feed,google/cloud/asset/v1/asset_service.proto#L753}
  ///
  /// [google.cloud.asset.v1.CreateFeedRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L499}
  /// [google.cloud.asset.v1.Feed]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L753}
  ///
  StatusOr<google::cloud::asset::v1::Feed> CreateFeed(std::string const& parent,
                                                      Options opts = {});

  ///
  /// Creates a feed in a parent project/folder/organization to listen to its
  /// asset updates.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::CreateFeedRequest,google/cloud/asset/v1/asset_service.proto#L499}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::Feed,google/cloud/asset/v1/asset_service.proto#L753}
  ///
  /// [google.cloud.asset.v1.CreateFeedRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L499}
  /// [google.cloud.asset.v1.Feed]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L753}
  ///
  StatusOr<google::cloud::asset::v1::Feed> CreateFeed(
      google::cloud::asset::v1::CreateFeedRequest const& request,
      Options opts = {});

  ///
  /// Gets details about an asset feed.
  ///
  /// @param name  Required. The name of the Feed and it must be in the format
  /// of:
  ///  projects/project_number/feeds/feed_id
  ///  folders/folder_number/feeds/feed_id
  ///  organizations/organization_number/feeds/feed_id
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::Feed,google/cloud/asset/v1/asset_service.proto#L753}
  ///
  /// [google.cloud.asset.v1.GetFeedRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L520}
  /// [google.cloud.asset.v1.Feed]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L753}
  ///
  StatusOr<google::cloud::asset::v1::Feed> GetFeed(std::string const& name,
                                                   Options opts = {});

  ///
  /// Gets details about an asset feed.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::GetFeedRequest,google/cloud/asset/v1/asset_service.proto#L520}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::Feed,google/cloud/asset/v1/asset_service.proto#L753}
  ///
  /// [google.cloud.asset.v1.GetFeedRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L520}
  /// [google.cloud.asset.v1.Feed]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L753}
  ///
  StatusOr<google::cloud::asset::v1::Feed> GetFeed(
      google::cloud::asset::v1::GetFeedRequest const& request,
      Options opts = {});

  ///
  /// Lists all asset feeds in a parent project/folder/organization.
  ///
  /// @param parent  Required. The parent project/folder/organization whose
  /// feeds are to be
  ///  listed. It can only be using project/folder/organization number (such as
  ///  "folders/12345")", or a project ID (such as "projects/my-project-id").
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::ListFeedsResponse,google/cloud/asset/v1/asset_service.proto#L541}
  ///
  /// [google.cloud.asset.v1.ListFeedsRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L534}
  /// [google.cloud.asset.v1.ListFeedsResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L541}
  ///
  StatusOr<google::cloud::asset::v1::ListFeedsResponse> ListFeeds(
      std::string const& parent, Options opts = {});

  ///
  /// Lists all asset feeds in a parent project/folder/organization.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::ListFeedsRequest,google/cloud/asset/v1/asset_service.proto#L534}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::ListFeedsResponse,google/cloud/asset/v1/asset_service.proto#L541}
  ///
  /// [google.cloud.asset.v1.ListFeedsRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L534}
  /// [google.cloud.asset.v1.ListFeedsResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L541}
  ///
  StatusOr<google::cloud::asset::v1::ListFeedsResponse> ListFeeds(
      google::cloud::asset::v1::ListFeedsRequest const& request,
      Options opts = {});

  ///
  /// Updates an asset feed configuration.
  ///
  /// @param feed  Required. The new values of feed details. It must match an
  /// existing feed and the
  ///  field `name` must be in the format of:
  ///  projects/project_number/feeds/feed_id or
  ///  folders/folder_number/feeds/feed_id or
  ///  organizations/organization_number/feeds/feed_id.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::Feed,google/cloud/asset/v1/asset_service.proto#L753}
  ///
  /// [google.cloud.asset.v1.UpdateFeedRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L547}
  /// [google.cloud.asset.v1.Feed]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L753}
  ///
  StatusOr<google::cloud::asset::v1::Feed> UpdateFeed(
      google::cloud::asset::v1::Feed const& feed, Options opts = {});

  ///
  /// Updates an asset feed configuration.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::UpdateFeedRequest,google/cloud/asset/v1/asset_service.proto#L547}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::Feed,google/cloud/asset/v1/asset_service.proto#L753}
  ///
  /// [google.cloud.asset.v1.UpdateFeedRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L547}
  /// [google.cloud.asset.v1.Feed]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L753}
  ///
  StatusOr<google::cloud::asset::v1::Feed> UpdateFeed(
      google::cloud::asset::v1::UpdateFeedRequest const& request,
      Options opts = {});

  ///
  /// Deletes an asset feed.
  ///
  /// @param name  Required. The name of the feed and it must be in the format
  /// of:
  ///  projects/project_number/feeds/feed_id
  ///  folders/folder_number/feeds/feed_id
  ///  organizations/organization_number/feeds/feed_id
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.asset.v1.DeleteFeedRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L561}
  ///
  Status DeleteFeed(std::string const& name, Options opts = {});

  ///
  /// Deletes an asset feed.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::DeleteFeedRequest,google/cloud/asset/v1/asset_service.proto#L561}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.asset.v1.DeleteFeedRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L561}
  ///
  Status DeleteFeed(google::cloud::asset::v1::DeleteFeedRequest const& request,
                    Options opts = {});

  ///
  /// Searches all Cloud resources within the specified scope, such as a
  /// project, folder, or organization. The caller must be granted the
  /// `cloudasset.assets.searchAllResources` permission on the desired scope,
  /// otherwise the request will be rejected.
  ///
  /// @param scope  Required. A scope can be a project, a folder, or an
  /// organization. The search is
  ///  limited to the resources within the `scope`. The caller must be granted
  ///  the
  ///  [`cloudasset.assets.searchAllResources`](https://cloud.google.com/asset-inventory/docs/access-control#required_permissions)
  ///  permission on the desired scope.
  ///  The allowed values are:
  ///  * projects/{PROJECT_ID} (e.g., "projects/foo-bar")
  ///  * projects/{PROJECT_NUMBER} (e.g., "projects/12345678")
  ///  * folders/{FOLDER_NUMBER} (e.g., "folders/1234567")
  ///  * organizations/{ORGANIZATION_NUMBER} (e.g., "organizations/123456")
  /// @param query  Optional. The query statement. See [how to construct a
  ///  query](https://cloud.google.com/asset-inventory/docs/searching-resources#how_to_construct_a_query)
  ///  for more information. If not specified or empty, it will search all the
  ///  resources within the specified `scope`.
  ///  Examples:
  ///  * `name:Important` to find Cloud resources whose name contains
  ///    "Important" as a word.
  ///  * `name=Important` to find the Cloud resource whose name is exactly
  ///    "Important".
  ///  * `displayName:Impor*` to find Cloud resources whose display name
  ///    contains "Impor" as a prefix of any word in the field.
  ///  * `location:us-west*` to find Cloud resources whose location contains
  ///  both
  ///    "us" and "west" as prefixes.
  ///  * `labels:prod` to find Cloud resources whose labels contain "prod" as
  ///    a key or value.
  ///  * `labels.env:prod` to find Cloud resources that have a label "env"
  ///    and its value is "prod".
  ///  * `labels.env:*` to find Cloud resources that have a label "env".
  ///  * `kmsKey:key` to find Cloud resources encrypted with a customer-managed
  ///    encryption key whose name contains "key" as a word. This field is
  ///    deprecated. Please use the `kmsKeys` field to retrieve KMS key
  ///    information.
  ///  * `kmsKeys:key` to find Cloud resources encrypted with customer-managed
  ///    encryption keys whose name contains the word "key".
  ///  * `relationships:instance-group-1` to find Cloud resources that have
  ///    relationships with "instance-group-1" in the related resource name.
  ///  * `relationships:INSTANCE_TO_INSTANCEGROUP` to find compute instances
  ///  that
  ///    have relationships of type "INSTANCE_TO_INSTANCEGROUP".
  ///  * `relationships.INSTANCE_TO_INSTANCEGROUP:instance-group-1` to find
  ///    compute instances that have relationships with "instance-group-1" in
  ///    the compute instance group resource name, for relationship type
  ///    "INSTANCE_TO_INSTANCEGROUP".
  ///  * `state:ACTIVE` to find Cloud resources whose state contains "ACTIVE" as
  ///  a
  ///    word.
  ///  * `NOT state:ACTIVE` to find Cloud resources whose state doesn't contain
  ///    "ACTIVE" as a word.
  ///  * `createTime<1609459200` to find Cloud resources that were created
  ///  before
  ///    "2021-01-01 00:00:00 UTC". 1609459200 is the epoch timestamp of
  ///    "2021-01-01 00:00:00 UTC" in seconds.
  ///  * `updateTime>1609459200` to find Cloud resources that were updated after
  ///    "2021-01-01 00:00:00 UTC". 1609459200 is the epoch timestamp of
  ///    "2021-01-01 00:00:00 UTC" in seconds.
  ///  * `Important` to find Cloud resources that contain "Important" as a word
  ///    in any of the searchable fields.
  ///  * `Impor*` to find Cloud resources that contain "Impor" as a prefix of
  ///  any
  ///    word in any of the searchable fields.
  ///  * `Important location:(us-west1 OR global)` to find Cloud
  ///    resources that contain "Important" as a word in any of the searchable
  ///    fields and are also located in the "us-west1" region or the "global"
  ///    location.
  /// @param asset_types  Optional. A list of asset types that this request
  /// searches for. If empty, it will
  ///  search all the [searchable asset
  ///  types](https://cloud.google.com/asset-inventory/docs/supported-asset-types#searchable_asset_types).
  ///  Regular expressions are also supported. For example:
  ///  * "compute.googleapis.com.*" snapshots resources whose asset type starts
  ///  with "compute.googleapis.com".
  ///  * ".*Instance" snapshots resources whose asset type ends with "Instance".
  ///  * ".*Instance.*" snapshots resources whose asset type contains
  ///  "Instance". See [RE2](https://github.com/google/re2/wiki/Syntax) for all
  ///  supported regular expression syntax. If the regular expression does not
  ///  match any supported asset type, an INVALID_ARGUMENT error will be
  ///  returned.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::ResourceSearchResult,google/cloud/asset/v1/assets.proto#L310}
  ///
  /// [google.cloud.asset.v1.SearchAllResourcesRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L831}
  /// [google.cloud.asset.v1.ResourceSearchResult]:
  /// @googleapis_reference_link{google/cloud/asset/v1/assets.proto#L310}
  ///
  StreamRange<google::cloud::asset::v1::ResourceSearchResult>
  SearchAllResources(std::string const& scope, std::string const& query,
                     std::vector<std::string> const& asset_types,
                     Options opts = {});

  ///
  /// Searches all Cloud resources within the specified scope, such as a
  /// project, folder, or organization. The caller must be granted the
  /// `cloudasset.assets.searchAllResources` permission on the desired scope,
  /// otherwise the request will be rejected.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::SearchAllResourcesRequest,google/cloud/asset/v1/asset_service.proto#L831}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::ResourceSearchResult,google/cloud/asset/v1/assets.proto#L310}
  ///
  /// [google.cloud.asset.v1.SearchAllResourcesRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L831}
  /// [google.cloud.asset.v1.ResourceSearchResult]:
  /// @googleapis_reference_link{google/cloud/asset/v1/assets.proto#L310}
  ///
  StreamRange<google::cloud::asset::v1::ResourceSearchResult>
  SearchAllResources(
      google::cloud::asset::v1::SearchAllResourcesRequest request,
      Options opts = {});

  ///
  /// Searches all IAM policies within the specified scope, such as a project,
  /// folder, or organization. The caller must be granted the
  /// `cloudasset.assets.searchAllIamPolicies` permission on the desired scope,
  /// otherwise the request will be rejected.
  ///
  /// @param scope  Required. A scope can be a project, a folder, or an
  /// organization. The search is
  ///  limited to the IAM policies within the `scope`. The caller must be
  ///  granted the
  ///  [`cloudasset.assets.searchAllIamPolicies`](https://cloud.google.com/asset-inventory/docs/access-control#required_permissions)
  ///  permission on the desired scope.
  ///  The allowed values are:
  ///  * projects/{PROJECT_ID} (e.g., "projects/foo-bar")
  ///  * projects/{PROJECT_NUMBER} (e.g., "projects/12345678")
  ///  * folders/{FOLDER_NUMBER} (e.g., "folders/1234567")
  ///  * organizations/{ORGANIZATION_NUMBER} (e.g., "organizations/123456")
  /// @param query  Optional. The query statement. See [how to construct a
  ///  query](https://cloud.google.com/asset-inventory/docs/searching-iam-policies#how_to_construct_a_query)
  ///  for more information. If not specified or empty, it will search all the
  ///  IAM policies within the specified `scope`. Note that the query string is
  ///  compared against each Cloud IAM policy binding, including its principals,
  ///  roles, and Cloud IAM conditions. The returned Cloud IAM policies will
  ///  only contain the bindings that match your query. To learn more about the
  ///  IAM policy structure, see the [IAM policy
  ///  documentation](https://cloud.google.com/iam/help/allow-policies/structure).
  ///  Examples:
  ///  * `policy:amy@gmail.com` to find IAM policy bindings that specify user
  ///    "amy@gmail.com".
  ///  * `policy:roles/compute.admin` to find IAM policy bindings that specify
  ///    the Compute Admin role.
  ///  * `policy:comp*` to find IAM policy bindings that contain "comp" as a
  ///    prefix of any word in the binding.
  ///  * `policy.role.permissions:storage.buckets.update` to find IAM policy
  ///    bindings that specify a role containing "storage.buckets.update"
  ///    permission. Note that if callers don't have `iam.roles.get` access to a
  ///    role's included permissions, policy bindings that specify this role
  ///    will be dropped from the search results.
  ///  * `policy.role.permissions:upd*` to find IAM policy bindings that specify
  ///  a
  ///    role containing "upd" as a prefix of any word in the role permission.
  ///    Note that if callers don't have `iam.roles.get` access to a role's
  ///    included permissions, policy bindings that specify this role will be
  ///    dropped from the search results.
  ///  * `resource:organizations/123456` to find IAM policy bindings
  ///    that are set on "organizations/123456".
  ///  * `resource=//cloudresourcemanager.googleapis.com/projects/myproject` to
  ///    find IAM policy bindings that are set on the project named "myproject".
  ///  * `Important` to find IAM policy bindings that contain "Important" as a
  ///    word in any of the searchable fields (except for the included
  ///    permissions).
  ///  * `resource:(instance1 OR instance2) policy:amy` to find
  ///    IAM policy bindings that are set on resources "instance1" or
  ///    "instance2" and also specify user "amy".
  ///  * `roles:roles/compute.admin` to find IAM policy bindings that specify
  ///  the
  ///    Compute Admin role.
  ///  * `memberTypes:user` to find IAM policy bindings that contain the
  ///    principal type "user".
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::IamPolicySearchResult,google/cloud/asset/v1/assets.proto#L664}
  ///
  /// [google.cloud.asset.v1.SearchAllIamPoliciesRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L999}
  /// [google.cloud.asset.v1.IamPolicySearchResult]:
  /// @googleapis_reference_link{google/cloud/asset/v1/assets.proto#L664}
  ///
  StreamRange<google::cloud::asset::v1::IamPolicySearchResult>
  SearchAllIamPolicies(std::string const& scope, std::string const& query,
                       Options opts = {});

  ///
  /// Searches all IAM policies within the specified scope, such as a project,
  /// folder, or organization. The caller must be granted the
  /// `cloudasset.assets.searchAllIamPolicies` permission on the desired scope,
  /// otherwise the request will be rejected.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::SearchAllIamPoliciesRequest,google/cloud/asset/v1/asset_service.proto#L999}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::IamPolicySearchResult,google/cloud/asset/v1/assets.proto#L664}
  ///
  /// [google.cloud.asset.v1.SearchAllIamPoliciesRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L999}
  /// [google.cloud.asset.v1.IamPolicySearchResult]:
  /// @googleapis_reference_link{google/cloud/asset/v1/assets.proto#L664}
  ///
  StreamRange<google::cloud::asset::v1::IamPolicySearchResult>
  SearchAllIamPolicies(
      google::cloud::asset::v1::SearchAllIamPoliciesRequest request,
      Options opts = {});

  ///
  /// Analyzes IAM policies to answer which identities have what accesses on
  /// which resources.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::AnalyzeIamPolicyRequest,google/cloud/asset/v1/asset_service.proto#L1297}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::AnalyzeIamPolicyResponse,google/cloud/asset/v1/asset_service.proto#L1333}
  ///
  /// [google.cloud.asset.v1.AnalyzeIamPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1297}
  /// [google.cloud.asset.v1.AnalyzeIamPolicyResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1333}
  ///
  StatusOr<google::cloud::asset::v1::AnalyzeIamPolicyResponse> AnalyzeIamPolicy(
      google::cloud::asset::v1::AnalyzeIamPolicyRequest const& request,
      Options opts = {});

  ///
  /// Analyzes IAM policies asynchronously to answer which identities have what
  /// accesses on which resources, and writes the analysis results to a Google
  /// Cloud Storage or a BigQuery destination. For Cloud Storage destination,
  /// the output format is the JSON format that represents a
  /// [AnalyzeIamPolicyResponse][google.cloud.asset.v1.AnalyzeIamPolicyResponse].
  /// This method implements the
  /// [google.longrunning.Operation][google.longrunning.Operation], which allows
  /// you to track the operation status. We recommend intervals of at least 2
  /// seconds with exponential backoff retry to poll the operation result. The
  /// metadata contains the metadata for the long-running operation.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::AnalyzeIamPolicyLongrunningRequest,google/cloud/asset/v1/asset_service.proto#L1442}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::AnalyzeIamPolicyLongrunningResponse,google/cloud/asset/v1/asset_service.proto#L1469}
  ///
  /// [google.cloud.asset.v1.AnalyzeIamPolicyLongrunningRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1442}
  /// [google.cloud.asset.v1.AnalyzeIamPolicyLongrunningResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1469}
  ///
  future<
      StatusOr<google::cloud::asset::v1::AnalyzeIamPolicyLongrunningResponse>>
  AnalyzeIamPolicyLongrunning(
      google::cloud::asset::v1::AnalyzeIamPolicyLongrunningRequest const&
          request,
      Options opts = {});

  ///
  /// Analyze moving a resource to a specified destination without kicking off
  /// the actual move. The analysis is best effort depending on the user's
  /// permissions of viewing different hierarchical policies and configurations.
  /// The policies and configuration are subject to change before the actual
  /// resource migration takes place.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::AnalyzeMoveRequest,google/cloud/asset/v1/asset_service.proto#L1645}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::AnalyzeMoveResponse,google/cloud/asset/v1/asset_service.proto#L1680}
  ///
  /// [google.cloud.asset.v1.AnalyzeMoveRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1645}
  /// [google.cloud.asset.v1.AnalyzeMoveResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1680}
  ///
  StatusOr<google::cloud::asset::v1::AnalyzeMoveResponse> AnalyzeMove(
      google::cloud::asset::v1::AnalyzeMoveRequest const& request,
      Options opts = {});

  ///
  /// Issue a job that queries assets using a SQL statement compatible with
  /// [BigQuery Standard
  /// SQL](http://cloud/bigquery/docs/reference/standard-sql/enabling-standard-sql).
  ///
  /// If the query execution finishes within timeout and there's no pagination,
  /// the full query results will be returned in the `QueryAssetsResponse`.
  ///
  /// Otherwise, full query results can be obtained by issuing extra requests
  /// with the `job_reference` from the a previous `QueryAssets` call.
  ///
  /// Note, the query result has approximately 10 GB limitation enforced by
  /// BigQuery
  /// https://cloud.google.com/bigquery/docs/best-practices-performance-output,
  /// queries return larger results will result in errors.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::QueryAssetsRequest,google/cloud/asset/v1/asset_service.proto#L1750}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::QueryAssetsResponse,google/cloud/asset/v1/asset_service.proto#L1832}
  ///
  /// [google.cloud.asset.v1.QueryAssetsRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1750}
  /// [google.cloud.asset.v1.QueryAssetsResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1832}
  ///
  StatusOr<google::cloud::asset::v1::QueryAssetsResponse> QueryAssets(
      google::cloud::asset::v1::QueryAssetsRequest const& request,
      Options opts = {});

  ///
  /// Creates a saved query in a parent project/folder/organization.
  ///
  /// @param parent  Required. The name of the project/folder/organization where
  /// this saved_query
  ///  should be created in. It can only be an organization number (such as
  ///  "organizations/123"), a folder number (such as "folders/123"), a project
  ///  ID (such as "projects/my-project-id")", or a project number (such as
  ///  "projects/12345").
  /// @param saved_query  Required. The saved_query details. The `name` field
  /// must be empty as it will be
  ///  generated based on the parent and saved_query_id.
  /// @param saved_query_id  Required. The ID to use for the saved query, which
  /// must be unique in the specified
  ///  parent. It will become the final component of the saved query's resource
  ///  name.
  ///  This value should be 4-63 characters, and valid characters
  ///  are /[a-z][0-9]-/.
  ///  Notice that this field is required in the saved query creation, and the
  ///  `name` field of the `saved_query` will be ignored.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::SavedQuery,google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  /// [google.cloud.asset.v1.CreateSavedQueryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1525}
  /// [google.cloud.asset.v1.SavedQuery]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  StatusOr<google::cloud::asset::v1::SavedQuery> CreateSavedQuery(
      std::string const& parent,
      google::cloud::asset::v1::SavedQuery const& saved_query,
      std::string const& saved_query_id, Options opts = {});

  ///
  /// Creates a saved query in a parent project/folder/organization.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::CreateSavedQueryRequest,google/cloud/asset/v1/asset_service.proto#L1525}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::SavedQuery,google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  /// [google.cloud.asset.v1.CreateSavedQueryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1525}
  /// [google.cloud.asset.v1.SavedQuery]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  StatusOr<google::cloud::asset::v1::SavedQuery> CreateSavedQuery(
      google::cloud::asset::v1::CreateSavedQueryRequest const& request,
      Options opts = {});

  ///
  /// Gets details about a saved query.
  ///
  /// @param name  Required. The name of the saved query and it must be in the
  /// format of:
  ///  * projects/project_number/savedQueries/saved_query_id
  ///  * folders/folder_number/savedQueries/saved_query_id
  ///  * organizations/organization_number/savedQueries/saved_query_id
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::SavedQuery,google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  /// [google.cloud.asset.v1.GetSavedQueryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1555}
  /// [google.cloud.asset.v1.SavedQuery]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  StatusOr<google::cloud::asset::v1::SavedQuery> GetSavedQuery(
      std::string const& name, Options opts = {});

  ///
  /// Gets details about a saved query.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::GetSavedQueryRequest,google/cloud/asset/v1/asset_service.proto#L1555}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::SavedQuery,google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  /// [google.cloud.asset.v1.GetSavedQueryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1555}
  /// [google.cloud.asset.v1.SavedQuery]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  StatusOr<google::cloud::asset::v1::SavedQuery> GetSavedQuery(
      google::cloud::asset::v1::GetSavedQueryRequest const& request,
      Options opts = {});

  ///
  /// Lists all saved queries in a parent project/folder/organization.
  ///
  /// @param parent  Required. The parent project/folder/organization whose
  /// savedQueries are to be
  ///  listed. It can only be using project/folder/organization number (such as
  ///  "folders/12345")", or a project ID (such as "projects/my-project-id").
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::SavedQuery,google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  /// [google.cloud.asset.v1.ListSavedQueriesRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1570}
  /// [google.cloud.asset.v1.SavedQuery]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  StreamRange<google::cloud::asset::v1::SavedQuery> ListSavedQueries(
      std::string const& parent, Options opts = {});

  ///
  /// Lists all saved queries in a parent project/folder/organization.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::ListSavedQueriesRequest,google/cloud/asset/v1/asset_service.proto#L1570}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::SavedQuery,google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  /// [google.cloud.asset.v1.ListSavedQueriesRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1570}
  /// [google.cloud.asset.v1.SavedQuery]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  StreamRange<google::cloud::asset::v1::SavedQuery> ListSavedQueries(
      google::cloud::asset::v1::ListSavedQueriesRequest request,
      Options opts = {});

  ///
  /// Updates a saved query.
  ///
  /// @param saved_query  Required. The saved query to update.
  ///  The saved query's `name` field is used to identify the one to update,
  ///  which has format as below:
  ///  * projects/project_number/savedQueries/saved_query_id
  ///  * folders/folder_number/savedQueries/saved_query_id
  ///  * organizations/organization_number/savedQueries/saved_query_id
  /// @param update_mask  Required. The list of fields to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::SavedQuery,google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  /// [google.cloud.asset.v1.UpdateSavedQueryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1614}
  /// [google.cloud.asset.v1.SavedQuery]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  StatusOr<google::cloud::asset::v1::SavedQuery> UpdateSavedQuery(
      google::cloud::asset::v1::SavedQuery const& saved_query,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates a saved query.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::UpdateSavedQueryRequest,google/cloud/asset/v1/asset_service.proto#L1614}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::SavedQuery,google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  /// [google.cloud.asset.v1.UpdateSavedQueryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1614}
  /// [google.cloud.asset.v1.SavedQuery]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1474}
  ///
  StatusOr<google::cloud::asset::v1::SavedQuery> UpdateSavedQuery(
      google::cloud::asset::v1::UpdateSavedQueryRequest const& request,
      Options opts = {});

  ///
  /// Deletes a saved query.
  ///
  /// @param name  Required. The name of the saved query to delete. It must be
  /// in the format of:
  ///  * projects/project_number/savedQueries/saved_query_id
  ///  * folders/folder_number/savedQueries/saved_query_id
  ///  * organizations/organization_number/savedQueries/saved_query_id
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.asset.v1.DeleteSavedQueryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1630}
  ///
  Status DeleteSavedQuery(std::string const& name, Options opts = {});

  ///
  /// Deletes a saved query.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::DeleteSavedQueryRequest,google/cloud/asset/v1/asset_service.proto#L1630}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.asset.v1.DeleteSavedQueryRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1630}
  ///
  Status DeleteSavedQuery(
      google::cloud::asset::v1::DeleteSavedQueryRequest const& request,
      Options opts = {});

  ///
  /// Gets effective IAM policies for a batch of resources.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::asset::v1::BatchGetEffectiveIamPoliciesRequest,google/cloud/asset/v1/asset_service.proto#L1916}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::asset::v1::BatchGetEffectiveIamPoliciesResponse,google/cloud/asset/v1/asset_service.proto#L1949}
  ///
  /// [google.cloud.asset.v1.BatchGetEffectiveIamPoliciesRequest]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1916}
  /// [google.cloud.asset.v1.BatchGetEffectiveIamPoliciesResponse]:
  /// @googleapis_reference_link{google/cloud/asset/v1/asset_service.proto#L1949}
  ///
  StatusOr<google::cloud::asset::v1::BatchGetEffectiveIamPoliciesResponse>
  BatchGetEffectiveIamPolicies(
      google::cloud::asset::v1::BatchGetEffectiveIamPoliciesRequest const&
          request,
      Options opts = {});

 private:
  std::shared_ptr<AssetServiceConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace asset
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_ASSET_ASSET_CLIENT_H
