// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/bigquery/migration/v2/migration_service.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_BIGQUERY_MIGRATION_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_BIGQUERY_MIGRATION_CLIENT_H

#include "google/cloud/bigquery/migration_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace bigquery {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Service to handle EDW migrations.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class MigrationServiceClient {
 public:
  explicit MigrationServiceClient(
      std::shared_ptr<MigrationServiceConnection> connection,
      Options opts = {});
  ~MigrationServiceClient();

  ///@{
  // @name Copy and move support
  MigrationServiceClient(MigrationServiceClient const&) = default;
  MigrationServiceClient& operator=(MigrationServiceClient const&) = default;
  MigrationServiceClient(MigrationServiceClient&&) = default;
  MigrationServiceClient& operator=(MigrationServiceClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(MigrationServiceClient const& a,
                         MigrationServiceClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(MigrationServiceClient const& a,
                         MigrationServiceClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Creates a migration workflow.
  ///
  /// @param parent  Required. The name of the project to which this migration
  /// workflow belongs.
  ///  Example: `projects/foo/locations/bar`
  /// @param migration_workflow  Required. The migration workflow to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationWorkflow,google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  /// [google.cloud.bigquery.migration.v2.CreateMigrationWorkflowRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L102}
  /// [google.cloud.bigquery.migration.v2.MigrationWorkflow]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  StatusOr<google::cloud::bigquery::migration::v2::MigrationWorkflow>
  CreateMigrationWorkflow(
      std::string const& parent,
      google::cloud::bigquery::migration::v2::MigrationWorkflow const&
          migration_workflow,
      Options opts = {});

  ///
  /// Creates a migration workflow.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::bigquery::migration::v2::CreateMigrationWorkflowRequest,google/cloud/bigquery/migration/v2/migration_service.proto#L102}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationWorkflow,google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  /// [google.cloud.bigquery.migration.v2.CreateMigrationWorkflowRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L102}
  /// [google.cloud.bigquery.migration.v2.MigrationWorkflow]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  StatusOr<google::cloud::bigquery::migration::v2::MigrationWorkflow>
  CreateMigrationWorkflow(google::cloud::bigquery::migration::v2::
                              CreateMigrationWorkflowRequest const& request,
                          Options opts = {});

  ///
  /// Gets a previously created migration workflow.
  ///
  /// @param name  Required. The unique identifier for the migration workflow.
  ///  Example: `projects/123/locations/us/workflows/1234`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationWorkflow,google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  /// [google.cloud.bigquery.migration.v2.GetMigrationWorkflowRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L117}
  /// [google.cloud.bigquery.migration.v2.MigrationWorkflow]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  StatusOr<google::cloud::bigquery::migration::v2::MigrationWorkflow>
  GetMigrationWorkflow(std::string const& name, Options opts = {});

  ///
  /// Gets a previously created migration workflow.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::bigquery::migration::v2::GetMigrationWorkflowRequest,google/cloud/bigquery/migration/v2/migration_service.proto#L117}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationWorkflow,google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  /// [google.cloud.bigquery.migration.v2.GetMigrationWorkflowRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L117}
  /// [google.cloud.bigquery.migration.v2.MigrationWorkflow]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  StatusOr<google::cloud::bigquery::migration::v2::MigrationWorkflow>
  GetMigrationWorkflow(
      google::cloud::bigquery::migration::v2::GetMigrationWorkflowRequest const&
          request,
      Options opts = {});

  ///
  /// Lists previously created migration workflow.
  ///
  /// @param parent  Required. The project and location of the migration
  /// workflows to list.
  ///  Example: `projects/123/locations/us`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationWorkflow,google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  /// [google.cloud.bigquery.migration.v2.ListMigrationWorkflowsRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L132}
  /// [google.cloud.bigquery.migration.v2.MigrationWorkflow]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  StreamRange<google::cloud::bigquery::migration::v2::MigrationWorkflow>
  ListMigrationWorkflows(std::string const& parent, Options opts = {});

  ///
  /// Lists previously created migration workflow.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::bigquery::migration::v2::ListMigrationWorkflowsRequest,google/cloud/bigquery/migration/v2/migration_service.proto#L132}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationWorkflow,google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  /// [google.cloud.bigquery.migration.v2.ListMigrationWorkflowsRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L132}
  /// [google.cloud.bigquery.migration.v2.MigrationWorkflow]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L36}
  ///
  StreamRange<google::cloud::bigquery::migration::v2::MigrationWorkflow>
  ListMigrationWorkflows(
      google::cloud::bigquery::migration::v2::ListMigrationWorkflowsRequest
          request,
      Options opts = {});

  ///
  /// Deletes a migration workflow by name.
  ///
  /// @param name  Required. The unique identifier for the migration workflow.
  ///  Example: `projects/123/locations/us/workflows/1234`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.bigquery.migration.v2.DeleteMigrationWorkflowRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L168}
  ///
  Status DeleteMigrationWorkflow(std::string const& name, Options opts = {});

  ///
  /// Deletes a migration workflow by name.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::bigquery::migration::v2::DeleteMigrationWorkflowRequest,google/cloud/bigquery/migration/v2/migration_service.proto#L168}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.bigquery.migration.v2.DeleteMigrationWorkflowRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L168}
  ///
  Status DeleteMigrationWorkflow(
      google::cloud::bigquery::migration::v2::
          DeleteMigrationWorkflowRequest const& request,
      Options opts = {});

  ///
  /// Starts a previously created migration workflow. I.e., the state
  /// transitions from DRAFT to RUNNING. This is a no-op if the state is already
  /// RUNNING. An error will be signaled if the state is anything other than
  /// DRAFT or RUNNING.
  ///
  /// @param name  Required. The unique identifier for the migration workflow.
  ///  Example: `projects/123/locations/us/workflows/1234`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.bigquery.migration.v2.StartMigrationWorkflowRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L180}
  ///
  Status StartMigrationWorkflow(std::string const& name, Options opts = {});

  ///
  /// Starts a previously created migration workflow. I.e., the state
  /// transitions from DRAFT to RUNNING. This is a no-op if the state is already
  /// RUNNING. An error will be signaled if the state is anything other than
  /// DRAFT or RUNNING.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::bigquery::migration::v2::StartMigrationWorkflowRequest,google/cloud/bigquery/migration/v2/migration_service.proto#L180}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.bigquery.migration.v2.StartMigrationWorkflowRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L180}
  ///
  Status StartMigrationWorkflow(
      google::cloud::bigquery::migration::v2::
          StartMigrationWorkflowRequest const& request,
      Options opts = {});

  ///
  /// Gets a previously created migration subtask.
  ///
  /// @param name  Required. The unique identifier for the migration subtask.
  ///  Example: `projects/123/locations/us/workflows/1234/subtasks/543`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationSubtask,google/cloud/bigquery/migration/v2/migration_entities.proto#L159}
  ///
  /// [google.cloud.bigquery.migration.v2.GetMigrationSubtaskRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L192}
  /// [google.cloud.bigquery.migration.v2.MigrationSubtask]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L159}
  ///
  StatusOr<google::cloud::bigquery::migration::v2::MigrationSubtask>
  GetMigrationSubtask(std::string const& name, Options opts = {});

  ///
  /// Gets a previously created migration subtask.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::bigquery::migration::v2::GetMigrationSubtaskRequest,google/cloud/bigquery/migration/v2/migration_service.proto#L192}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationSubtask,google/cloud/bigquery/migration/v2/migration_entities.proto#L159}
  ///
  /// [google.cloud.bigquery.migration.v2.GetMigrationSubtaskRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L192}
  /// [google.cloud.bigquery.migration.v2.MigrationSubtask]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L159}
  ///
  StatusOr<google::cloud::bigquery::migration::v2::MigrationSubtask>
  GetMigrationSubtask(
      google::cloud::bigquery::migration::v2::GetMigrationSubtaskRequest const&
          request,
      Options opts = {});

  ///
  /// Lists previously created migration subtasks.
  ///
  /// @param parent  Required. The migration task of the subtasks to list.
  ///  Example: `projects/123/locations/us/workflows/1234`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationSubtask,google/cloud/bigquery/migration/v2/migration_entities.proto#L159}
  ///
  /// [google.cloud.bigquery.migration.v2.ListMigrationSubtasksRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L207}
  /// [google.cloud.bigquery.migration.v2.MigrationSubtask]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L159}
  ///
  StreamRange<google::cloud::bigquery::migration::v2::MigrationSubtask>
  ListMigrationSubtasks(std::string const& parent, Options opts = {});

  ///
  /// Lists previously created migration subtasks.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::bigquery::migration::v2::ListMigrationSubtasksRequest,google/cloud/bigquery/migration/v2/migration_service.proto#L207}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::bigquery::migration::v2::MigrationSubtask,google/cloud/bigquery/migration/v2/migration_entities.proto#L159}
  ///
  /// [google.cloud.bigquery.migration.v2.ListMigrationSubtasksRequest]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_service.proto#L207}
  /// [google.cloud.bigquery.migration.v2.MigrationSubtask]:
  /// @googleapis_reference_link{google/cloud/bigquery/migration/v2/migration_entities.proto#L159}
  ///
  StreamRange<google::cloud::bigquery::migration::v2::MigrationSubtask>
  ListMigrationSubtasks(
      google::cloud::bigquery::migration::v2::ListMigrationSubtasksRequest
          request,
      Options opts = {});

 private:
  std::shared_ptr<MigrationServiceConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace bigquery
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_BIGQUERY_MIGRATION_CLIENT_H
