// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/contactcenterinsights/v1/contact_center_insights.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_CONTACTCENTERINSIGHTS_CONTACT_CENTER_INSIGHTS_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_CONTACTCENTERINSIGHTS_CONTACT_CENTER_INSIGHTS_CLIENT_H

#include "google/cloud/contactcenterinsights/contact_center_insights_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace contactcenterinsights {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// An API that lets users analyze and explore their business conversation data.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class ContactCenterInsightsClient {
 public:
  explicit ContactCenterInsightsClient(
      std::shared_ptr<ContactCenterInsightsConnection> connection,
      Options opts = {});
  ~ContactCenterInsightsClient();

  ///@{
  // @name Copy and move support
  ContactCenterInsightsClient(ContactCenterInsightsClient const&) = default;
  ContactCenterInsightsClient& operator=(ContactCenterInsightsClient const&) =
      default;
  ContactCenterInsightsClient(ContactCenterInsightsClient&&) = default;
  ContactCenterInsightsClient& operator=(ContactCenterInsightsClient&&) =
      default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(ContactCenterInsightsClient const& a,
                         ContactCenterInsightsClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(ContactCenterInsightsClient const& a,
                         ContactCenterInsightsClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Creates a conversation.
  ///
  /// @param parent  Required. The parent resource of the conversation.
  /// @param conversation  Required. The conversation resource to create.
  /// @param conversation_id  A unique ID for the new conversation. This ID will
  /// become the final
  ///  component of the conversation's resource name. If no ID is specified, a
  ///  server-generated ID will be used.
  ///  This value should be 4-64 characters and must match the regular
  ///  expression `^[a-z0-9-]{4,64}$`. Valid characters are `[a-z][0-9]-`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Conversation,google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreateConversationRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L462}
  /// [google.cloud.contactcenterinsights.v1.Conversation]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Conversation>
  CreateConversation(
      std::string const& parent,
      google::cloud::contactcenterinsights::v1::Conversation const&
          conversation,
      std::string const& conversation_id, Options opts = {});

  ///
  /// Creates a conversation.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CreateConversationRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L462}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Conversation,google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreateConversationRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L462}
  /// [google.cloud.contactcenterinsights.v1.Conversation]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Conversation>
  CreateConversation(
      google::cloud::contactcenterinsights::v1::CreateConversationRequest const&
          request,
      Options opts = {});

  ///
  /// Updates a conversation.
  ///
  /// @param conversation  Required. The new values for the conversation.
  /// @param update_mask  The list of fields to be updated.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Conversation,google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateConversationRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L538}
  /// [google.cloud.contactcenterinsights.v1.Conversation]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Conversation>
  UpdateConversation(
      google::cloud::contactcenterinsights::v1::Conversation const&
          conversation,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates a conversation.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UpdateConversationRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L538}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Conversation,google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateConversationRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L538}
  /// [google.cloud.contactcenterinsights.v1.Conversation]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Conversation>
  UpdateConversation(
      google::cloud::contactcenterinsights::v1::UpdateConversationRequest const&
          request,
      Options opts = {});

  ///
  /// Gets a conversation.
  ///
  /// @param name  Required. The name of the conversation to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Conversation,google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetConversationRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L524}
  /// [google.cloud.contactcenterinsights.v1.Conversation]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Conversation>
  GetConversation(std::string const& name, Options opts = {});

  ///
  /// Gets a conversation.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::GetConversationRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L524}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Conversation,google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetConversationRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L524}
  /// [google.cloud.contactcenterinsights.v1.Conversation]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Conversation>
  GetConversation(
      google::cloud::contactcenterinsights::v1::GetConversationRequest const&
          request,
      Options opts = {});

  ///
  /// Lists conversations.
  ///
  /// @param parent  Required. The parent resource of the conversation.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Conversation,google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListConversationsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L484}
  /// [google.cloud.contactcenterinsights.v1.Conversation]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  StreamRange<google::cloud::contactcenterinsights::v1::Conversation>
  ListConversations(std::string const& parent, Options opts = {});

  ///
  /// Lists conversations.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListConversationsRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L484}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Conversation,google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListConversationsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L484}
  /// [google.cloud.contactcenterinsights.v1.Conversation]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L38}
  ///
  StreamRange<google::cloud::contactcenterinsights::v1::Conversation>
  ListConversations(
      google::cloud::contactcenterinsights::v1::ListConversationsRequest
          request,
      Options opts = {});

  ///
  /// Deletes a conversation.
  ///
  /// @param name  Required. The name of the conversation to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.contactcenterinsights.v1.DeleteConversationRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L547}
  ///
  Status DeleteConversation(std::string const& name, Options opts = {});

  ///
  /// Deletes a conversation.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeleteConversationRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L547}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.contactcenterinsights.v1.DeleteConversationRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L547}
  ///
  Status DeleteConversation(
      google::cloud::contactcenterinsights::v1::DeleteConversationRequest const&
          request,
      Options opts = {});

  ///
  /// Creates an analysis. The long running operation is done when the analysis
  /// has completed.
  ///
  /// @param parent  Required. The parent resource of the analysis.
  /// @param analysis  Required. The analysis to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Analysis,google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreateAnalysisRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L563}
  /// [google.cloud.contactcenterinsights.v1.Analysis]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  future<StatusOr<google::cloud::contactcenterinsights::v1::Analysis>>
  CreateAnalysis(
      std::string const& parent,
      google::cloud::contactcenterinsights::v1::Analysis const& analysis,
      Options opts = {});

  ///
  /// Creates an analysis. The long running operation is done when the analysis
  /// has completed.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CreateAnalysisRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L563}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Analysis,google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreateAnalysisRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L563}
  /// [google.cloud.contactcenterinsights.v1.Analysis]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  future<StatusOr<google::cloud::contactcenterinsights::v1::Analysis>>
  CreateAnalysis(
      google::cloud::contactcenterinsights::v1::CreateAnalysisRequest const&
          request,
      Options opts = {});

  ///
  /// Gets an analysis.
  ///
  /// @param name  Required. The name of the analysis to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Analysis,google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetAnalysisRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L613}
  /// [google.cloud.contactcenterinsights.v1.Analysis]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Analysis> GetAnalysis(
      std::string const& name, Options opts = {});

  ///
  /// Gets an analysis.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::GetAnalysisRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L613}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Analysis,google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetAnalysisRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L613}
  /// [google.cloud.contactcenterinsights.v1.Analysis]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Analysis> GetAnalysis(
      google::cloud::contactcenterinsights::v1::GetAnalysisRequest const&
          request,
      Options opts = {});

  ///
  /// Lists analyses.
  ///
  /// @param parent  Required. The parent resource of the analyses.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Analysis,google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListAnalysesRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L577}
  /// [google.cloud.contactcenterinsights.v1.Analysis]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  StreamRange<google::cloud::contactcenterinsights::v1::Analysis> ListAnalyses(
      std::string const& parent, Options opts = {});

  ///
  /// Lists analyses.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListAnalysesRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L577}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Analysis,google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListAnalysesRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L577}
  /// [google.cloud.contactcenterinsights.v1.Analysis]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L209}
  ///
  StreamRange<google::cloud::contactcenterinsights::v1::Analysis> ListAnalyses(
      google::cloud::contactcenterinsights::v1::ListAnalysesRequest request,
      Options opts = {});

  ///
  /// Deletes an analysis.
  ///
  /// @param name  Required. The name of the analysis to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.contactcenterinsights.v1.DeleteAnalysisRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L624}
  ///
  Status DeleteAnalysis(std::string const& name, Options opts = {});

  ///
  /// Deletes an analysis.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeleteAnalysisRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L624}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.contactcenterinsights.v1.DeleteAnalysisRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L624}
  ///
  Status DeleteAnalysis(
      google::cloud::contactcenterinsights::v1::DeleteAnalysisRequest const&
          request,
      Options opts = {});

  ///
  /// Export insights data to a destination defined in the request body.
  ///
  /// @param parent  Required. The parent resource to export data from.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ExportInsightsDataResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L711}
  ///
  /// [google.cloud.contactcenterinsights.v1.ExportInsightsDataRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L635}
  /// [google.cloud.contactcenterinsights.v1.ExportInsightsDataResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L711}
  ///
  future<StatusOr<
      google::cloud::contactcenterinsights::v1::ExportInsightsDataResponse>>
  ExportInsightsData(std::string const& parent, Options opts = {});

  ///
  /// Export insights data to a destination defined in the request body.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ExportInsightsDataRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L635}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ExportInsightsDataResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L711}
  ///
  /// [google.cloud.contactcenterinsights.v1.ExportInsightsDataRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L635}
  /// [google.cloud.contactcenterinsights.v1.ExportInsightsDataResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L711}
  ///
  future<StatusOr<
      google::cloud::contactcenterinsights::v1::ExportInsightsDataResponse>>
  ExportInsightsData(
      google::cloud::contactcenterinsights::v1::ExportInsightsDataRequest const&
          request,
      Options opts = {});

  ///
  /// Creates an issue model.
  ///
  /// @param parent  Required. The parent resource of the issue model.
  /// @param issue_model  Required. The issue model to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::IssueModel,google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreateIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L716}
  /// [google.cloud.contactcenterinsights.v1.IssueModel]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  future<StatusOr<google::cloud::contactcenterinsights::v1::IssueModel>>
  CreateIssueModel(
      std::string const& parent,
      google::cloud::contactcenterinsights::v1::IssueModel const& issue_model,
      Options opts = {});

  ///
  /// Creates an issue model.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CreateIssueModelRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L716}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::IssueModel,google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreateIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L716}
  /// [google.cloud.contactcenterinsights.v1.IssueModel]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  future<StatusOr<google::cloud::contactcenterinsights::v1::IssueModel>>
  CreateIssueModel(
      google::cloud::contactcenterinsights::v1::CreateIssueModelRequest const&
          request,
      Options opts = {});

  ///
  /// Updates an issue model.
  ///
  /// @param issue_model  Required. The new values for the issue model.
  /// @param update_mask  The list of fields to be updated.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::IssueModel,google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L742}
  /// [google.cloud.contactcenterinsights.v1.IssueModel]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::IssueModel>
  UpdateIssueModel(
      google::cloud::contactcenterinsights::v1::IssueModel const& issue_model,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates an issue model.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UpdateIssueModelRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L742}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::IssueModel,google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L742}
  /// [google.cloud.contactcenterinsights.v1.IssueModel]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::IssueModel>
  UpdateIssueModel(
      google::cloud::contactcenterinsights::v1::UpdateIssueModelRequest const&
          request,
      Options opts = {});

  ///
  /// Gets an issue model.
  ///
  /// @param name  Required. The name of the issue model to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::IssueModel,google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L768}
  /// [google.cloud.contactcenterinsights.v1.IssueModel]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::IssueModel> GetIssueModel(
      std::string const& name, Options opts = {});

  ///
  /// Gets an issue model.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::GetIssueModelRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L768}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::IssueModel,google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L768}
  /// [google.cloud.contactcenterinsights.v1.IssueModel]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L583}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::IssueModel> GetIssueModel(
      google::cloud::contactcenterinsights::v1::GetIssueModelRequest const&
          request,
      Options opts = {});

  ///
  /// Lists issue models.
  ///
  /// @param parent  Required. The parent resource of the issue model.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListIssueModelsResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L762}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListIssueModelsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L751}
  /// [google.cloud.contactcenterinsights.v1.ListIssueModelsResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L762}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::ListIssueModelsResponse>
  ListIssueModels(std::string const& parent, Options opts = {});

  ///
  /// Lists issue models.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListIssueModelsRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L751}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListIssueModelsResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L762}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListIssueModelsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L751}
  /// [google.cloud.contactcenterinsights.v1.ListIssueModelsResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L762}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::ListIssueModelsResponse>
  ListIssueModels(
      google::cloud::contactcenterinsights::v1::ListIssueModelsRequest const&
          request,
      Options opts = {});

  ///
  /// Deletes an issue model.
  ///
  /// @param name  Required. The name of the issue model to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeleteIssueModelMetadata,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L790}
  ///
  /// [google.cloud.contactcenterinsights.v1.DeleteIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L779}
  /// [google.cloud.contactcenterinsights.v1.DeleteIssueModelMetadata]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L790}
  ///
  future<StatusOr<
      google::cloud::contactcenterinsights::v1::DeleteIssueModelMetadata>>
  DeleteIssueModel(std::string const& name, Options opts = {});

  ///
  /// Deletes an issue model.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeleteIssueModelRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L779}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeleteIssueModelMetadata,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L790}
  ///
  /// [google.cloud.contactcenterinsights.v1.DeleteIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L779}
  /// [google.cloud.contactcenterinsights.v1.DeleteIssueModelMetadata]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L790}
  ///
  future<StatusOr<
      google::cloud::contactcenterinsights::v1::DeleteIssueModelMetadata>>
  DeleteIssueModel(
      google::cloud::contactcenterinsights::v1::DeleteIssueModelRequest const&
          request,
      Options opts = {});

  ///
  /// Deploys an issue model. Returns an error if a model is already deployed.
  /// An issue model can only be used in analysis after it has been deployed.
  ///
  /// @param name  Required. The issue model to deploy.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeployIssueModelResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L813}
  ///
  /// [google.cloud.contactcenterinsights.v1.DeployIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L802}
  /// [google.cloud.contactcenterinsights.v1.DeployIssueModelResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L813}
  ///
  future<StatusOr<
      google::cloud::contactcenterinsights::v1::DeployIssueModelResponse>>
  DeployIssueModel(std::string const& name, Options opts = {});

  ///
  /// Deploys an issue model. Returns an error if a model is already deployed.
  /// An issue model can only be used in analysis after it has been deployed.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeployIssueModelRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L802}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeployIssueModelResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L813}
  ///
  /// [google.cloud.contactcenterinsights.v1.DeployIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L802}
  /// [google.cloud.contactcenterinsights.v1.DeployIssueModelResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L813}
  ///
  future<StatusOr<
      google::cloud::contactcenterinsights::v1::DeployIssueModelResponse>>
  DeployIssueModel(
      google::cloud::contactcenterinsights::v1::DeployIssueModelRequest const&
          request,
      Options opts = {});

  ///
  /// Undeploys an issue model.
  /// An issue model can not be used in analysis after it has been undeployed.
  ///
  /// @param name  Required. The issue model to undeploy.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UndeployIssueModelResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L841}
  ///
  /// [google.cloud.contactcenterinsights.v1.UndeployIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L830}
  /// [google.cloud.contactcenterinsights.v1.UndeployIssueModelResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L841}
  ///
  future<StatusOr<
      google::cloud::contactcenterinsights::v1::UndeployIssueModelResponse>>
  UndeployIssueModel(std::string const& name, Options opts = {});

  ///
  /// Undeploys an issue model.
  /// An issue model can not be used in analysis after it has been undeployed.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UndeployIssueModelRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L830}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UndeployIssueModelResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L841}
  ///
  /// [google.cloud.contactcenterinsights.v1.UndeployIssueModelRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L830}
  /// [google.cloud.contactcenterinsights.v1.UndeployIssueModelResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L841}
  ///
  future<StatusOr<
      google::cloud::contactcenterinsights::v1::UndeployIssueModelResponse>>
  UndeployIssueModel(
      google::cloud::contactcenterinsights::v1::UndeployIssueModelRequest const&
          request,
      Options opts = {});

  ///
  /// Gets an issue.
  ///
  /// @param name  Required. The name of the issue to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Issue,google/cloud/contactcenterinsights/v1/resources.proto#L654}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetIssueRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L858}
  /// [google.cloud.contactcenterinsights.v1.Issue]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L654}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Issue> GetIssue(
      std::string const& name, Options opts = {});

  ///
  /// Gets an issue.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::GetIssueRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L858}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Issue,google/cloud/contactcenterinsights/v1/resources.proto#L654}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetIssueRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L858}
  /// [google.cloud.contactcenterinsights.v1.Issue]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L654}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Issue> GetIssue(
      google::cloud::contactcenterinsights::v1::GetIssueRequest const& request,
      Options opts = {});

  ///
  /// Lists issues.
  ///
  /// @param parent  Required. The parent resource of the issue.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListIssuesResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L880}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListIssuesRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L869}
  /// [google.cloud.contactcenterinsights.v1.ListIssuesResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L880}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::ListIssuesResponse>
  ListIssues(std::string const& parent, Options opts = {});

  ///
  /// Lists issues.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListIssuesRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L869}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListIssuesResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L880}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListIssuesRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L869}
  /// [google.cloud.contactcenterinsights.v1.ListIssuesResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L880}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::ListIssuesResponse>
  ListIssues(google::cloud::contactcenterinsights::v1::ListIssuesRequest const&
                 request,
             Options opts = {});

  ///
  /// Updates an issue.
  ///
  /// @param issue  Required. The new values for the issue.
  /// @param update_mask  The list of fields to be updated.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Issue,google/cloud/contactcenterinsights/v1/resources.proto#L654}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateIssueRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L886}
  /// [google.cloud.contactcenterinsights.v1.Issue]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L654}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Issue> UpdateIssue(
      google::cloud::contactcenterinsights::v1::Issue const& issue,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates an issue.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UpdateIssueRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L886}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Issue,google/cloud/contactcenterinsights/v1/resources.proto#L654}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateIssueRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L886}
  /// [google.cloud.contactcenterinsights.v1.Issue]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L654}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Issue> UpdateIssue(
      google::cloud::contactcenterinsights::v1::UpdateIssueRequest const&
          request,
      Options opts = {});

  ///
  /// Gets an issue model's statistics.
  ///
  /// @param issue_model  Required. The resource name of the issue model to
  /// query against.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CalculateIssueModelStatsResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L906}
  ///
  /// [google.cloud.contactcenterinsights.v1.CalculateIssueModelStatsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L895}
  /// [google.cloud.contactcenterinsights.v1.CalculateIssueModelStatsResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L906}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::
               CalculateIssueModelStatsResponse>
  CalculateIssueModelStats(std::string const& issue_model, Options opts = {});

  ///
  /// Gets an issue model's statistics.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CalculateIssueModelStatsRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L895}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CalculateIssueModelStatsResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L906}
  ///
  /// [google.cloud.contactcenterinsights.v1.CalculateIssueModelStatsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L895}
  /// [google.cloud.contactcenterinsights.v1.CalculateIssueModelStatsResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L906}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::
               CalculateIssueModelStatsResponse>
  CalculateIssueModelStats(google::cloud::contactcenterinsights::v1::
                               CalculateIssueModelStatsRequest const& request,
                           Options opts = {});

  ///
  /// Creates a phrase matcher.
  ///
  /// @param parent  Required. The parent resource of the phrase matcher.
  /// Required. The location to create
  ///  a phrase matcher for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>` or
  ///  `projects/<Project Number>/locations/<Location ID>`
  /// @param phrase_matcher  Required. The phrase matcher resource to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::PhraseMatcher,google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreatePhraseMatcherRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L913}
  /// [google.cloud.contactcenterinsights.v1.PhraseMatcher]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::PhraseMatcher>
  CreatePhraseMatcher(
      std::string const& parent,
      google::cloud::contactcenterinsights::v1::PhraseMatcher const&
          phrase_matcher,
      Options opts = {});

  ///
  /// Creates a phrase matcher.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CreatePhraseMatcherRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L913}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::PhraseMatcher,google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreatePhraseMatcherRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L913}
  /// [google.cloud.contactcenterinsights.v1.PhraseMatcher]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::PhraseMatcher>
  CreatePhraseMatcher(google::cloud::contactcenterinsights::v1::
                          CreatePhraseMatcherRequest const& request,
                      Options opts = {});

  ///
  /// Gets a phrase matcher.
  ///
  /// @param name  Required. The name of the phrase matcher to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::PhraseMatcher,google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetPhraseMatcherRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L966}
  /// [google.cloud.contactcenterinsights.v1.PhraseMatcher]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::PhraseMatcher>
  GetPhraseMatcher(std::string const& name, Options opts = {});

  ///
  /// Gets a phrase matcher.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::GetPhraseMatcherRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L966}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::PhraseMatcher,google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetPhraseMatcherRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L966}
  /// [google.cloud.contactcenterinsights.v1.PhraseMatcher]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::PhraseMatcher>
  GetPhraseMatcher(
      google::cloud::contactcenterinsights::v1::GetPhraseMatcherRequest const&
          request,
      Options opts = {});

  ///
  /// Lists phrase matchers.
  ///
  /// @param parent  Required. The parent resource of the phrase matcher.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::PhraseMatcher,google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListPhraseMatchersRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L930}
  /// [google.cloud.contactcenterinsights.v1.PhraseMatcher]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  StreamRange<google::cloud::contactcenterinsights::v1::PhraseMatcher>
  ListPhraseMatchers(std::string const& parent, Options opts = {});

  ///
  /// Lists phrase matchers.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListPhraseMatchersRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L930}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::PhraseMatcher,google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListPhraseMatchersRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L930}
  /// [google.cloud.contactcenterinsights.v1.PhraseMatcher]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  StreamRange<google::cloud::contactcenterinsights::v1::PhraseMatcher>
  ListPhraseMatchers(
      google::cloud::contactcenterinsights::v1::ListPhraseMatchersRequest
          request,
      Options opts = {});

  ///
  /// Deletes a phrase matcher.
  ///
  /// @param name  Required. The name of the phrase matcher to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.contactcenterinsights.v1.DeletePhraseMatcherRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L977}
  ///
  Status DeletePhraseMatcher(std::string const& name, Options opts = {});

  ///
  /// Deletes a phrase matcher.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeletePhraseMatcherRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L977}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.contactcenterinsights.v1.DeletePhraseMatcherRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L977}
  ///
  Status DeletePhraseMatcher(google::cloud::contactcenterinsights::v1::
                                 DeletePhraseMatcherRequest const& request,
                             Options opts = {});

  ///
  /// Updates a phrase matcher.
  ///
  /// @param phrase_matcher  Required. The new values for the phrase matcher.
  /// @param update_mask  The list of fields to be updated.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::PhraseMatcher,google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdatePhraseMatcherRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L988}
  /// [google.cloud.contactcenterinsights.v1.PhraseMatcher]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::PhraseMatcher>
  UpdatePhraseMatcher(
      google::cloud::contactcenterinsights::v1::PhraseMatcher const&
          phrase_matcher,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates a phrase matcher.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UpdatePhraseMatcherRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L988}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::PhraseMatcher,google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdatePhraseMatcherRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L988}
  /// [google.cloud.contactcenterinsights.v1.PhraseMatcher]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L703}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::PhraseMatcher>
  UpdatePhraseMatcher(google::cloud::contactcenterinsights::v1::
                          UpdatePhraseMatcherRequest const& request,
                      Options opts = {});

  ///
  /// Gets conversation statistics.
  ///
  /// @param location  Required. The location of the conversations.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CalculateStatsResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L389}
  ///
  /// [google.cloud.contactcenterinsights.v1.CalculateStatsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L374}
  /// [google.cloud.contactcenterinsights.v1.CalculateStatsResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L389}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::CalculateStatsResponse>
  CalculateStats(std::string const& location, Options opts = {});

  ///
  /// Gets conversation statistics.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CalculateStatsRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L374}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CalculateStatsResponse,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L389}
  ///
  /// [google.cloud.contactcenterinsights.v1.CalculateStatsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L374}
  /// [google.cloud.contactcenterinsights.v1.CalculateStatsResponse]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L389}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::CalculateStatsResponse>
  CalculateStats(
      google::cloud::contactcenterinsights::v1::CalculateStatsRequest const&
          request,
      Options opts = {});

  ///
  /// Gets project-level settings.
  ///
  /// @param name  Required. The name of the settings resource to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Settings,google/cloud/contactcenterinsights/v1/resources.proto#L820}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetSettingsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L997}
  /// [google.cloud.contactcenterinsights.v1.Settings]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L820}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Settings> GetSettings(
      std::string const& name, Options opts = {});

  ///
  /// Gets project-level settings.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::GetSettingsRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L997}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Settings,google/cloud/contactcenterinsights/v1/resources.proto#L820}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetSettingsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L997}
  /// [google.cloud.contactcenterinsights.v1.Settings]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L820}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Settings> GetSettings(
      google::cloud::contactcenterinsights::v1::GetSettingsRequest const&
          request,
      Options opts = {});

  ///
  /// Updates project-level settings.
  ///
  /// @param settings  Required. The new settings values.
  /// @param update_mask  Required. The list of fields to be updated.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Settings,google/cloud/contactcenterinsights/v1/resources.proto#L820}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateSettingsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1008}
  /// [google.cloud.contactcenterinsights.v1.Settings]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L820}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Settings> UpdateSettings(
      google::cloud::contactcenterinsights::v1::Settings const& settings,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates project-level settings.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UpdateSettingsRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1008}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::Settings,google/cloud/contactcenterinsights/v1/resources.proto#L820}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateSettingsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1008}
  /// [google.cloud.contactcenterinsights.v1.Settings]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L820}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::Settings> UpdateSettings(
      google::cloud::contactcenterinsights::v1::UpdateSettingsRequest const&
          request,
      Options opts = {});

  ///
  /// Creates a view.
  ///
  /// @param parent  Required. The parent resource of the view. Required. The
  /// location to create
  ///  a view for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>` or
  ///  `projects/<Project Number>/locations/<Location ID>`
  /// @param view  Required. The view resource to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::View,google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreateViewRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1017}
  /// [google.cloud.contactcenterinsights.v1.View]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::View> CreateView(
      std::string const& parent,
      google::cloud::contactcenterinsights::v1::View const& view,
      Options opts = {});

  ///
  /// Creates a view.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::CreateViewRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1017}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::View,google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  /// [google.cloud.contactcenterinsights.v1.CreateViewRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1017}
  /// [google.cloud.contactcenterinsights.v1.View]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::View> CreateView(
      google::cloud::contactcenterinsights::v1::CreateViewRequest const&
          request,
      Options opts = {});

  ///
  /// Gets a view.
  ///
  /// @param name  Required. The name of the view to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::View,google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetViewRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1034}
  /// [google.cloud.contactcenterinsights.v1.View]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::View> GetView(
      std::string const& name, Options opts = {});

  ///
  /// Gets a view.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::GetViewRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1034}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::View,google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  /// [google.cloud.contactcenterinsights.v1.GetViewRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1034}
  /// [google.cloud.contactcenterinsights.v1.View]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::View> GetView(
      google::cloud::contactcenterinsights::v1::GetViewRequest const& request,
      Options opts = {});

  ///
  /// Lists views.
  ///
  /// @param parent  Required. The parent resource of the views.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::View,google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListViewsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1045}
  /// [google.cloud.contactcenterinsights.v1.View]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  StreamRange<google::cloud::contactcenterinsights::v1::View> ListViews(
      std::string const& parent, Options opts = {});

  ///
  /// Lists views.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::ListViewsRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1045}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::View,google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  /// [google.cloud.contactcenterinsights.v1.ListViewsRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1045}
  /// [google.cloud.contactcenterinsights.v1.View]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  StreamRange<google::cloud::contactcenterinsights::v1::View> ListViews(
      google::cloud::contactcenterinsights::v1::ListViewsRequest request,
      Options opts = {});

  ///
  /// Updates a view.
  ///
  /// @param view  Required. The new view.
  /// @param update_mask  The list of fields to be updated.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::View,google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateViewRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1077}
  /// [google.cloud.contactcenterinsights.v1.View]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::View> UpdateView(
      google::cloud::contactcenterinsights::v1::View const& view,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates a view.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::UpdateViewRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1077}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::View,google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  /// [google.cloud.contactcenterinsights.v1.UpdateViewRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1077}
  /// [google.cloud.contactcenterinsights.v1.View]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/resources.proto#L1094}
  ///
  StatusOr<google::cloud::contactcenterinsights::v1::View> UpdateView(
      google::cloud::contactcenterinsights::v1::UpdateViewRequest const&
          request,
      Options opts = {});

  ///
  /// Deletes a view.
  ///
  /// @param name  Required. The name of the view to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.contactcenterinsights.v1.DeleteViewRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1086}
  ///
  Status DeleteView(std::string const& name, Options opts = {});

  ///
  /// Deletes a view.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::contactcenterinsights::v1::DeleteViewRequest,google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1086}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.contactcenterinsights.v1.DeleteViewRequest]:
  /// @googleapis_reference_link{google/cloud/contactcenterinsights/v1/contact_center_insights.proto#L1086}
  ///
  Status DeleteView(
      google::cloud::contactcenterinsights::v1::DeleteViewRequest const&
          request,
      Options opts = {});

 private:
  std::shared_ptr<ContactCenterInsightsConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace contactcenterinsights
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_CONTACTCENTERINSIGHTS_CONTACT_CENTER_INSIGHTS_CLIENT_H
