// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/datacatalog/v1/datacatalog.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATACATALOG_DATA_CATALOG_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATACATALOG_DATA_CATALOG_CLIENT_H

#include "google/cloud/datacatalog/data_catalog_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/iam_updater.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace datacatalog {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Data Catalog API service allows you to discover, understand, and manage
/// your data.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class DataCatalogClient {
 public:
  explicit DataCatalogClient(std::shared_ptr<DataCatalogConnection> connection,
                             Options opts = {});
  ~DataCatalogClient();

  ///@{
  // @name Copy and move support
  DataCatalogClient(DataCatalogClient const&) = default;
  DataCatalogClient& operator=(DataCatalogClient const&) = default;
  DataCatalogClient(DataCatalogClient&&) = default;
  DataCatalogClient& operator=(DataCatalogClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(DataCatalogClient const& a,
                         DataCatalogClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(DataCatalogClient const& a,
                         DataCatalogClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Searches Data Catalog for multiple resources like entries and tags that
  /// match a query.
  ///
  /// This is a [Custom Method]
  /// (https://cloud.google.com/apis/design/custom_methods) that doesn't return
  /// all information on a resource, only its ID and high level fields. To get
  /// more information, you can subsequently call specific get methods.
  ///
  /// Note: Data Catalog search queries don't guarantee full recall. Results
  /// that match your query might not be returned, even in subsequent
  /// result pages. Additionally, returned (and not returned) results can vary
  /// if you repeat search queries.
  ///
  /// For more information, see [Data Catalog search syntax]
  /// (https://cloud.google.com/data-catalog/docs/how-to/search-reference).
  ///
  /// @param scope  Required. The scope of this search request.
  ///  The `scope` is invalid if `include_org_ids`, `include_project_ids` are
  ///  empty AND `include_gcp_public_datasets` is set to `false`. In this case,
  ///  the request returns an error.
  /// @param query  Optional. The query string with a minimum of 3 characters
  /// and specific syntax.
  ///  For more information, see
  ///  [Data Catalog search
  ///  syntax](https://cloud.google.com/data-catalog/docs/how-to/search-reference).
  ///  An empty query string returns all data assets (in the specified scope)
  ///  that you have access to.
  ///  A query string can be a simple `xyz` or qualified by predicates:
  ///  * `name:x`
  ///  * `column:y`
  ///  * `description:z`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::SearchCatalogResult,google/cloud/datacatalog/v1/search.proto#L34}
  ///
  /// [google.cloud.datacatalog.v1.SearchCatalogRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L563}
  /// [google.cloud.datacatalog.v1.SearchCatalogResult]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/search.proto#L34}
  ///
  StreamRange<google::cloud::datacatalog::v1::SearchCatalogResult>
  SearchCatalog(
      google::cloud::datacatalog::v1::SearchCatalogRequest::Scope const& scope,
      std::string const& query, Options opts = {});

  ///
  /// Searches Data Catalog for multiple resources like entries and tags that
  /// match a query.
  ///
  /// This is a [Custom Method]
  /// (https://cloud.google.com/apis/design/custom_methods) that doesn't return
  /// all information on a resource, only its ID and high level fields. To get
  /// more information, you can subsequently call specific get methods.
  ///
  /// Note: Data Catalog search queries don't guarantee full recall. Results
  /// that match your query might not be returned, even in subsequent
  /// result pages. Additionally, returned (and not returned) results can vary
  /// if you repeat search queries.
  ///
  /// For more information, see [Data Catalog search syntax]
  /// (https://cloud.google.com/data-catalog/docs/how-to/search-reference).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::SearchCatalogRequest,google/cloud/datacatalog/v1/datacatalog.proto#L563}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::SearchCatalogResult,google/cloud/datacatalog/v1/search.proto#L34}
  ///
  /// [google.cloud.datacatalog.v1.SearchCatalogRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L563}
  /// [google.cloud.datacatalog.v1.SearchCatalogResult]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/search.proto#L34}
  ///
  StreamRange<google::cloud::datacatalog::v1::SearchCatalogResult>
  SearchCatalog(google::cloud::datacatalog::v1::SearchCatalogRequest request,
                Options opts = {});

  ///
  /// Creates an entry group.
  ///
  /// An entry group contains logically related entries together with [Cloud
  /// Identity and Access Management](/data-catalog/docs/concepts/iam) policies.
  /// These policies specify users who can create, edit, and view entries
  /// within entry groups.
  ///
  /// Data Catalog automatically creates entry groups with names that start with
  /// the `@` symbol for the following resources:
  ///
  /// * BigQuery entries (`@bigquery`)
  /// * Pub/Sub topics (`@pubsub`)
  /// * Dataproc Metastore services (`@dataproc_metastore_{SERVICE_NAME_HASH}`)
  ///
  /// You can create your own entry groups for Cloud Storage fileset entries
  /// and custom entries together with the corresponding IAM policies.
  /// User-created entry groups can't contain the `@` symbol, it is reserved
  /// for automatically created groups.
  ///
  /// Entry groups, like entries, can be searched.
  ///
  /// A maximum of 10,000 entry groups may be created per organization across
  /// all locations.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `parent` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param parent  Required. The names of the project and location that the
  /// new entry group belongs to.
  ///  Note: The entry group itself and its child resources might not be
  ///  stored in the location specified in its name.
  /// @param entry_group_id  Required. The ID of the entry group to create.
  ///  The ID must contain only letters (a-z, A-Z), numbers (0-9),
  ///  underscores (_), and must start with a letter or underscore.
  ///  The maximum size is 64 bytes when encoded in UTF-8.
  /// @param entry_group  The entry group to create. Defaults to empty.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.CreateEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L682}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryGroup> CreateEntryGroup(
      std::string const& parent, std::string const& entry_group_id,
      google::cloud::datacatalog::v1::EntryGroup const& entry_group,
      Options opts = {});

  ///
  /// Creates an entry group.
  ///
  /// An entry group contains logically related entries together with [Cloud
  /// Identity and Access Management](/data-catalog/docs/concepts/iam) policies.
  /// These policies specify users who can create, edit, and view entries
  /// within entry groups.
  ///
  /// Data Catalog automatically creates entry groups with names that start with
  /// the `@` symbol for the following resources:
  ///
  /// * BigQuery entries (`@bigquery`)
  /// * Pub/Sub topics (`@pubsub`)
  /// * Dataproc Metastore services (`@dataproc_metastore_{SERVICE_NAME_HASH}`)
  ///
  /// You can create your own entry groups for Cloud Storage fileset entries
  /// and custom entries together with the corresponding IAM policies.
  /// User-created entry groups can't contain the `@` symbol, it is reserved
  /// for automatically created groups.
  ///
  /// Entry groups, like entries, can be searched.
  ///
  /// A maximum of 10,000 entry groups may be created per organization across
  /// all locations.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `parent` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::CreateEntryGroupRequest,google/cloud/datacatalog/v1/datacatalog.proto#L682}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.CreateEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L682}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryGroup> CreateEntryGroup(
      google::cloud::datacatalog::v1::CreateEntryGroupRequest const& request,
      Options opts = {});

  ///
  /// Gets an entry group.
  ///
  /// @param name  Required. The name of the entry group to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.GetEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L721}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryGroup> GetEntryGroup(
      std::string const& name, Options opts = {});

  ///
  /// Gets an entry group.
  ///
  /// @param name  Required. The name of the entry group to get.
  /// @param read_mask  The fields to return. If empty or omitted, all fields
  /// are returned.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.GetEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L721}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryGroup> GetEntryGroup(
      std::string const& name, google::protobuf::FieldMask const& read_mask,
      Options opts = {});

  ///
  /// Gets an entry group.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::GetEntryGroupRequest,google/cloud/datacatalog/v1/datacatalog.proto#L721}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.GetEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L721}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryGroup> GetEntryGroup(
      google::cloud::datacatalog::v1::GetEntryGroupRequest const& request,
      Options opts = {});

  ///
  /// Updates an entry group.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `entry_group.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param entry_group  Required. Updates for the entry group. The `name`
  /// field must be set.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.UpdateEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L707}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryGroup> UpdateEntryGroup(
      google::cloud::datacatalog::v1::EntryGroup const& entry_group,
      Options opts = {});

  ///
  /// Updates an entry group.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `entry_group.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param entry_group  Required. Updates for the entry group. The `name`
  /// field must be set.
  /// @param update_mask  Names of fields whose values to overwrite on an entry
  /// group.
  ///  If this parameter is absent or empty, all modifiable fields
  ///  are overwritten. If such fields are non-required and omitted in the
  ///  request body, their values are emptied.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.UpdateEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L707}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryGroup> UpdateEntryGroup(
      google::cloud::datacatalog::v1::EntryGroup const& entry_group,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates an entry group.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `entry_group.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::UpdateEntryGroupRequest,google/cloud/datacatalog/v1/datacatalog.proto#L707}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.UpdateEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L707}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryGroup> UpdateEntryGroup(
      google::cloud::datacatalog::v1::UpdateEntryGroupRequest const& request,
      Options opts = {});

  ///
  /// Deletes an entry group.
  ///
  /// You must enable the Data Catalog API in the project
  /// identified by the `name` parameter. For more information, see [Data
  /// Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param name  Required. The name of the entry group to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L736}
  ///
  Status DeleteEntryGroup(std::string const& name, Options opts = {});

  ///
  /// Deletes an entry group.
  ///
  /// You must enable the Data Catalog API in the project
  /// identified by the `name` parameter. For more information, see [Data
  /// Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::DeleteEntryGroupRequest,google/cloud/datacatalog/v1/datacatalog.proto#L736}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteEntryGroupRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L736}
  ///
  Status DeleteEntryGroup(
      google::cloud::datacatalog::v1::DeleteEntryGroupRequest const& request,
      Options opts = {});

  ///
  /// Lists entry groups.
  ///
  /// @param parent  Required. The name of the location that contains the entry
  /// groups to list.
  ///  Can be provided as a URL.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.ListEntryGroupsRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L751}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StreamRange<google::cloud::datacatalog::v1::EntryGroup> ListEntryGroups(
      std::string const& parent, Options opts = {});

  ///
  /// Lists entry groups.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::ListEntryGroupsRequest,google/cloud/datacatalog/v1/datacatalog.proto#L751}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryGroup,google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  /// [google.cloud.datacatalog.v1.ListEntryGroupsRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L751}
  /// [google.cloud.datacatalog.v1.EntryGroup]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1275}
  ///
  StreamRange<google::cloud::datacatalog::v1::EntryGroup> ListEntryGroups(
      google::cloud::datacatalog::v1::ListEntryGroupsRequest request,
      Options opts = {});

  ///
  /// Creates an entry.
  ///
  /// You can create entries only with 'FILESET', 'CLUSTER', 'DATA_STREAM',
  /// or custom types. Data Catalog automatically creates entries with other
  /// types during metadata ingestion from integrated systems.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `parent` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// An entry group can have a maximum of 100,000 entries.
  ///
  /// @param parent  Required. The name of the entry group this entry belongs
  /// to.
  ///  Note: The entry itself and its child resources might not be stored in
  ///  the location specified in its name.
  /// @param entry_id  Required. The ID of the entry to create.
  ///  The ID must contain only letters (a-z, A-Z), numbers (0-9),
  ///  and underscores (_).
  ///  The maximum size is 64 bytes when encoded in UTF-8.
  /// @param entry  Required. The entry to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.CreateEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L786}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StatusOr<google::cloud::datacatalog::v1::Entry> CreateEntry(
      std::string const& parent, std::string const& entry_id,
      google::cloud::datacatalog::v1::Entry const& entry, Options opts = {});

  ///
  /// Creates an entry.
  ///
  /// You can create entries only with 'FILESET', 'CLUSTER', 'DATA_STREAM',
  /// or custom types. Data Catalog automatically creates entries with other
  /// types during metadata ingestion from integrated systems.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `parent` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// An entry group can have a maximum of 100,000 entries.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::CreateEntryRequest,google/cloud/datacatalog/v1/datacatalog.proto#L786}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.CreateEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L786}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StatusOr<google::cloud::datacatalog::v1::Entry> CreateEntry(
      google::cloud::datacatalog::v1::CreateEntryRequest const& request,
      Options opts = {});

  ///
  /// Updates an existing entry.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `entry.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param entry  Required. Updates for the entry. The `name` field must be
  /// set.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.UpdateEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L811}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StatusOr<google::cloud::datacatalog::v1::Entry> UpdateEntry(
      google::cloud::datacatalog::v1::Entry const& entry, Options opts = {});

  ///
  /// Updates an existing entry.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `entry.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param entry  Required. Updates for the entry. The `name` field must be
  /// set.
  /// @param update_mask  Names of fields whose values to overwrite on an entry.
  ///  If this parameter is absent or empty, all modifiable fields
  ///  are overwritten. If such fields are non-required and omitted in the
  ///  request body, their values are emptied.
  ///  You can modify only the fields listed below.
  ///  For entries with type `DATA_STREAM`:
  ///  * `schema`
  ///  For entries with type `FILESET`:
  ///  * `schema`
  ///  * `display_name`
  ///  * `description`
  ///  * `gcs_fileset_spec`
  ///  * `gcs_fileset_spec.file_patterns`
  ///  For entries with `user_specified_type`:
  ///  * `schema`
  ///  * `display_name`
  ///  * `description`
  ///  * `user_specified_type`
  ///  * `user_specified_system`
  ///  * `linked_resource`
  ///  * `source_system_timestamps`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.UpdateEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L811}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StatusOr<google::cloud::datacatalog::v1::Entry> UpdateEntry(
      google::cloud::datacatalog::v1::Entry const& entry,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates an existing entry.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `entry.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::UpdateEntryRequest,google/cloud/datacatalog/v1/datacatalog.proto#L811}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.UpdateEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L811}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StatusOr<google::cloud::datacatalog::v1::Entry> UpdateEntry(
      google::cloud::datacatalog::v1::UpdateEntryRequest const& request,
      Options opts = {});

  ///
  /// Deletes an existing entry.
  ///
  /// You can delete only the entries created by the
  /// [CreateEntry][google.cloud.datacatalog.v1.DataCatalog.CreateEntry]
  /// method.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param name  Required. The name of the entry to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L849}
  ///
  Status DeleteEntry(std::string const& name, Options opts = {});

  ///
  /// Deletes an existing entry.
  ///
  /// You can delete only the entries created by the
  /// [CreateEntry][google.cloud.datacatalog.v1.DataCatalog.CreateEntry]
  /// method.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::DeleteEntryRequest,google/cloud/datacatalog/v1/datacatalog.proto#L849}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L849}
  ///
  Status DeleteEntry(
      google::cloud::datacatalog::v1::DeleteEntryRequest const& request,
      Options opts = {});

  ///
  /// Gets an entry.
  ///
  /// @param name  Required. The name of the entry to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.GetEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L861}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StatusOr<google::cloud::datacatalog::v1::Entry> GetEntry(
      std::string const& name, Options opts = {});

  ///
  /// Gets an entry.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::GetEntryRequest,google/cloud/datacatalog/v1/datacatalog.proto#L861}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.GetEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L861}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StatusOr<google::cloud::datacatalog::v1::Entry> GetEntry(
      google::cloud::datacatalog::v1::GetEntryRequest const& request,
      Options opts = {});

  ///
  /// Gets an entry by its target resource name.
  ///
  /// The resource name comes from the source Google Cloud Platform service.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::LookupEntryRequest,google/cloud/datacatalog/v1/datacatalog.proto#L873}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.LookupEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L873}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StatusOr<google::cloud::datacatalog::v1::Entry> LookupEntry(
      google::cloud::datacatalog::v1::LookupEntryRequest const& request,
      Options opts = {});

  ///
  /// Lists entries.
  ///
  /// Note: Currently, this method can list only custom entries.
  /// To get a list of both custom and automatically created entries, use
  /// [SearchCatalog][google.cloud.datacatalog.v1.DataCatalog.SearchCatalog].
  ///
  /// @param parent  Required. The name of the entry group that contains the
  /// entries to list.
  ///  Can be provided in URL format.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.ListEntriesRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1560}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StreamRange<google::cloud::datacatalog::v1::Entry> ListEntries(
      std::string const& parent, Options opts = {});

  ///
  /// Lists entries.
  ///
  /// Note: Currently, this method can list only custom entries.
  /// To get a list of both custom and automatically created entries, use
  /// [SearchCatalog][google.cloud.datacatalog.v1.DataCatalog.SearchCatalog].
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::ListEntriesRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1560}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Entry,google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  /// [google.cloud.datacatalog.v1.ListEntriesRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1560}
  /// [google.cloud.datacatalog.v1.Entry]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L931}
  ///
  StreamRange<google::cloud::datacatalog::v1::Entry> ListEntries(
      google::cloud::datacatalog::v1::ListEntriesRequest request,
      Options opts = {});

  ///
  /// Modifies entry overview, part of the business context of an
  /// [Entry][google.cloud.datacatalog.v1.Entry].
  ///
  /// To call this method, you must have the
  /// `datacatalog.entries.updateOverview` IAM permission on the corresponding
  /// project.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::ModifyEntryOverviewRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1638}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::EntryOverview,google/cloud/datacatalog/v1/datacatalog.proto#L1244}
  ///
  /// [google.cloud.datacatalog.v1.ModifyEntryOverviewRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1638}
  /// [google.cloud.datacatalog.v1.EntryOverview]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1244}
  ///
  StatusOr<google::cloud::datacatalog::v1::EntryOverview> ModifyEntryOverview(
      google::cloud::datacatalog::v1::ModifyEntryOverviewRequest const& request,
      Options opts = {});

  ///
  /// Modifies contacts, part of the business context of an
  /// [Entry][google.cloud.datacatalog.v1.Entry].
  ///
  /// To call this method, you must have the
  /// `datacatalog.entries.updateContacts` IAM permission on the corresponding
  /// project.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::ModifyEntryContactsRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1653}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Contacts,google/cloud/datacatalog/v1/datacatalog.proto#L1256}
  ///
  /// [google.cloud.datacatalog.v1.ModifyEntryContactsRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1653}
  /// [google.cloud.datacatalog.v1.Contacts]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1256}
  ///
  StatusOr<google::cloud::datacatalog::v1::Contacts> ModifyEntryContacts(
      google::cloud::datacatalog::v1::ModifyEntryContactsRequest const& request,
      Options opts = {});

  ///
  /// Creates a tag template.
  ///
  /// You must enable the Data Catalog API in the project identified by the
  /// `parent` parameter.
  /// For more information, see [Data Catalog resource project]
  /// (https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param parent  Required. The name of the project and the template location
  ///  [region](https://cloud.google.com/data-catalog/docs/concepts/regions).
  /// @param tag_template_id  Required. The ID of the tag template to create.
  ///  The ID must contain only lowercase letters (a-z), numbers (0-9),
  ///  or underscores (_), and must start with a letter or underscore.
  ///  The maximum size is 64 bytes when encoded in UTF-8.
  /// @param tag_template  Required. The tag template to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplate,google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  /// [google.cloud.datacatalog.v1.CreateTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1302}
  /// [google.cloud.datacatalog.v1.TagTemplate]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplate> CreateTagTemplate(
      std::string const& parent, std::string const& tag_template_id,
      google::cloud::datacatalog::v1::TagTemplate const& tag_template,
      Options opts = {});

  ///
  /// Creates a tag template.
  ///
  /// You must enable the Data Catalog API in the project identified by the
  /// `parent` parameter.
  /// For more information, see [Data Catalog resource project]
  /// (https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::CreateTagTemplateRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1302}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplate,google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  /// [google.cloud.datacatalog.v1.CreateTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1302}
  /// [google.cloud.datacatalog.v1.TagTemplate]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplate> CreateTagTemplate(
      google::cloud::datacatalog::v1::CreateTagTemplateRequest const& request,
      Options opts = {});

  ///
  /// Gets a tag template.
  ///
  /// @param name  Required. The name of the tag template to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplate,google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  /// [google.cloud.datacatalog.v1.GetTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1325}
  /// [google.cloud.datacatalog.v1.TagTemplate]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplate> GetTagTemplate(
      std::string const& name, Options opts = {});

  ///
  /// Gets a tag template.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::GetTagTemplateRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1325}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplate,google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  /// [google.cloud.datacatalog.v1.GetTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1325}
  /// [google.cloud.datacatalog.v1.TagTemplate]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplate> GetTagTemplate(
      google::cloud::datacatalog::v1::GetTagTemplateRequest const& request,
      Options opts = {});

  ///
  /// Updates a tag template.
  ///
  /// You can't update template fields with this method. These fields are
  /// separate resources with their own create, update, and delete methods.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `tag_template.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param tag_template  Required. The template to update. The `name` field
  /// must be set.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplate,google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1337}
  /// [google.cloud.datacatalog.v1.TagTemplate]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplate> UpdateTagTemplate(
      google::cloud::datacatalog::v1::TagTemplate const& tag_template,
      Options opts = {});

  ///
  /// Updates a tag template.
  ///
  /// You can't update template fields with this method. These fields are
  /// separate resources with their own create, update, and delete methods.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `tag_template.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param tag_template  Required. The template to update. The `name` field
  /// must be set.
  /// @param update_mask  Names of fields whose values to overwrite on a tag
  /// template. Currently,
  ///  only `display_name` and `is_publicly_readable` can be overwritten.
  ///  If this parameter is absent or empty, all modifiable fields
  ///  are overwritten. If such fields are non-required and omitted in the
  ///  request body, their values are emptied.
  ///  Note: Updating the `is_publicly_readable` field may require up to 12
  ///  hours to take effect in search results.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplate,google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1337}
  /// [google.cloud.datacatalog.v1.TagTemplate]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplate> UpdateTagTemplate(
      google::cloud::datacatalog::v1::TagTemplate const& tag_template,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates a tag template.
  ///
  /// You can't update template fields with this method. These fields are
  /// separate resources with their own create, update, and delete methods.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `tag_template.name` parameter. For more information, see [Data Catalog
  /// resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::UpdateTagTemplateRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1337}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplate,google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1337}
  /// [google.cloud.datacatalog.v1.TagTemplate]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L143}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplate> UpdateTagTemplate(
      google::cloud::datacatalog::v1::UpdateTagTemplateRequest const& request,
      Options opts = {});

  ///
  /// Deletes a tag template and all tags that use it.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `name` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param name  Required. The name of the tag template to delete.
  /// @param force  Required. If true, deletes all tags that use this template.
  ///  Currently, `true` is the only supported value.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1355}
  ///
  Status DeleteTagTemplate(std::string const& name, bool force,
                           Options opts = {});

  ///
  /// Deletes a tag template and all tags that use it.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `name` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::DeleteTagTemplateRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1355}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteTagTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1355}
  ///
  Status DeleteTagTemplate(
      google::cloud::datacatalog::v1::DeleteTagTemplateRequest const& request,
      Options opts = {});

  ///
  /// Creates a field in a tag template.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `parent` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param parent  Required. The name of the project and the template location
  ///  [region](https://cloud.google.com/data-catalog/docs/concepts/regions).
  /// @param tag_template_field_id  Required. The ID of the tag template field
  /// to create.
  ///  Note: Adding a required field to an existing template is *not* allowed.
  ///  Field IDs can contain letters (both uppercase and lowercase), numbers
  ///  (0-9), underscores (_) and dashes (-). Field IDs must be at least 1
  ///  character long and at most 128 characters long. Field IDs must also be
  ///  unique within their template.
  /// @param tag_template_field  Required. The tag template field to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.CreateTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1420}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  CreateTagTemplateField(std::string const& parent,
                         std::string const& tag_template_field_id,
                         google::cloud::datacatalog::v1::TagTemplateField const&
                             tag_template_field,
                         Options opts = {});

  ///
  /// Creates a field in a tag template.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `parent` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::CreateTagTemplateFieldRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1420}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.CreateTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1420}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  CreateTagTemplateField(
      google::cloud::datacatalog::v1::CreateTagTemplateFieldRequest const&
          request,
      Options opts = {});

  ///
  /// Updates a field in a tag template.
  ///
  /// You can't update the field type with this method.
  ///
  /// You must enable the Data Catalog API in the project
  /// identified by the `name` parameter. For more information, see [Data
  /// Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param name  Required. The name of the tag template field.
  /// @param tag_template_field  Required. The template to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1446}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  UpdateTagTemplateField(std::string const& name,
                         google::cloud::datacatalog::v1::TagTemplateField const&
                             tag_template_field,
                         Options opts = {});

  ///
  /// Updates a field in a tag template.
  ///
  /// You can't update the field type with this method.
  ///
  /// You must enable the Data Catalog API in the project
  /// identified by the `name` parameter. For more information, see [Data
  /// Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param name  Required. The name of the tag template field.
  /// @param tag_template_field  Required. The template to update.
  /// @param update_mask  Optional. Names of fields whose values to overwrite on
  /// an individual field of a tag
  ///  template. The following fields are modifiable:
  ///  * `display_name`
  ///  * `type.enum_type`
  ///  * `is_required`
  ///  If this parameter is absent or empty, all modifiable fields
  ///  are overwritten. If such fields are non-required and omitted in the
  ///  request body, their values are emptied with one exception: when updating
  ///  an enum type, the provided values are merged with the existing values.
  ///  Therefore, enum values can only be added, existing enum values cannot be
  ///  deleted or renamed. Additionally, updating a template field from optional
  ///  to required is *not* allowed.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1446}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  UpdateTagTemplateField(std::string const& name,
                         google::cloud::datacatalog::v1::TagTemplateField const&
                             tag_template_field,
                         google::protobuf::FieldMask const& update_mask,
                         Options opts = {});

  ///
  /// Updates a field in a tag template.
  ///
  /// You can't update the field type with this method.
  ///
  /// You must enable the Data Catalog API in the project
  /// identified by the `name` parameter. For more information, see [Data
  /// Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::UpdateTagTemplateFieldRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1446}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1446}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  UpdateTagTemplateField(
      google::cloud::datacatalog::v1::UpdateTagTemplateFieldRequest const&
          request,
      Options opts = {});

  ///
  /// Renames a field in a tag template.
  ///
  /// You must enable the Data Catalog API in the project identified by the
  /// `name` parameter. For more information, see [Data Catalog resource
  /// project]
  /// (https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param name  Required. The name of the tag template field.
  /// @param new_tag_template_field_id  Required. The new ID of this tag
  /// template field. For example, `my_new_field`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.RenameTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1479}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  RenameTagTemplateField(std::string const& name,
                         std::string const& new_tag_template_field_id,
                         Options opts = {});

  ///
  /// Renames a field in a tag template.
  ///
  /// You must enable the Data Catalog API in the project identified by the
  /// `name` parameter. For more information, see [Data Catalog resource
  /// project]
  /// (https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::RenameTagTemplateFieldRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1479}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.RenameTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1479}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  RenameTagTemplateField(
      google::cloud::datacatalog::v1::RenameTagTemplateFieldRequest const&
          request,
      Options opts = {});

  ///
  /// Renames an enum value in a tag template.
  ///
  /// Within a single enum field, enum values must be unique.
  ///
  /// @param name  Required. The name of the enum field value.
  /// @param new_enum_value_display_name  Required. The new display name of the
  /// enum value. For example, `my_new_enum_value`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.RenameTagTemplateFieldEnumValueRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1494}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  RenameTagTemplateFieldEnumValue(
      std::string const& name, std::string const& new_enum_value_display_name,
      Options opts = {});

  ///
  /// Renames an enum value in a tag template.
  ///
  /// Within a single enum field, enum values must be unique.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::RenameTagTemplateFieldEnumValueRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1494}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::TagTemplateField,google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  /// [google.cloud.datacatalog.v1.RenameTagTemplateFieldEnumValueRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1494}
  /// [google.cloud.datacatalog.v1.TagTemplateField]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L185}
  ///
  StatusOr<google::cloud::datacatalog::v1::TagTemplateField>
  RenameTagTemplateFieldEnumValue(
      google::cloud::datacatalog::v1::
          RenameTagTemplateFieldEnumValueRequest const& request,
      Options opts = {});

  ///
  /// Deletes a field in a tag template and all uses of this field from the tags
  /// based on this template.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `name` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param name  Required. The name of the tag template field to delete.
  /// @param force  Required. If true, deletes this field from any tags that use
  /// it.
  ///  Currently, `true` is the only supported value.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1509}
  ///
  Status DeleteTagTemplateField(std::string const& name, bool force,
                                Options opts = {});

  ///
  /// Deletes a field in a tag template and all uses of this field from the tags
  /// based on this template.
  ///
  /// You must enable the Data Catalog API in the project identified by
  /// the `name` parameter. For more information, see [Data Catalog resource
  /// project](https://cloud.google.com/data-catalog/docs/concepts/resource-project).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::DeleteTagTemplateFieldRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1509}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteTagTemplateFieldRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1509}
  ///
  Status DeleteTagTemplateField(
      google::cloud::datacatalog::v1::DeleteTagTemplateFieldRequest const&
          request,
      Options opts = {});

  ///
  /// Creates a tag and assigns it to:
  ///
  /// * An [Entry][google.cloud.datacatalog.v1.Entry] if the method name is
  ///   `projects.locations.entryGroups.entries.tags.create`.
  /// * Or [EntryGroup][google.cloud.datacatalog.v1.EntryGroup]if the method
  ///   name is `projects.locations.entryGroups.tags.create`.
  ///
  /// Note: The project identified by the `parent` parameter for the [tag]
  /// (https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.entryGroups.entries.tags/create#path-parameters)
  /// and the [tag template]
  /// (https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.tagTemplates/create#path-parameters)
  /// used to create the tag must be in the same organization.
  ///
  /// @param parent  Required. The name of the resource to attach this tag to.
  ///  Tags can be attached to entries or entry groups. An entry can have up to
  ///  1000 attached tags.
  ///  Note: The tag and its child resources might not be stored in
  ///  the location specified in its name.
  /// @param tag  Required. The tag to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Tag,google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  /// [google.cloud.datacatalog.v1.CreateTagRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1372}
  /// [google.cloud.datacatalog.v1.Tag]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  StatusOr<google::cloud::datacatalog::v1::Tag> CreateTag(
      std::string const& parent, google::cloud::datacatalog::v1::Tag const& tag,
      Options opts = {});

  ///
  /// Creates a tag and assigns it to:
  ///
  /// * An [Entry][google.cloud.datacatalog.v1.Entry] if the method name is
  ///   `projects.locations.entryGroups.entries.tags.create`.
  /// * Or [EntryGroup][google.cloud.datacatalog.v1.EntryGroup]if the method
  ///   name is `projects.locations.entryGroups.tags.create`.
  ///
  /// Note: The project identified by the `parent` parameter for the [tag]
  /// (https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.entryGroups.entries.tags/create#path-parameters)
  /// and the [tag template]
  /// (https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.tagTemplates/create#path-parameters)
  /// used to create the tag must be in the same organization.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::CreateTagRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1372}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Tag,google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  /// [google.cloud.datacatalog.v1.CreateTagRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1372}
  /// [google.cloud.datacatalog.v1.Tag]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  StatusOr<google::cloud::datacatalog::v1::Tag> CreateTag(
      google::cloud::datacatalog::v1::CreateTagRequest const& request,
      Options opts = {});

  ///
  /// Updates an existing tag.
  ///
  /// @param tag  Required. The updated tag. The "name" field must be set.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Tag,google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1393}
  /// [google.cloud.datacatalog.v1.Tag]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  StatusOr<google::cloud::datacatalog::v1::Tag> UpdateTag(
      google::cloud::datacatalog::v1::Tag const& tag, Options opts = {});

  ///
  /// Updates an existing tag.
  ///
  /// @param tag  Required. The updated tag. The "name" field must be set.
  /// @param update_mask  Names of fields whose values to overwrite on a tag.
  /// Currently, a tag has
  ///  the only modifiable field with the name `fields`.
  ///  In general, if this parameter is absent or empty, all modifiable fields
  ///  are overwritten. If such fields are non-required and omitted in the
  ///  request body, their values are emptied.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Tag,google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1393}
  /// [google.cloud.datacatalog.v1.Tag]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  StatusOr<google::cloud::datacatalog::v1::Tag> UpdateTag(
      google::cloud::datacatalog::v1::Tag const& tag,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates an existing tag.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::UpdateTagRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1393}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Tag,google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  /// [google.cloud.datacatalog.v1.UpdateTagRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1393}
  /// [google.cloud.datacatalog.v1.Tag]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  StatusOr<google::cloud::datacatalog::v1::Tag> UpdateTag(
      google::cloud::datacatalog::v1::UpdateTagRequest const& request,
      Options opts = {});

  ///
  /// Deletes a tag.
  ///
  /// @param name  Required. The name of the tag to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteTagRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1408}
  ///
  Status DeleteTag(std::string const& name, Options opts = {});

  ///
  /// Deletes a tag.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::DeleteTagRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1408}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.datacatalog.v1.DeleteTagRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1408}
  ///
  Status DeleteTag(
      google::cloud::datacatalog::v1::DeleteTagRequest const& request,
      Options opts = {});

  ///
  /// Lists tags assigned to an [Entry][google.cloud.datacatalog.v1.Entry].
  /// The [columns][google.cloud.datacatalog.v1.Tag.column] in the response are
  /// lowercased.
  ///
  /// @param parent  Required. The name of the Data Catalog resource to list the
  /// tags of.
  ///  The resource can be an [Entry][google.cloud.datacatalog.v1.Entry]
  ///  or an [EntryGroup][google.cloud.datacatalog.v1.EntryGroup]
  ///  (without `/entries/{entries}` at the end).
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Tag,google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  /// [google.cloud.datacatalog.v1.ListTagsRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1526}
  /// [google.cloud.datacatalog.v1.Tag]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  StreamRange<google::cloud::datacatalog::v1::Tag> ListTags(
      std::string const& parent, Options opts = {});

  ///
  /// Lists tags assigned to an [Entry][google.cloud.datacatalog.v1.Entry].
  /// The [columns][google.cloud.datacatalog.v1.Tag.column] in the response are
  /// lowercased.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::ListTagsRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1526}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::Tag,google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  /// [google.cloud.datacatalog.v1.ListTagsRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1526}
  /// [google.cloud.datacatalog.v1.Tag]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/tags.proto#L37}
  ///
  StreamRange<google::cloud::datacatalog::v1::Tag> ListTags(
      google::cloud::datacatalog::v1::ListTagsRequest request,
      Options opts = {});

  ///
  /// Marks an [Entry][google.cloud.datacatalog.v1.Entry] as starred by
  /// the current user. Starring information is private to each user.
  ///
  /// @param name  Required. The name of the entry to mark as starred.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::StarEntryResponse,google/cloud/datacatalog/v1/datacatalog.proto#L1613}
  ///
  /// [google.cloud.datacatalog.v1.StarEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1600}
  /// [google.cloud.datacatalog.v1.StarEntryResponse]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1613}
  ///
  StatusOr<google::cloud::datacatalog::v1::StarEntryResponse> StarEntry(
      std::string const& name, Options opts = {});

  ///
  /// Marks an [Entry][google.cloud.datacatalog.v1.Entry] as starred by
  /// the current user. Starring information is private to each user.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::StarEntryRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1600}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::StarEntryResponse,google/cloud/datacatalog/v1/datacatalog.proto#L1613}
  ///
  /// [google.cloud.datacatalog.v1.StarEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1600}
  /// [google.cloud.datacatalog.v1.StarEntryResponse]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1613}
  ///
  StatusOr<google::cloud::datacatalog::v1::StarEntryResponse> StarEntry(
      google::cloud::datacatalog::v1::StarEntryRequest const& request,
      Options opts = {});

  ///
  /// Marks an [Entry][google.cloud.datacatalog.v1.Entry] as NOT starred by
  /// the current user. Starring information is private to each user.
  ///
  /// @param name  Required. The name of the entry to mark as **not** starred.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::UnstarEntryResponse,google/cloud/datacatalog/v1/datacatalog.proto#L1632}
  ///
  /// [google.cloud.datacatalog.v1.UnstarEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1619}
  /// [google.cloud.datacatalog.v1.UnstarEntryResponse]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1632}
  ///
  StatusOr<google::cloud::datacatalog::v1::UnstarEntryResponse> UnstarEntry(
      std::string const& name, Options opts = {});

  ///
  /// Marks an [Entry][google.cloud.datacatalog.v1.Entry] as NOT starred by
  /// the current user. Starring information is private to each user.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datacatalog::v1::UnstarEntryRequest,google/cloud/datacatalog/v1/datacatalog.proto#L1619}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datacatalog::v1::UnstarEntryResponse,google/cloud/datacatalog/v1/datacatalog.proto#L1632}
  ///
  /// [google.cloud.datacatalog.v1.UnstarEntryRequest]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1619}
  /// [google.cloud.datacatalog.v1.UnstarEntryResponse]:
  /// @googleapis_reference_link{google/cloud/datacatalog/v1/datacatalog.proto#L1632}
  ///
  StatusOr<google::cloud::datacatalog::v1::UnstarEntryResponse> UnstarEntry(
      google::cloud::datacatalog::v1::UnstarEntryRequest const& request,
      Options opts = {});

  ///
  /// Sets an access control policy for a resource. Replaces any existing
  /// policy.
  ///
  /// Supported resources are:
  ///
  /// - Tag templates
  /// - Entry groups
  ///
  /// Note: This method sets policies only within Data Catalog and can't be
  /// used to manage policies in BigQuery, Pub/Sub, Dataproc Metastore, and any
  /// external Google Cloud Platform resources synced with the Data Catalog.
  ///
  /// To call this method, you must have the following Google IAM permissions:
  ///
  /// - `datacatalog.tagTemplates.setIamPolicy` to set policies on tag
  ///   templates.
  /// - `datacatalog.entryGroups.setIamPolicy` to set policies on entry groups.
  ///
  /// @param resource  REQUIRED: The resource for which the policy is being
  /// specified.
  ///  See the operation documentation for the appropriate value for this field.
  /// @param policy  REQUIRED: The complete policy to be applied to the
  /// `resource`. The size of
  ///  the policy is limited to a few 10s of KB. An empty policy is a
  ///  valid policy but certain Cloud Platform services (such as Projects)
  ///  might reject them.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::iam::v1::Policy,google/iam/v1/policy.proto#L96}
  ///
  /// [google.iam.v1.SetIamPolicyRequest]:
  /// @googleapis_reference_link{google/iam/v1/iam_policy.proto#L101}
  /// [google.iam.v1.Policy]:
  /// @googleapis_reference_link{google/iam/v1/policy.proto#L96}
  ///
  StatusOr<google::iam::v1::Policy> SetIamPolicy(
      std::string const& resource, google::iam::v1::Policy const& policy,
      Options opts = {});

  /**
   * Updates the IAM policy for @p resource using an optimistic concurrency
   * control loop.
   *
   * The loop fetches the current policy for @p resource, and passes it to @p
   * updater, which should return the new policy. This new policy should use the
   * current etag so that the read-modify-write cycle can detect races and rerun
   * the update when there is a mismatch. If the new policy does not have an
   * etag, the existing policy will be blindly overwritten. If @p updater does
   * not yield a policy, the control loop is terminated and kCancelled is
   * returned.
   *
   * @param resource  Required. The resource for which the policy is being
   * specified. See the operation documentation for the appropriate value for
   * this field.
   * @param updater  Required. Functor to map the current policy to a new one.
   * @param opts  Optional. Override the class-level options, such as retry and
   *    backoff policies.
   * @return google::iam::v1::Policy
   */
  StatusOr<google::iam::v1::Policy> SetIamPolicy(std::string const& resource,
                                                 IamUpdater const& updater,
                                                 Options opts = {});

  ///
  /// Sets an access control policy for a resource. Replaces any existing
  /// policy.
  ///
  /// Supported resources are:
  ///
  /// - Tag templates
  /// - Entry groups
  ///
  /// Note: This method sets policies only within Data Catalog and can't be
  /// used to manage policies in BigQuery, Pub/Sub, Dataproc Metastore, and any
  /// external Google Cloud Platform resources synced with the Data Catalog.
  ///
  /// To call this method, you must have the following Google IAM permissions:
  ///
  /// - `datacatalog.tagTemplates.setIamPolicy` to set policies on tag
  ///   templates.
  /// - `datacatalog.entryGroups.setIamPolicy` to set policies on entry groups.
  ///
  /// @param request
  /// @googleapis_link{google::iam::v1::SetIamPolicyRequest,google/iam/v1/iam_policy.proto#L101}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::iam::v1::Policy,google/iam/v1/policy.proto#L96}
  ///
  /// [google.iam.v1.SetIamPolicyRequest]:
  /// @googleapis_reference_link{google/iam/v1/iam_policy.proto#L101}
  /// [google.iam.v1.Policy]:
  /// @googleapis_reference_link{google/iam/v1/policy.proto#L96}
  ///
  StatusOr<google::iam::v1::Policy> SetIamPolicy(
      google::iam::v1::SetIamPolicyRequest const& request, Options opts = {});

  ///
  /// Gets the access control policy for a resource.
  ///
  /// May return:
  ///
  /// * A`NOT_FOUND` error if the resource doesn't exist or you don't have the
  ///   permission to view it.
  /// * An empty policy if the resource exists but doesn't have a set policy.
  ///
  /// Supported resources are:
  ///
  /// - Tag templates
  /// - Entry groups
  ///
  /// Note: This method doesn't get policies from Google Cloud Platform
  /// resources ingested into Data Catalog.
  ///
  /// To call this method, you must have the following Google IAM permissions:
  ///
  /// - `datacatalog.tagTemplates.getIamPolicy` to get policies on tag
  ///   templates.
  /// - `datacatalog.entryGroups.getIamPolicy` to get policies on entry groups.
  ///
  /// @param resource  REQUIRED: The resource for which the policy is being
  /// requested.
  ///  See the operation documentation for the appropriate value for this field.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::iam::v1::Policy,google/iam/v1/policy.proto#L96}
  ///
  /// [google.iam.v1.GetIamPolicyRequest]:
  /// @googleapis_reference_link{google/iam/v1/iam_policy.proto#L123}
  /// [google.iam.v1.Policy]:
  /// @googleapis_reference_link{google/iam/v1/policy.proto#L96}
  ///
  StatusOr<google::iam::v1::Policy> GetIamPolicy(std::string const& resource,
                                                 Options opts = {});

  ///
  /// Gets the access control policy for a resource.
  ///
  /// May return:
  ///
  /// * A`NOT_FOUND` error if the resource doesn't exist or you don't have the
  ///   permission to view it.
  /// * An empty policy if the resource exists but doesn't have a set policy.
  ///
  /// Supported resources are:
  ///
  /// - Tag templates
  /// - Entry groups
  ///
  /// Note: This method doesn't get policies from Google Cloud Platform
  /// resources ingested into Data Catalog.
  ///
  /// To call this method, you must have the following Google IAM permissions:
  ///
  /// - `datacatalog.tagTemplates.getIamPolicy` to get policies on tag
  ///   templates.
  /// - `datacatalog.entryGroups.getIamPolicy` to get policies on entry groups.
  ///
  /// @param request
  /// @googleapis_link{google::iam::v1::GetIamPolicyRequest,google/iam/v1/iam_policy.proto#L123}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::iam::v1::Policy,google/iam/v1/policy.proto#L96}
  ///
  /// [google.iam.v1.GetIamPolicyRequest]:
  /// @googleapis_reference_link{google/iam/v1/iam_policy.proto#L123}
  /// [google.iam.v1.Policy]:
  /// @googleapis_reference_link{google/iam/v1/policy.proto#L96}
  ///
  StatusOr<google::iam::v1::Policy> GetIamPolicy(
      google::iam::v1::GetIamPolicyRequest const& request, Options opts = {});

  ///
  /// Gets your permissions on a resource.
  ///
  /// Returns an empty set of permissions if the resource doesn't exist.
  ///
  /// Supported resources are:
  ///
  /// - Tag templates
  /// - Entry groups
  ///
  /// Note: This method gets policies only within Data Catalog and can't be
  /// used to get policies from BigQuery, Pub/Sub, Dataproc Metastore, and any
  /// external Google Cloud Platform resources ingested into Data Catalog.
  ///
  /// No Google IAM permissions are required to call this method.
  ///
  /// @param request
  /// @googleapis_link{google::iam::v1::TestIamPermissionsRequest,google/iam/v1/iam_policy.proto#L136}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::iam::v1::TestIamPermissionsResponse,google/iam/v1/iam_policy.proto#L151}
  ///
  /// [google.iam.v1.TestIamPermissionsRequest]:
  /// @googleapis_reference_link{google/iam/v1/iam_policy.proto#L136}
  /// [google.iam.v1.TestIamPermissionsResponse]:
  /// @googleapis_reference_link{google/iam/v1/iam_policy.proto#L151}
  ///
  StatusOr<google::iam::v1::TestIamPermissionsResponse> TestIamPermissions(
      google::iam::v1::TestIamPermissionsRequest const& request,
      Options opts = {});

 private:
  std::shared_ptr<DataCatalogConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace datacatalog
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATACATALOG_DATA_CATALOG_CLIENT_H
