// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dataplex/v1/service.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATAPLEX_DATAPLEX_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATAPLEX_DATAPLEX_CLIENT_H

#include "google/cloud/dataplex/dataplex_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace dataplex {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Dataplex service provides data lakes as a service. The primary resources
/// offered by this service are Lakes, Zones and Assets which collectively allow
/// a data administrator to organize, manage, secure and catalog data across
/// their organization located across cloud projects in a variety of storage
/// systems including Cloud Storage and BigQuery.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class DataplexServiceClient {
 public:
  explicit DataplexServiceClient(
      std::shared_ptr<DataplexServiceConnection> connection, Options opts = {});
  ~DataplexServiceClient();

  ///@{
  // @name Copy and move support
  DataplexServiceClient(DataplexServiceClient const&) = default;
  DataplexServiceClient& operator=(DataplexServiceClient const&) = default;
  DataplexServiceClient(DataplexServiceClient&&) = default;
  DataplexServiceClient& operator=(DataplexServiceClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(DataplexServiceClient const& a,
                         DataplexServiceClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(DataplexServiceClient const& a,
                         DataplexServiceClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Creates a lake resource.
  ///
  /// @param parent  Required. The resource name of the lake location, of the
  /// form:
  ///  projects/{project_number}/locations/{location_id}
  ///  where `location_id` refers to a GCP region.
  /// @param lake  Required. Lake resource
  /// @param lake_id  Required. Lake identifier.
  ///  This ID will be used to generate names such as database and dataset names
  ///  when publishing metadata to Hive Metastore and BigQuery.
  ///  * Must contain only lowercase letters, numbers and hyphens.
  ///  * Must start with a letter.
  ///  * Must end with a number or a letter.
  ///  * Must be between 1-63 characters.
  ///  * Must be unique within the customer project / location.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Lake,google/cloud/dataplex/v1/resources.proto#L37}
  ///
  /// [google.cloud.dataplex.v1.CreateLakeRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L378}
  /// [google.cloud.dataplex.v1.Lake]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L37}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Lake>> CreateLake(
      std::string const& parent, google::cloud::dataplex::v1::Lake const& lake,
      std::string const& lake_id, Options opts = {});

  ///
  /// Creates a lake resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::CreateLakeRequest,google/cloud/dataplex/v1/service.proto#L378}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Lake,google/cloud/dataplex/v1/resources.proto#L37}
  ///
  /// [google.cloud.dataplex.v1.CreateLakeRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L378}
  /// [google.cloud.dataplex.v1.Lake]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L37}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Lake>> CreateLake(
      google::cloud::dataplex::v1::CreateLakeRequest const& request,
      Options opts = {});

  ///
  /// Updates a lake resource.
  ///
  /// @param lake  Required. Update description.
  ///  Only fields specified in `update_mask` are updated.
  /// @param update_mask  Required. Mask of fields to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Lake,google/cloud/dataplex/v1/resources.proto#L37}
  ///
  /// [google.cloud.dataplex.v1.UpdateLakeRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L408}
  /// [google.cloud.dataplex.v1.Lake]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L37}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Lake>> UpdateLake(
      google::cloud::dataplex::v1::Lake const& lake,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates a lake resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::UpdateLakeRequest,google/cloud/dataplex/v1/service.proto#L408}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Lake,google/cloud/dataplex/v1/resources.proto#L37}
  ///
  /// [google.cloud.dataplex.v1.UpdateLakeRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L408}
  /// [google.cloud.dataplex.v1.Lake]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L37}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Lake>> UpdateLake(
      google::cloud::dataplex::v1::UpdateLakeRequest const& request,
      Options opts = {});

  ///
  /// Deletes a lake resource. All zones within the lake must be deleted before
  /// the lake can be deleted.
  ///
  /// @param name  Required. The resource name of the lake:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteLakeRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L422}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>> DeleteLake(
      std::string const& name, Options opts = {});

  ///
  /// Deletes a lake resource. All zones within the lake must be deleted before
  /// the lake can be deleted.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::DeleteLakeRequest,google/cloud/dataplex/v1/service.proto#L422}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteLakeRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L422}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>> DeleteLake(
      google::cloud::dataplex::v1::DeleteLakeRequest const& request,
      Options opts = {});

  ///
  /// Lists lake resources in a project and location.
  ///
  /// @param parent  Required. The resource name of the lake location, of the
  /// form:
  ///  `projects/{project_number}/locations/{location_id}`
  ///  where `location_id` refers to a GCP region.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Lake,google/cloud/dataplex/v1/resources.proto#L37}
  ///
  /// [google.cloud.dataplex.v1.ListLakesRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L434}
  /// [google.cloud.dataplex.v1.Lake]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L37}
  ///
  StreamRange<google::cloud::dataplex::v1::Lake> ListLakes(
      std::string const& parent, Options opts = {});

  ///
  /// Lists lake resources in a project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListLakesRequest,google/cloud/dataplex/v1/service.proto#L434}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Lake,google/cloud/dataplex/v1/resources.proto#L37}
  ///
  /// [google.cloud.dataplex.v1.ListLakesRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L434}
  /// [google.cloud.dataplex.v1.Lake]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L37}
  ///
  StreamRange<google::cloud::dataplex::v1::Lake> ListLakes(
      google::cloud::dataplex::v1::ListLakesRequest request, Options opts = {});

  ///
  /// Retrieves a lake resource.
  ///
  /// @param name  Required. The resource name of the lake:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Lake,google/cloud/dataplex/v1/resources.proto#L37}
  ///
  /// [google.cloud.dataplex.v1.GetLakeRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L509}
  /// [google.cloud.dataplex.v1.Lake]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L37}
  ///
  StatusOr<google::cloud::dataplex::v1::Lake> GetLake(std::string const& name,
                                                      Options opts = {});

  ///
  /// Retrieves a lake resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::GetLakeRequest,google/cloud/dataplex/v1/service.proto#L509}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Lake,google/cloud/dataplex/v1/resources.proto#L37}
  ///
  /// [google.cloud.dataplex.v1.GetLakeRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L509}
  /// [google.cloud.dataplex.v1.Lake]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L37}
  ///
  StatusOr<google::cloud::dataplex::v1::Lake> GetLake(
      google::cloud::dataplex::v1::GetLakeRequest const& request,
      Options opts = {});

  ///
  /// Lists action resources in a lake.
  ///
  /// @param parent  Required. The resource name of the parent lake:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Action,google/cloud/dataplex/v1/resources.proto#L308}
  ///
  /// [google.cloud.dataplex.v1.ListLakeActionsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L476}
  /// [google.cloud.dataplex.v1.Action]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L308}
  ///
  StreamRange<google::cloud::dataplex::v1::Action> ListLakeActions(
      std::string const& parent, Options opts = {});

  ///
  /// Lists action resources in a lake.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListLakeActionsRequest,google/cloud/dataplex/v1/service.proto#L476}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Action,google/cloud/dataplex/v1/resources.proto#L308}
  ///
  /// [google.cloud.dataplex.v1.ListLakeActionsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L476}
  /// [google.cloud.dataplex.v1.Action]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L308}
  ///
  StreamRange<google::cloud::dataplex::v1::Action> ListLakeActions(
      google::cloud::dataplex::v1::ListLakeActionsRequest request,
      Options opts = {});

  ///
  /// Creates a zone resource within a lake.
  ///
  /// @param parent  Required. The resource name of the parent lake:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}`.
  /// @param zone  Required. Zone resource.
  /// @param zone_id  Required. Zone identifier.
  ///  This ID will be used to generate names such as database and dataset names
  ///  when publishing metadata to Hive Metastore and BigQuery.
  ///  * Must contain only lowercase letters, numbers and hyphens.
  ///  * Must start with a letter.
  ///  * Must end with a number or a letter.
  ///  * Must be between 1-63 characters.
  ///  * Must be unique across all lakes from all locations in a project.
  ///  * Must not be one of the reserved IDs (i.e. "default", "global-temp")
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Zone,google/cloud/dataplex/v1/resources.proto#L148}
  ///
  /// [google.cloud.dataplex.v1.CreateZoneRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L521}
  /// [google.cloud.dataplex.v1.Zone]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L148}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Zone>> CreateZone(
      std::string const& parent, google::cloud::dataplex::v1::Zone const& zone,
      std::string const& zone_id, Options opts = {});

  ///
  /// Creates a zone resource within a lake.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::CreateZoneRequest,google/cloud/dataplex/v1/service.proto#L521}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Zone,google/cloud/dataplex/v1/resources.proto#L148}
  ///
  /// [google.cloud.dataplex.v1.CreateZoneRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L521}
  /// [google.cloud.dataplex.v1.Zone]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L148}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Zone>> CreateZone(
      google::cloud::dataplex::v1::CreateZoneRequest const& request,
      Options opts = {});

  ///
  /// Updates a zone resource.
  ///
  /// @param zone  Required. Update description.
  ///  Only fields specified in `update_mask` are updated.
  /// @param update_mask  Required. Mask of fields to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Zone,google/cloud/dataplex/v1/resources.proto#L148}
  ///
  /// [google.cloud.dataplex.v1.UpdateZoneRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L551}
  /// [google.cloud.dataplex.v1.Zone]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L148}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Zone>> UpdateZone(
      google::cloud::dataplex::v1::Zone const& zone,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates a zone resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::UpdateZoneRequest,google/cloud/dataplex/v1/service.proto#L551}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Zone,google/cloud/dataplex/v1/resources.proto#L148}
  ///
  /// [google.cloud.dataplex.v1.UpdateZoneRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L551}
  /// [google.cloud.dataplex.v1.Zone]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L148}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Zone>> UpdateZone(
      google::cloud::dataplex::v1::UpdateZoneRequest const& request,
      Options opts = {});

  ///
  /// Deletes a zone resource. All assets within a zone must be deleted before
  /// the zone can be deleted.
  ///
  /// @param name  Required. The resource name of the zone:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/zones/{zone_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteZoneRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L565}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>> DeleteZone(
      std::string const& name, Options opts = {});

  ///
  /// Deletes a zone resource. All assets within a zone must be deleted before
  /// the zone can be deleted.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::DeleteZoneRequest,google/cloud/dataplex/v1/service.proto#L565}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteZoneRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L565}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>> DeleteZone(
      google::cloud::dataplex::v1::DeleteZoneRequest const& request,
      Options opts = {});

  ///
  /// Lists zone resources in a lake.
  ///
  /// @param parent  Required. The resource name of the parent lake:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Zone,google/cloud/dataplex/v1/resources.proto#L148}
  ///
  /// [google.cloud.dataplex.v1.ListZonesRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L577}
  /// [google.cloud.dataplex.v1.Zone]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L148}
  ///
  StreamRange<google::cloud::dataplex::v1::Zone> ListZones(
      std::string const& parent, Options opts = {});

  ///
  /// Lists zone resources in a lake.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListZonesRequest,google/cloud/dataplex/v1/service.proto#L577}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Zone,google/cloud/dataplex/v1/resources.proto#L148}
  ///
  /// [google.cloud.dataplex.v1.ListZonesRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L577}
  /// [google.cloud.dataplex.v1.Zone]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L148}
  ///
  StreamRange<google::cloud::dataplex::v1::Zone> ListZones(
      google::cloud::dataplex::v1::ListZonesRequest request, Options opts = {});

  ///
  /// Retrieves a zone resource.
  ///
  /// @param name  Required. The resource name of the zone:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/zones/{zone_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Zone,google/cloud/dataplex/v1/resources.proto#L148}
  ///
  /// [google.cloud.dataplex.v1.GetZoneRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L638}
  /// [google.cloud.dataplex.v1.Zone]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L148}
  ///
  StatusOr<google::cloud::dataplex::v1::Zone> GetZone(std::string const& name,
                                                      Options opts = {});

  ///
  /// Retrieves a zone resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::GetZoneRequest,google/cloud/dataplex/v1/service.proto#L638}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Zone,google/cloud/dataplex/v1/resources.proto#L148}
  ///
  /// [google.cloud.dataplex.v1.GetZoneRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L638}
  /// [google.cloud.dataplex.v1.Zone]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L148}
  ///
  StatusOr<google::cloud::dataplex::v1::Zone> GetZone(
      google::cloud::dataplex::v1::GetZoneRequest const& request,
      Options opts = {});

  ///
  /// Lists action resources in a zone.
  ///
  /// @param parent  Required. The resource name of the parent zone:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/zones/{zone_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Action,google/cloud/dataplex/v1/resources.proto#L308}
  ///
  /// [google.cloud.dataplex.v1.ListZoneActionsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L615}
  /// [google.cloud.dataplex.v1.Action]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L308}
  ///
  StreamRange<google::cloud::dataplex::v1::Action> ListZoneActions(
      std::string const& parent, Options opts = {});

  ///
  /// Lists action resources in a zone.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListZoneActionsRequest,google/cloud/dataplex/v1/service.proto#L615}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Action,google/cloud/dataplex/v1/resources.proto#L308}
  ///
  /// [google.cloud.dataplex.v1.ListZoneActionsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L615}
  /// [google.cloud.dataplex.v1.Action]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L308}
  ///
  StreamRange<google::cloud::dataplex::v1::Action> ListZoneActions(
      google::cloud::dataplex::v1::ListZoneActionsRequest request,
      Options opts = {});

  ///
  /// Creates an asset resource.
  ///
  /// @param parent  Required. The resource name of the parent zone:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/zones/{zone_id}`.
  /// @param asset  Required. Asset resource.
  /// @param asset_id  Required. Asset identifier.
  ///  This ID will be used to generate names such as table names when
  ///  publishing metadata to Hive Metastore and BigQuery.
  ///  * Must contain only lowercase letters, numbers and hyphens.
  ///  * Must start with a letter.
  ///  * Must end with a number or a letter.
  ///  * Must be between 1-63 characters.
  ///  * Must be unique within the zone.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Asset,google/cloud/dataplex/v1/resources.proto#L514}
  ///
  /// [google.cloud.dataplex.v1.CreateAssetRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L650}
  /// [google.cloud.dataplex.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L514}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Asset>> CreateAsset(
      std::string const& parent,
      google::cloud::dataplex::v1::Asset const& asset,
      std::string const& asset_id, Options opts = {});

  ///
  /// Creates an asset resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::CreateAssetRequest,google/cloud/dataplex/v1/service.proto#L650}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Asset,google/cloud/dataplex/v1/resources.proto#L514}
  ///
  /// [google.cloud.dataplex.v1.CreateAssetRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L650}
  /// [google.cloud.dataplex.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L514}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Asset>> CreateAsset(
      google::cloud::dataplex::v1::CreateAssetRequest const& request,
      Options opts = {});

  ///
  /// Updates an asset resource.
  ///
  /// @param asset  Required. Update description.
  ///  Only fields specified in `update_mask` are updated.
  /// @param update_mask  Required. Mask of fields to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Asset,google/cloud/dataplex/v1/resources.proto#L514}
  ///
  /// [google.cloud.dataplex.v1.UpdateAssetRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L679}
  /// [google.cloud.dataplex.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L514}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Asset>> UpdateAsset(
      google::cloud::dataplex::v1::Asset const& asset,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates an asset resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::UpdateAssetRequest,google/cloud/dataplex/v1/service.proto#L679}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Asset,google/cloud/dataplex/v1/resources.proto#L514}
  ///
  /// [google.cloud.dataplex.v1.UpdateAssetRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L679}
  /// [google.cloud.dataplex.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L514}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Asset>> UpdateAsset(
      google::cloud::dataplex::v1::UpdateAssetRequest const& request,
      Options opts = {});

  ///
  /// Deletes an asset resource. The referenced storage resource is detached
  /// (default) or deleted based on the associated Lifecycle policy.
  ///
  /// @param name  Required. The resource name of the asset:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/zones/{zone_id}/assets/{asset_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteAssetRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L693}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>> DeleteAsset(
      std::string const& name, Options opts = {});

  ///
  /// Deletes an asset resource. The referenced storage resource is detached
  /// (default) or deleted based on the associated Lifecycle policy.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::DeleteAssetRequest,google/cloud/dataplex/v1/service.proto#L693}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteAssetRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L693}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>> DeleteAsset(
      google::cloud::dataplex::v1::DeleteAssetRequest const& request,
      Options opts = {});

  ///
  /// Lists asset resources in a zone.
  ///
  /// @param parent  Required. The resource name of the parent zone:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/zones/{zone_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Asset,google/cloud/dataplex/v1/resources.proto#L514}
  ///
  /// [google.cloud.dataplex.v1.ListAssetsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L705}
  /// [google.cloud.dataplex.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L514}
  ///
  StreamRange<google::cloud::dataplex::v1::Asset> ListAssets(
      std::string const& parent, Options opts = {});

  ///
  /// Lists asset resources in a zone.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListAssetsRequest,google/cloud/dataplex/v1/service.proto#L705}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Asset,google/cloud/dataplex/v1/resources.proto#L514}
  ///
  /// [google.cloud.dataplex.v1.ListAssetsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L705}
  /// [google.cloud.dataplex.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L514}
  ///
  StreamRange<google::cloud::dataplex::v1::Asset> ListAssets(
      google::cloud::dataplex::v1::ListAssetsRequest request,
      Options opts = {});

  ///
  /// Retrieves an asset resource.
  ///
  /// @param name  Required. The resource name of the asset:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/zones/{zone_id}/assets/{asset_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Asset,google/cloud/dataplex/v1/resources.proto#L514}
  ///
  /// [google.cloud.dataplex.v1.GetAssetRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L767}
  /// [google.cloud.dataplex.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L514}
  ///
  StatusOr<google::cloud::dataplex::v1::Asset> GetAsset(std::string const& name,
                                                        Options opts = {});

  ///
  /// Retrieves an asset resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::GetAssetRequest,google/cloud/dataplex/v1/service.proto#L767}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Asset,google/cloud/dataplex/v1/resources.proto#L514}
  ///
  /// [google.cloud.dataplex.v1.GetAssetRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L767}
  /// [google.cloud.dataplex.v1.Asset]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L514}
  ///
  StatusOr<google::cloud::dataplex::v1::Asset> GetAsset(
      google::cloud::dataplex::v1::GetAssetRequest const& request,
      Options opts = {});

  ///
  /// Lists action resources in an asset.
  ///
  /// @param parent  Required. The resource name of the parent asset:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/zones/{zone_id}/assets/{asset_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Action,google/cloud/dataplex/v1/resources.proto#L308}
  ///
  /// [google.cloud.dataplex.v1.ListAssetActionsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L744}
  /// [google.cloud.dataplex.v1.Action]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L308}
  ///
  StreamRange<google::cloud::dataplex::v1::Action> ListAssetActions(
      std::string const& parent, Options opts = {});

  ///
  /// Lists action resources in an asset.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListAssetActionsRequest,google/cloud/dataplex/v1/service.proto#L744}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Action,google/cloud/dataplex/v1/resources.proto#L308}
  ///
  /// [google.cloud.dataplex.v1.ListAssetActionsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L744}
  /// [google.cloud.dataplex.v1.Action]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/resources.proto#L308}
  ///
  StreamRange<google::cloud::dataplex::v1::Action> ListAssetActions(
      google::cloud::dataplex::v1::ListAssetActionsRequest request,
      Options opts = {});

  ///
  /// Creates a task resource within a lake.
  ///
  /// @param parent  Required. The resource name of the parent lake:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}`.
  /// @param task  Required. Task resource.
  /// @param task_id  Required. Task identifier.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Task,google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  /// [google.cloud.dataplex.v1.CreateTaskRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L806}
  /// [google.cloud.dataplex.v1.Task]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Task>> CreateTask(
      std::string const& parent, google::cloud::dataplex::v1::Task const& task,
      std::string const& task_id, Options opts = {});

  ///
  /// Creates a task resource within a lake.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::CreateTaskRequest,google/cloud/dataplex/v1/service.proto#L806}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Task,google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  /// [google.cloud.dataplex.v1.CreateTaskRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L806}
  /// [google.cloud.dataplex.v1.Task]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Task>> CreateTask(
      google::cloud::dataplex::v1::CreateTaskRequest const& request,
      Options opts = {});

  ///
  /// Update the task resource.
  ///
  /// @param task  Required. Update description.
  ///  Only fields specified in `update_mask` are updated.
  /// @param update_mask  Required. Mask of fields to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Task,google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  /// [google.cloud.dataplex.v1.UpdateTaskRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L828}
  /// [google.cloud.dataplex.v1.Task]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Task>> UpdateTask(
      google::cloud::dataplex::v1::Task const& task,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Update the task resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::UpdateTaskRequest,google/cloud/dataplex/v1/service.proto#L828}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Task,google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  /// [google.cloud.dataplex.v1.UpdateTaskRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L828}
  /// [google.cloud.dataplex.v1.Task]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Task>> UpdateTask(
      google::cloud::dataplex::v1::UpdateTaskRequest const& request,
      Options opts = {});

  ///
  /// Delete the task resource.
  ///
  /// @param name  Required. The resource name of the task:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/task/{task_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteTaskRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L842}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>> DeleteTask(
      std::string const& name, Options opts = {});

  ///
  /// Delete the task resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::DeleteTaskRequest,google/cloud/dataplex/v1/service.proto#L842}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteTaskRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L842}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>> DeleteTask(
      google::cloud::dataplex::v1::DeleteTaskRequest const& request,
      Options opts = {});

  ///
  /// Lists tasks under the given lake.
  ///
  /// @param parent  Required. The resource name of the parent lake:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Task,google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  /// [google.cloud.dataplex.v1.ListTasksRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L854}
  /// [google.cloud.dataplex.v1.Task]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  StreamRange<google::cloud::dataplex::v1::Task> ListTasks(
      std::string const& parent, Options opts = {});

  ///
  /// Lists tasks under the given lake.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListTasksRequest,google/cloud/dataplex/v1/service.proto#L854}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Task,google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  /// [google.cloud.dataplex.v1.ListTasksRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L854}
  /// [google.cloud.dataplex.v1.Task]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  StreamRange<google::cloud::dataplex::v1::Task> ListTasks(
      google::cloud::dataplex::v1::ListTasksRequest request, Options opts = {});

  ///
  /// Get task resource.
  ///
  /// @param name  Required. The resource name of the task:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/tasks/{tasks_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Task,google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  /// [google.cloud.dataplex.v1.GetTaskRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L895}
  /// [google.cloud.dataplex.v1.Task]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  StatusOr<google::cloud::dataplex::v1::Task> GetTask(std::string const& name,
                                                      Options opts = {});

  ///
  /// Get task resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::GetTaskRequest,google/cloud/dataplex/v1/service.proto#L895}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Task,google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  /// [google.cloud.dataplex.v1.GetTaskRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L895}
  /// [google.cloud.dataplex.v1.Task]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L31}
  ///
  StatusOr<google::cloud::dataplex::v1::Task> GetTask(
      google::cloud::dataplex::v1::GetTaskRequest const& request,
      Options opts = {});

  ///
  /// Lists Jobs under the given task.
  ///
  /// @param parent  Required. The resource name of the parent environment:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/tasks/{task_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Job,google/cloud/dataplex/v1/tasks.proto#L323}
  ///
  /// [google.cloud.dataplex.v1.ListJobsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L919}
  /// [google.cloud.dataplex.v1.Job]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L323}
  ///
  StreamRange<google::cloud::dataplex::v1::Job> ListJobs(
      std::string const& parent, Options opts = {});

  ///
  /// Lists Jobs under the given task.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListJobsRequest,google/cloud/dataplex/v1/service.proto#L919}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Job,google/cloud/dataplex/v1/tasks.proto#L323}
  ///
  /// [google.cloud.dataplex.v1.ListJobsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L919}
  /// [google.cloud.dataplex.v1.Job]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L323}
  ///
  StreamRange<google::cloud::dataplex::v1::Job> ListJobs(
      google::cloud::dataplex::v1::ListJobsRequest request, Options opts = {});

  ///
  /// Get job resource.
  ///
  /// @param name  Required. The resource name of the job:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/tasks/{task_id}/jobs/{job_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Job,google/cloud/dataplex/v1/tasks.proto#L323}
  ///
  /// [google.cloud.dataplex.v1.GetJobRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L907}
  /// [google.cloud.dataplex.v1.Job]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L323}
  ///
  StatusOr<google::cloud::dataplex::v1::Job> GetJob(std::string const& name,
                                                    Options opts = {});

  ///
  /// Get job resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::GetJobRequest,google/cloud/dataplex/v1/service.proto#L907}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Job,google/cloud/dataplex/v1/tasks.proto#L323}
  ///
  /// [google.cloud.dataplex.v1.GetJobRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L907}
  /// [google.cloud.dataplex.v1.Job]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/tasks.proto#L323}
  ///
  StatusOr<google::cloud::dataplex::v1::Job> GetJob(
      google::cloud::dataplex::v1::GetJobRequest const& request,
      Options opts = {});

  ///
  /// Cancel jobs running for the task resource.
  ///
  /// @param name  Required. The resource name of the job:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/task/{task_id}/job/{job_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dataplex.v1.CancelJobRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L952}
  ///
  Status CancelJob(std::string const& name, Options opts = {});

  ///
  /// Cancel jobs running for the task resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::CancelJobRequest,google/cloud/dataplex/v1/service.proto#L952}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dataplex.v1.CancelJobRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L952}
  ///
  Status CancelJob(google::cloud::dataplex::v1::CancelJobRequest const& request,
                   Options opts = {});

  ///
  /// Create an environment resource.
  ///
  /// @param parent  Required. The resource name of the parent lake:
  ///  `projects/{project_id}/locations/{location_id}/lakes/{lake_id}`.
  /// @param environment  Required. Environment resource.
  /// @param environment_id  Required. Environment identifier.
  ///  * Must contain only lowercase letters, numbers and hyphens.
  ///  * Must start with a letter.
  ///  * Must be between 1-63 characters.
  ///  * Must end with a number or a letter.
  ///  * Must be unique within the lake.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Environment,google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  /// [google.cloud.dataplex.v1.CreateEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L964}
  /// [google.cloud.dataplex.v1.Environment]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Environment>> CreateEnvironment(
      std::string const& parent,
      google::cloud::dataplex::v1::Environment const& environment,
      std::string const& environment_id, Options opts = {});

  ///
  /// Create an environment resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::CreateEnvironmentRequest,google/cloud/dataplex/v1/service.proto#L964}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Environment,google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  /// [google.cloud.dataplex.v1.CreateEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L964}
  /// [google.cloud.dataplex.v1.Environment]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Environment>> CreateEnvironment(
      google::cloud::dataplex::v1::CreateEnvironmentRequest const& request,
      Options opts = {});

  ///
  /// Update the environment resource.
  ///
  /// @param environment  Required. Update description.
  ///  Only fields specified in `update_mask` are updated.
  /// @param update_mask  Required. Mask of fields to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Environment,google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  /// [google.cloud.dataplex.v1.UpdateEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L991}
  /// [google.cloud.dataplex.v1.Environment]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Environment>> UpdateEnvironment(
      google::cloud::dataplex::v1::Environment const& environment,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Update the environment resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::UpdateEnvironmentRequest,google/cloud/dataplex/v1/service.proto#L991}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Environment,google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  /// [google.cloud.dataplex.v1.UpdateEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L991}
  /// [google.cloud.dataplex.v1.Environment]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  future<StatusOr<google::cloud::dataplex::v1::Environment>> UpdateEnvironment(
      google::cloud::dataplex::v1::UpdateEnvironmentRequest const& request,
      Options opts = {});

  ///
  /// Delete the environment resource. All the child resources must have been
  /// deleted before environment deletion can be initiated.
  ///
  /// @param name  Required. The resource name of the environment:
  ///  `projects/{project_id}/locations/{location_id}/lakes/{lake_id}/environments/{environment_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L1005}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>>
  DeleteEnvironment(std::string const& name, Options opts = {});

  ///
  /// Delete the environment resource. All the child resources must have been
  /// deleted before environment deletion can be initiated.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::DeleteEnvironmentRequest,google/cloud/dataplex/v1/service.proto#L1005}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::OperationMetadata,google/cloud/dataplex/v1/service.proto#L779}
  ///
  /// [google.cloud.dataplex.v1.DeleteEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L1005}
  /// [google.cloud.dataplex.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L779}
  ///
  future<StatusOr<google::cloud::dataplex::v1::OperationMetadata>>
  DeleteEnvironment(
      google::cloud::dataplex::v1::DeleteEnvironmentRequest const& request,
      Options opts = {});

  ///
  /// Lists environments under the given lake.
  ///
  /// @param parent  Required. The resource name of the parent lake:
  ///  `projects/{project_id}/locations/{location_id}/lakes/{lake_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Environment,google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  /// [google.cloud.dataplex.v1.ListEnvironmentsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L1017}
  /// [google.cloud.dataplex.v1.Environment]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  StreamRange<google::cloud::dataplex::v1::Environment> ListEnvironments(
      std::string const& parent, Options opts = {});

  ///
  /// Lists environments under the given lake.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListEnvironmentsRequest,google/cloud/dataplex/v1/service.proto#L1017}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Environment,google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  /// [google.cloud.dataplex.v1.ListEnvironmentsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L1017}
  /// [google.cloud.dataplex.v1.Environment]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  StreamRange<google::cloud::dataplex::v1::Environment> ListEnvironments(
      google::cloud::dataplex::v1::ListEnvironmentsRequest request,
      Options opts = {});

  ///
  /// Get environment resource.
  ///
  /// @param name  Required. The resource name of the environment:
  ///  `projects/{project_id}/locations/{location_id}/lakes/{lake_id}/environments/{environment_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Environment,google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  /// [google.cloud.dataplex.v1.GetEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L1056}
  /// [google.cloud.dataplex.v1.Environment]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  StatusOr<google::cloud::dataplex::v1::Environment> GetEnvironment(
      std::string const& name, Options opts = {});

  ///
  /// Get environment resource.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::GetEnvironmentRequest,google/cloud/dataplex/v1/service.proto#L1056}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Environment,google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  /// [google.cloud.dataplex.v1.GetEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L1056}
  /// [google.cloud.dataplex.v1.Environment]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L32}
  ///
  StatusOr<google::cloud::dataplex::v1::Environment> GetEnvironment(
      google::cloud::dataplex::v1::GetEnvironmentRequest const& request,
      Options opts = {});

  ///
  /// Lists session resources in an environment.
  ///
  /// @param parent  Required. The resource name of the parent environment:
  ///  `projects/{project_number}/locations/{location_id}/lakes/{lake_id}/environment/{environment_id}`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Session,google/cloud/dataplex/v1/analyze.proto#L242}
  ///
  /// [google.cloud.dataplex.v1.ListSessionsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L1068}
  /// [google.cloud.dataplex.v1.Session]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L242}
  ///
  StreamRange<google::cloud::dataplex::v1::Session> ListSessions(
      std::string const& parent, Options opts = {});

  ///
  /// Lists session resources in an environment.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dataplex::v1::ListSessionsRequest,google/cloud/dataplex/v1/service.proto#L1068}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dataplex::v1::Session,google/cloud/dataplex/v1/analyze.proto#L242}
  ///
  /// [google.cloud.dataplex.v1.ListSessionsRequest]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/service.proto#L1068}
  /// [google.cloud.dataplex.v1.Session]:
  /// @googleapis_reference_link{google/cloud/dataplex/v1/analyze.proto#L242}
  ///
  StreamRange<google::cloud::dataplex::v1::Session> ListSessions(
      google::cloud::dataplex::v1::ListSessionsRequest request,
      Options opts = {});

 private:
  std::shared_ptr<DataplexServiceConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dataplex
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATAPLEX_DATAPLEX_CLIENT_H
