// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/datastream/v1/datastream.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATASTREAM_DATASTREAM_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATASTREAM_DATASTREAM_CLIENT_H

#include "google/cloud/datastream/datastream_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace datastream {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Datastream service
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class DatastreamClient {
 public:
  explicit DatastreamClient(std::shared_ptr<DatastreamConnection> connection,
                            Options opts = {});
  ~DatastreamClient();

  ///@{
  // @name Copy and move support
  DatastreamClient(DatastreamClient const&) = default;
  DatastreamClient& operator=(DatastreamClient const&) = default;
  DatastreamClient(DatastreamClient&&) = default;
  DatastreamClient& operator=(DatastreamClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(DatastreamClient const& a, DatastreamClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(DatastreamClient const& a, DatastreamClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Use this method to list connection profiles created in a project and
  /// location.
  ///
  /// @param parent  Required. The parent that owns the collection of connection
  /// profiles.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::ConnectionProfile,google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  /// [google.cloud.datastream.v1.ListConnectionProfilesRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L386}
  /// [google.cloud.datastream.v1.ConnectionProfile]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  StreamRange<google::cloud::datastream::v1::ConnectionProfile>
  ListConnectionProfiles(std::string const& parent, Options opts = {});

  ///
  /// Use this method to list connection profiles created in a project and
  /// location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::ListConnectionProfilesRequest,google/cloud/datastream/v1/datastream.proto#L386}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::ConnectionProfile,google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  /// [google.cloud.datastream.v1.ListConnectionProfilesRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L386}
  /// [google.cloud.datastream.v1.ConnectionProfile]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  StreamRange<google::cloud::datastream::v1::ConnectionProfile>
  ListConnectionProfiles(
      google::cloud::datastream::v1::ListConnectionProfilesRequest request,
      Options opts = {});

  ///
  /// Use this method to get details about a connection profile.
  ///
  /// @param name  Required. The name of the connection profile resource to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::ConnectionProfile,google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  /// [google.cloud.datastream.v1.GetConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L428}
  /// [google.cloud.datastream.v1.ConnectionProfile]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  StatusOr<google::cloud::datastream::v1::ConnectionProfile>
  GetConnectionProfile(std::string const& name, Options opts = {});

  ///
  /// Use this method to get details about a connection profile.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::GetConnectionProfileRequest,google/cloud/datastream/v1/datastream.proto#L428}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::ConnectionProfile,google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  /// [google.cloud.datastream.v1.GetConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L428}
  /// [google.cloud.datastream.v1.ConnectionProfile]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  StatusOr<google::cloud::datastream::v1::ConnectionProfile>
  GetConnectionProfile(
      google::cloud::datastream::v1::GetConnectionProfileRequest const& request,
      Options opts = {});

  ///
  /// Use this method to create a connection profile in a project and location.
  ///
  /// @param parent  Required. The parent that owns the collection of
  /// ConnectionProfiles.
  /// @param connection_profile  Required. The connection profile resource to
  /// create.
  /// @param connection_profile_id  Required. The connection profile identifier.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::ConnectionProfile,google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  /// [google.cloud.datastream.v1.CreateConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L439}
  /// [google.cloud.datastream.v1.ConnectionProfile]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  future<StatusOr<google::cloud::datastream::v1::ConnectionProfile>>
  CreateConnectionProfile(
      std::string const& parent,
      google::cloud::datastream::v1::ConnectionProfile const&
          connection_profile,
      std::string const& connection_profile_id, Options opts = {});

  ///
  /// Use this method to create a connection profile in a project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::CreateConnectionProfileRequest,google/cloud/datastream/v1/datastream.proto#L439}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::ConnectionProfile,google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  /// [google.cloud.datastream.v1.CreateConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L439}
  /// [google.cloud.datastream.v1.ConnectionProfile]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  future<StatusOr<google::cloud::datastream::v1::ConnectionProfile>>
  CreateConnectionProfile(
      google::cloud::datastream::v1::CreateConnectionProfileRequest const&
          request,
      Options opts = {});

  ///
  /// Use this method to update the parameters of a connection profile.
  ///
  /// @param connection_profile  Required. The connection profile to update.
  /// @param update_mask  Optional. Field mask is used to specify the fields to
  /// be overwritten in the
  ///  ConnectionProfile resource by the update.
  ///  The fields specified in the update_mask are relative to the resource, not
  ///  the full request. A field will be overwritten if it is in the mask. If
  ///  the user does not provide a mask then all fields will be overwritten.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::ConnectionProfile,google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  /// [google.cloud.datastream.v1.UpdateConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L478}
  /// [google.cloud.datastream.v1.ConnectionProfile]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  future<StatusOr<google::cloud::datastream::v1::ConnectionProfile>>
  UpdateConnectionProfile(
      google::cloud::datastream::v1::ConnectionProfile const&
          connection_profile,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Use this method to update the parameters of a connection profile.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::UpdateConnectionProfileRequest,google/cloud/datastream/v1/datastream.proto#L478}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::ConnectionProfile,google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  /// [google.cloud.datastream.v1.UpdateConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L478}
  /// [google.cloud.datastream.v1.ConnectionProfile]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L275}
  ///
  future<StatusOr<google::cloud::datastream::v1::ConnectionProfile>>
  UpdateConnectionProfile(
      google::cloud::datastream::v1::UpdateConnectionProfileRequest const&
          request,
      Options opts = {});

  ///
  /// Use this method to delete a connection profile.
  ///
  /// @param name  Required. The name of the connection profile resource to
  /// delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::OperationMetadata,google/cloud/datastream/v1/datastream.proto#L785}
  ///
  /// [google.cloud.datastream.v1.DeleteConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L513}
  /// [google.cloud.datastream.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L785}
  ///
  future<StatusOr<google::cloud::datastream::v1::OperationMetadata>>
  DeleteConnectionProfile(std::string const& name, Options opts = {});

  ///
  /// Use this method to delete a connection profile.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::DeleteConnectionProfileRequest,google/cloud/datastream/v1/datastream.proto#L513}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::OperationMetadata,google/cloud/datastream/v1/datastream.proto#L785}
  ///
  /// [google.cloud.datastream.v1.DeleteConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L513}
  /// [google.cloud.datastream.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L785}
  ///
  future<StatusOr<google::cloud::datastream::v1::OperationMetadata>>
  DeleteConnectionProfile(
      google::cloud::datastream::v1::DeleteConnectionProfileRequest const&
          request,
      Options opts = {});

  ///
  /// Use this method to discover a connection profile.
  /// The discover API call exposes the data objects and metadata belonging to
  /// the profile. Typically, a request returns children data objects of a
  /// parent data object that's optionally supplied in the request.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::DiscoverConnectionProfileRequest,google/cloud/datastream/v1/datastream.proto#L299}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::DiscoverConnectionProfileResponse,google/cloud/datastream/v1/datastream.proto#L342}
  ///
  /// [google.cloud.datastream.v1.DiscoverConnectionProfileRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L299}
  /// [google.cloud.datastream.v1.DiscoverConnectionProfileResponse]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L342}
  ///
  StatusOr<google::cloud::datastream::v1::DiscoverConnectionProfileResponse>
  DiscoverConnectionProfile(
      google::cloud::datastream::v1::DiscoverConnectionProfileRequest const&
          request,
      Options opts = {});

  ///
  /// Use this method to list streams in a project and location.
  ///
  /// @param parent  Required. The parent that owns the collection of streams.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Stream,google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  /// [google.cloud.datastream.v1.ListStreamsRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L539}
  /// [google.cloud.datastream.v1.Stream]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  StreamRange<google::cloud::datastream::v1::Stream> ListStreams(
      std::string const& parent, Options opts = {});

  ///
  /// Use this method to list streams in a project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::ListStreamsRequest,google/cloud/datastream/v1/datastream.proto#L539}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Stream,google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  /// [google.cloud.datastream.v1.ListStreamsRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L539}
  /// [google.cloud.datastream.v1.Stream]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  StreamRange<google::cloud::datastream::v1::Stream> ListStreams(
      google::cloud::datastream::v1::ListStreamsRequest request,
      Options opts = {});

  ///
  /// Use this method to get details about a stream.
  ///
  /// @param name  Required. The name of the stream resource to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Stream,google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  /// [google.cloud.datastream.v1.GetStreamRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L581}
  /// [google.cloud.datastream.v1.Stream]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  StatusOr<google::cloud::datastream::v1::Stream> GetStream(
      std::string const& name, Options opts = {});

  ///
  /// Use this method to get details about a stream.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::GetStreamRequest,google/cloud/datastream/v1/datastream.proto#L581}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Stream,google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  /// [google.cloud.datastream.v1.GetStreamRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L581}
  /// [google.cloud.datastream.v1.Stream]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  StatusOr<google::cloud::datastream::v1::Stream> GetStream(
      google::cloud::datastream::v1::GetStreamRequest const& request,
      Options opts = {});

  ///
  /// Use this method to create a stream.
  ///
  /// @param parent  Required. The parent that owns the collection of streams.
  /// @param stream  Required. The stream resource to create.
  /// @param stream_id  Required. The stream identifier.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Stream,google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  /// [google.cloud.datastream.v1.CreateStreamRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L592}
  /// [google.cloud.datastream.v1.Stream]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  future<StatusOr<google::cloud::datastream::v1::Stream>> CreateStream(
      std::string const& parent,
      google::cloud::datastream::v1::Stream const& stream,
      std::string const& stream_id, Options opts = {});

  ///
  /// Use this method to create a stream.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::CreateStreamRequest,google/cloud/datastream/v1/datastream.proto#L592}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Stream,google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  /// [google.cloud.datastream.v1.CreateStreamRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L592}
  /// [google.cloud.datastream.v1.Stream]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  future<StatusOr<google::cloud::datastream::v1::Stream>> CreateStream(
      google::cloud::datastream::v1::CreateStreamRequest const& request,
      Options opts = {});

  ///
  /// Use this method to update the configuration of a stream.
  ///
  /// @param stream  Required. The stream resource to update.
  /// @param update_mask  Optional. Field mask is used to specify the fields to
  /// be overwritten in the
  ///  stream resource by the update.
  ///  The fields specified in the update_mask are relative to the resource, not
  ///  the full request. A field will be overwritten if it is in the mask. If
  ///  the user does not provide a mask then all fields will be overwritten.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Stream,google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  /// [google.cloud.datastream.v1.UpdateStreamRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L631}
  /// [google.cloud.datastream.v1.Stream]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  future<StatusOr<google::cloud::datastream::v1::Stream>> UpdateStream(
      google::cloud::datastream::v1::Stream const& stream,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Use this method to update the configuration of a stream.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::UpdateStreamRequest,google/cloud/datastream/v1/datastream.proto#L631}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Stream,google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  /// [google.cloud.datastream.v1.UpdateStreamRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L631}
  /// [google.cloud.datastream.v1.Stream]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L705}
  ///
  future<StatusOr<google::cloud::datastream::v1::Stream>> UpdateStream(
      google::cloud::datastream::v1::UpdateStreamRequest const& request,
      Options opts = {});

  ///
  /// Use this method to delete a stream.
  ///
  /// @param name  Required. The name of the stream resource to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::OperationMetadata,google/cloud/datastream/v1/datastream.proto#L785}
  ///
  /// [google.cloud.datastream.v1.DeleteStreamRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L666}
  /// [google.cloud.datastream.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L785}
  ///
  future<StatusOr<google::cloud::datastream::v1::OperationMetadata>>
  DeleteStream(std::string const& name, Options opts = {});

  ///
  /// Use this method to delete a stream.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::DeleteStreamRequest,google/cloud/datastream/v1/datastream.proto#L666}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::OperationMetadata,google/cloud/datastream/v1/datastream.proto#L785}
  ///
  /// [google.cloud.datastream.v1.DeleteStreamRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L666}
  /// [google.cloud.datastream.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L785}
  ///
  future<StatusOr<google::cloud::datastream::v1::OperationMetadata>>
  DeleteStream(
      google::cloud::datastream::v1::DeleteStreamRequest const& request,
      Options opts = {});

  ///
  /// Use this method to get details about a stream object.
  ///
  /// @param name  Required. The name of the stream object resource to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StreamObject,google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  /// [google.cloud.datastream.v1.GetStreamObjectRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L692}
  /// [google.cloud.datastream.v1.StreamObject]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  StatusOr<google::cloud::datastream::v1::StreamObject> GetStreamObject(
      std::string const& name, Options opts = {});

  ///
  /// Use this method to get details about a stream object.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::GetStreamObjectRequest,google/cloud/datastream/v1/datastream.proto#L692}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StreamObject,google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  /// [google.cloud.datastream.v1.GetStreamObjectRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L692}
  /// [google.cloud.datastream.v1.StreamObject]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  StatusOr<google::cloud::datastream::v1::StreamObject> GetStreamObject(
      google::cloud::datastream::v1::GetStreamObjectRequest const& request,
      Options opts = {});

  ///
  /// Use this method to look up a stream object by its source object
  /// identifier.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::LookupStreamObjectRequest,google/cloud/datastream/v1/datastream.proto#L704}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StreamObject,google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  /// [google.cloud.datastream.v1.LookupStreamObjectRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L704}
  /// [google.cloud.datastream.v1.StreamObject]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  StatusOr<google::cloud::datastream::v1::StreamObject> LookupStreamObject(
      google::cloud::datastream::v1::LookupStreamObjectRequest const& request,
      Options opts = {});

  ///
  /// Use this method to list the objects of a specific stream.
  ///
  /// @param parent  Required. The parent stream that owns the collection of
  /// objects.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StreamObject,google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  /// [google.cloud.datastream.v1.ListStreamObjectsRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L753}
  /// [google.cloud.datastream.v1.StreamObject]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  StreamRange<google::cloud::datastream::v1::StreamObject> ListStreamObjects(
      std::string const& parent, Options opts = {});

  ///
  /// Use this method to list the objects of a specific stream.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::ListStreamObjectsRequest,google/cloud/datastream/v1/datastream.proto#L753}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StreamObject,google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  /// [google.cloud.datastream.v1.ListStreamObjectsRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L753}
  /// [google.cloud.datastream.v1.StreamObject]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L811}
  ///
  StreamRange<google::cloud::datastream::v1::StreamObject> ListStreamObjects(
      google::cloud::datastream::v1::ListStreamObjectsRequest request,
      Options opts = {});

  ///
  /// Use this method to start a backfill job for the specified stream object.
  ///
  /// @param object  Required. The name of the stream object resource to start a
  /// backfill job for.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StartBackfillJobResponse,google/cloud/datastream/v1/datastream.proto#L729}
  ///
  /// [google.cloud.datastream.v1.StartBackfillJobRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L718}
  /// [google.cloud.datastream.v1.StartBackfillJobResponse]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L729}
  ///
  StatusOr<google::cloud::datastream::v1::StartBackfillJobResponse>
  StartBackfillJob(std::string const& object, Options opts = {});

  ///
  /// Use this method to start a backfill job for the specified stream object.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::StartBackfillJobRequest,google/cloud/datastream/v1/datastream.proto#L718}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StartBackfillJobResponse,google/cloud/datastream/v1/datastream.proto#L729}
  ///
  /// [google.cloud.datastream.v1.StartBackfillJobRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L718}
  /// [google.cloud.datastream.v1.StartBackfillJobResponse]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L729}
  ///
  StatusOr<google::cloud::datastream::v1::StartBackfillJobResponse>
  StartBackfillJob(
      google::cloud::datastream::v1::StartBackfillJobRequest const& request,
      Options opts = {});

  ///
  /// Use this method to stop a backfill job for the specified stream object.
  ///
  /// @param object  Required. The name of the stream object resource to stop
  /// the backfill job for.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StopBackfillJobResponse,google/cloud/datastream/v1/datastream.proto#L747}
  ///
  /// [google.cloud.datastream.v1.StopBackfillJobRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L736}
  /// [google.cloud.datastream.v1.StopBackfillJobResponse]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L747}
  ///
  StatusOr<google::cloud::datastream::v1::StopBackfillJobResponse>
  StopBackfillJob(std::string const& object, Options opts = {});

  ///
  /// Use this method to stop a backfill job for the specified stream object.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::StopBackfillJobRequest,google/cloud/datastream/v1/datastream.proto#L736}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::StopBackfillJobResponse,google/cloud/datastream/v1/datastream.proto#L747}
  ///
  /// [google.cloud.datastream.v1.StopBackfillJobRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L736}
  /// [google.cloud.datastream.v1.StopBackfillJobResponse]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L747}
  ///
  StatusOr<google::cloud::datastream::v1::StopBackfillJobResponse>
  StopBackfillJob(
      google::cloud::datastream::v1::StopBackfillJobRequest const& request,
      Options opts = {});

  ///
  /// The FetchStaticIps API call exposes the static IP addresses used by
  /// Datastream.
  ///
  /// @param name  Required. The resource name for the location for which static
  /// IPs should be returned.
  ///  Must be in the format `projects/*/locations/*`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return std::string
  ///
  /// [google.cloud.datastream.v1.FetchStaticIpsRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L357}
  ///
  StreamRange<std::string> FetchStaticIps(std::string const& name,
                                          Options opts = {});

  ///
  /// The FetchStaticIps API call exposes the static IP addresses used by
  /// Datastream.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::FetchStaticIpsRequest,google/cloud/datastream/v1/datastream.proto#L357}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return std::string
  ///
  /// [google.cloud.datastream.v1.FetchStaticIpsRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L357}
  ///
  StreamRange<std::string> FetchStaticIps(
      google::cloud::datastream::v1::FetchStaticIpsRequest request,
      Options opts = {});

  ///
  /// Use this method to create a private connectivity configuration.
  ///
  /// @param parent  Required. The parent that owns the collection of
  /// PrivateConnections.
  /// @param private_connection  Required. The Private Connectivity resource to
  /// create.
  /// @param private_connection_id  Required. The private connectivity
  /// identifier.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::PrivateConnection,google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  /// [google.cloud.datastream.v1.CreatePrivateConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L815}
  /// [google.cloud.datastream.v1.PrivateConnection]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  future<StatusOr<google::cloud::datastream::v1::PrivateConnection>>
  CreatePrivateConnection(
      std::string const& parent,
      google::cloud::datastream::v1::PrivateConnection const&
          private_connection,
      std::string const& private_connection_id, Options opts = {});

  ///
  /// Use this method to create a private connectivity configuration.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::CreatePrivateConnectionRequest,google/cloud/datastream/v1/datastream.proto#L815}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::PrivateConnection,google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  /// [google.cloud.datastream.v1.CreatePrivateConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L815}
  /// [google.cloud.datastream.v1.PrivateConnection]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  future<StatusOr<google::cloud::datastream::v1::PrivateConnection>>
  CreatePrivateConnection(
      google::cloud::datastream::v1::CreatePrivateConnectionRequest const&
          request,
      Options opts = {});

  ///
  /// Use this method to get details about a private connectivity configuration.
  ///
  /// @param name  Required. The name of the  private connectivity configuration
  /// to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::PrivateConnection,google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  /// [google.cloud.datastream.v1.GetPrivateConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L921}
  /// [google.cloud.datastream.v1.PrivateConnection]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  StatusOr<google::cloud::datastream::v1::PrivateConnection>
  GetPrivateConnection(std::string const& name, Options opts = {});

  ///
  /// Use this method to get details about a private connectivity configuration.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::GetPrivateConnectionRequest,google/cloud/datastream/v1/datastream.proto#L921}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::PrivateConnection,google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  /// [google.cloud.datastream.v1.GetPrivateConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L921}
  /// [google.cloud.datastream.v1.PrivateConnection]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  StatusOr<google::cloud::datastream::v1::PrivateConnection>
  GetPrivateConnection(
      google::cloud::datastream::v1::GetPrivateConnectionRequest const& request,
      Options opts = {});

  ///
  /// Use this method to list private connectivity configurations in a project
  /// and location.
  ///
  /// @param parent  Required. The parent that owns the collection of private
  /// connectivity configurations.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::PrivateConnection,google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  /// [google.cloud.datastream.v1.ListPrivateConnectionsRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L847}
  /// [google.cloud.datastream.v1.PrivateConnection]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  StreamRange<google::cloud::datastream::v1::PrivateConnection>
  ListPrivateConnections(std::string const& parent, Options opts = {});

  ///
  /// Use this method to list private connectivity configurations in a project
  /// and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::ListPrivateConnectionsRequest,google/cloud/datastream/v1/datastream.proto#L847}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::PrivateConnection,google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  /// [google.cloud.datastream.v1.ListPrivateConnectionsRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L847}
  /// [google.cloud.datastream.v1.PrivateConnection]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L153}
  ///
  StreamRange<google::cloud::datastream::v1::PrivateConnection>
  ListPrivateConnections(
      google::cloud::datastream::v1::ListPrivateConnectionsRequest request,
      Options opts = {});

  ///
  /// Use this method to delete a private connectivity configuration.
  ///
  /// @param name  Required. The name of the private connectivity configuration
  /// to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::OperationMetadata,google/cloud/datastream/v1/datastream.proto#L785}
  ///
  /// [google.cloud.datastream.v1.DeletePrivateConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L891}
  /// [google.cloud.datastream.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L785}
  ///
  future<StatusOr<google::cloud::datastream::v1::OperationMetadata>>
  DeletePrivateConnection(std::string const& name, Options opts = {});

  ///
  /// Use this method to delete a private connectivity configuration.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::DeletePrivateConnectionRequest,google/cloud/datastream/v1/datastream.proto#L891}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::OperationMetadata,google/cloud/datastream/v1/datastream.proto#L785}
  ///
  /// [google.cloud.datastream.v1.DeletePrivateConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L891}
  /// [google.cloud.datastream.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L785}
  ///
  future<StatusOr<google::cloud::datastream::v1::OperationMetadata>>
  DeletePrivateConnection(
      google::cloud::datastream::v1::DeletePrivateConnectionRequest const&
          request,
      Options opts = {});

  ///
  /// Use this method to create a route for a private connectivity configuration
  /// in a project and location.
  ///
  /// @param parent  Required. The parent that owns the collection of Routes.
  /// @param route  Required. The Route resource to create.
  /// @param route_id  Required. The Route identifier.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Route,google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  /// [google.cloud.datastream.v1.CreateRouteRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L932}
  /// [google.cloud.datastream.v1.Route]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  future<StatusOr<google::cloud::datastream::v1::Route>> CreateRoute(
      std::string const& parent,
      google::cloud::datastream::v1::Route const& route,
      std::string const& route_id, Options opts = {});

  ///
  /// Use this method to create a route for a private connectivity configuration
  /// in a project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::CreateRouteRequest,google/cloud/datastream/v1/datastream.proto#L932}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Route,google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  /// [google.cloud.datastream.v1.CreateRouteRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L932}
  /// [google.cloud.datastream.v1.Route]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  future<StatusOr<google::cloud::datastream::v1::Route>> CreateRoute(
      google::cloud::datastream::v1::CreateRouteRequest const& request,
      Options opts = {});

  ///
  /// Use this method to get details about a route.
  ///
  /// @param name  Required. The name of the Route resource to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Route,google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  /// [google.cloud.datastream.v1.GetRouteRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L1034}
  /// [google.cloud.datastream.v1.Route]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  StatusOr<google::cloud::datastream::v1::Route> GetRoute(
      std::string const& name, Options opts = {});

  ///
  /// Use this method to get details about a route.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::GetRouteRequest,google/cloud/datastream/v1/datastream.proto#L1034}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Route,google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  /// [google.cloud.datastream.v1.GetRouteRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L1034}
  /// [google.cloud.datastream.v1.Route]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  StatusOr<google::cloud::datastream::v1::Route> GetRoute(
      google::cloud::datastream::v1::GetRouteRequest const& request,
      Options opts = {});

  ///
  /// Use this method to list routes created for a private connectivity
  /// configuration in a project and location.
  ///
  /// @param parent  Required. The parent that owns the collection of Routess.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Route,google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  /// [google.cloud.datastream.v1.ListRoutesRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L964}
  /// [google.cloud.datastream.v1.Route]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  StreamRange<google::cloud::datastream::v1::Route> ListRoutes(
      std::string const& parent, Options opts = {});

  ///
  /// Use this method to list routes created for a private connectivity
  /// configuration in a project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::ListRoutesRequest,google/cloud/datastream/v1/datastream.proto#L964}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::Route,google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  /// [google.cloud.datastream.v1.ListRoutesRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L964}
  /// [google.cloud.datastream.v1.Route]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream_resources.proto#L219}
  ///
  StreamRange<google::cloud::datastream::v1::Route> ListRoutes(
      google::cloud::datastream::v1::ListRoutesRequest request,
      Options opts = {});

  ///
  /// Use this method to delete a route.
  ///
  /// @param name  Required. The name of the Route resource to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::OperationMetadata,google/cloud/datastream/v1/datastream.proto#L785}
  ///
  /// [google.cloud.datastream.v1.DeleteRouteRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L1008}
  /// [google.cloud.datastream.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L785}
  ///
  future<StatusOr<google::cloud::datastream::v1::OperationMetadata>>
  DeleteRoute(std::string const& name, Options opts = {});

  ///
  /// Use this method to delete a route.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::datastream::v1::DeleteRouteRequest,google/cloud/datastream/v1/datastream.proto#L1008}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::datastream::v1::OperationMetadata,google/cloud/datastream/v1/datastream.proto#L785}
  ///
  /// [google.cloud.datastream.v1.DeleteRouteRequest]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L1008}
  /// [google.cloud.datastream.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/datastream/v1/datastream.proto#L785}
  ///
  future<StatusOr<google::cloud::datastream::v1::OperationMetadata>>
  DeleteRoute(google::cloud::datastream::v1::DeleteRouteRequest const& request,
              Options opts = {});

 private:
  std::shared_ptr<DatastreamConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace datastream
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DATASTREAM_DATASTREAM_CLIENT_H
