// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/cx/v3/deployment.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_DEPLOYMENTS_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_DEPLOYMENTS_CLIENT_H

#include "google/cloud/dialogflow_cx/deployments_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <memory>

namespace google {
namespace cloud {
namespace dialogflow_cx {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Service for managing
/// [Deployments][google.cloud.dialogflow.cx.v3.Deployment].
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class DeploymentsClient {
 public:
  explicit DeploymentsClient(std::shared_ptr<DeploymentsConnection> connection,
                             Options opts = {});
  ~DeploymentsClient();

  ///@{
  // @name Copy and move support
  DeploymentsClient(DeploymentsClient const&) = default;
  DeploymentsClient& operator=(DeploymentsClient const&) = default;
  DeploymentsClient(DeploymentsClient&&) = default;
  DeploymentsClient& operator=(DeploymentsClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(DeploymentsClient const& a,
                         DeploymentsClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(DeploymentsClient const& a,
                         DeploymentsClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Returns the list of all deployments in the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// @param parent  Required. The
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment] to list all
  /// environments for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>/environments/<Environment ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Deployment,google/cloud/dialogflow/cx/v3/deployment.proto#L62}
  ///
  /// [google.cloud.dialogflow.cx.v3.ListDeploymentsRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/deployment.proto#L126}
  /// [google.cloud.dialogflow.cx.v3.Deployment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/deployment.proto#L62}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::Deployment> ListDeployments(
      std::string const& parent, Options opts = {});

  ///
  /// Returns the list of all deployments in the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::ListDeploymentsRequest,google/cloud/dialogflow/cx/v3/deployment.proto#L126}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Deployment,google/cloud/dialogflow/cx/v3/deployment.proto#L62}
  ///
  /// [google.cloud.dialogflow.cx.v3.ListDeploymentsRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/deployment.proto#L126}
  /// [google.cloud.dialogflow.cx.v3.Deployment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/deployment.proto#L62}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::Deployment> ListDeployments(
      google::cloud::dialogflow::cx::v3::ListDeploymentsRequest request,
      Options opts = {});

  ///
  /// Retrieves the specified
  /// [Deployment][google.cloud.dialogflow.cx.v3.Deployment].
  ///
  /// @param name  Required. The name of the
  /// [Deployment][google.cloud.dialogflow.cx.v3.Deployment].
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>/environments/<Environment ID>/deployments/<Deployment ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Deployment,google/cloud/dialogflow/cx/v3/deployment.proto#L62}
  ///
  /// [google.cloud.dialogflow.cx.v3.GetDeploymentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/deployment.proto#L159}
  /// [google.cloud.dialogflow.cx.v3.Deployment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/deployment.proto#L62}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::Deployment> GetDeployment(
      std::string const& name, Options opts = {});

  ///
  /// Retrieves the specified
  /// [Deployment][google.cloud.dialogflow.cx.v3.Deployment].
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::GetDeploymentRequest,google/cloud/dialogflow/cx/v3/deployment.proto#L159}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Deployment,google/cloud/dialogflow/cx/v3/deployment.proto#L62}
  ///
  /// [google.cloud.dialogflow.cx.v3.GetDeploymentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/deployment.proto#L159}
  /// [google.cloud.dialogflow.cx.v3.Deployment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/deployment.proto#L62}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::Deployment> GetDeployment(
      google::cloud::dialogflow::cx::v3::GetDeploymentRequest const& request,
      Options opts = {});

 private:
  std::shared_ptr<DeploymentsConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_cx
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_DEPLOYMENTS_CLIENT_H
