// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/cx/v3/entity_type.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_ENTITY_TYPES_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_ENTITY_TYPES_CLIENT_H

#include "google/cloud/dialogflow_cx/entity_types_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <memory>

namespace google {
namespace cloud {
namespace dialogflow_cx {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Service for managing
/// [EntityTypes][google.cloud.dialogflow.cx.v3.EntityType].
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class EntityTypesClient {
 public:
  explicit EntityTypesClient(std::shared_ptr<EntityTypesConnection> connection,
                             Options opts = {});
  ~EntityTypesClient();

  ///@{
  // @name Copy and move support
  EntityTypesClient(EntityTypesClient const&) = default;
  EntityTypesClient& operator=(EntityTypesClient const&) = default;
  EntityTypesClient(EntityTypesClient&&) = default;
  EntityTypesClient& operator=(EntityTypesClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(EntityTypesClient const& a,
                         EntityTypesClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(EntityTypesClient const& a,
                         EntityTypesClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Returns the list of all entity types in the specified agent.
  ///
  /// @param parent  Required. The agent to list all entity types for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::EntityType,google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  /// [google.cloud.dialogflow.cx.v3.ListEntityTypesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L228}
  /// [google.cloud.dialogflow.cx.v3.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::EntityType> ListEntityTypes(
      std::string const& parent, Options opts = {});

  ///
  /// Returns the list of all entity types in the specified agent.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::ListEntityTypesRequest,google/cloud/dialogflow/cx/v3/entity_type.proto#L228}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::EntityType,google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  /// [google.cloud.dialogflow.cx.v3.ListEntityTypesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L228}
  /// [google.cloud.dialogflow.cx.v3.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::EntityType> ListEntityTypes(
      google::cloud::dialogflow::cx::v3::ListEntityTypesRequest request,
      Options opts = {});

  ///
  /// Retrieves the specified entity type.
  ///
  /// @param name  Required. The name of the entity type.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>/entityTypes/<Entity Type ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::EntityType,google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  /// [google.cloud.dialogflow.cx.v3.GetEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L272}
  /// [google.cloud.dialogflow.cx.v3.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::EntityType> GetEntityType(
      std::string const& name, Options opts = {});

  ///
  /// Retrieves the specified entity type.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::GetEntityTypeRequest,google/cloud/dialogflow/cx/v3/entity_type.proto#L272}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::EntityType,google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  /// [google.cloud.dialogflow.cx.v3.GetEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L272}
  /// [google.cloud.dialogflow.cx.v3.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::EntityType> GetEntityType(
      google::cloud::dialogflow::cx::v3::GetEntityTypeRequest const& request,
      Options opts = {});

  ///
  /// Creates an entity type in the specified agent.
  ///
  /// Note: You should always train a flow prior to sending it queries. See the
  /// [training
  /// documentation](https://cloud.google.com/dialogflow/cx/docs/concept/training).
  ///
  /// @param parent  Required. The agent to create a entity type for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>`.
  /// @param entity_type  Required. The entity type to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::EntityType,google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  /// [google.cloud.dialogflow.cx.v3.CreateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L299}
  /// [google.cloud.dialogflow.cx.v3.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::EntityType> CreateEntityType(
      std::string const& parent,
      google::cloud::dialogflow::cx::v3::EntityType const& entity_type,
      Options opts = {});

  ///
  /// Creates an entity type in the specified agent.
  ///
  /// Note: You should always train a flow prior to sending it queries. See the
  /// [training
  /// documentation](https://cloud.google.com/dialogflow/cx/docs/concept/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::CreateEntityTypeRequest,google/cloud/dialogflow/cx/v3/entity_type.proto#L299}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::EntityType,google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  /// [google.cloud.dialogflow.cx.v3.CreateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L299}
  /// [google.cloud.dialogflow.cx.v3.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::EntityType> CreateEntityType(
      google::cloud::dialogflow::cx::v3::CreateEntityTypeRequest const& request,
      Options opts = {});

  ///
  /// Updates the specified entity type.
  ///
  /// Note: You should always train a flow prior to sending it queries. See the
  /// [training
  /// documentation](https://cloud.google.com/dialogflow/cx/docs/concept/training).
  ///
  /// @param entity_type  Required. The entity type to update.
  /// @param update_mask  The mask to control which fields get updated.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::EntityType,google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  /// [google.cloud.dialogflow.cx.v3.UpdateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L327}
  /// [google.cloud.dialogflow.cx.v3.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::EntityType> UpdateEntityType(
      google::cloud::dialogflow::cx::v3::EntityType const& entity_type,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates the specified entity type.
  ///
  /// Note: You should always train a flow prior to sending it queries. See the
  /// [training
  /// documentation](https://cloud.google.com/dialogflow/cx/docs/concept/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::UpdateEntityTypeRequest,google/cloud/dialogflow/cx/v3/entity_type.proto#L327}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::EntityType,google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  /// [google.cloud.dialogflow.cx.v3.UpdateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L327}
  /// [google.cloud.dialogflow.cx.v3.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L124}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::EntityType> UpdateEntityType(
      google::cloud::dialogflow::cx::v3::UpdateEntityTypeRequest const& request,
      Options opts = {});

  ///
  /// Deletes the specified entity type.
  ///
  /// Note: You should always train a flow prior to sending it queries. See the
  /// [training
  /// documentation](https://cloud.google.com/dialogflow/cx/docs/concept/training).
  ///
  /// @param name  Required. The name of the entity type to delete.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>/entityTypes/<Entity Type ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.cx.v3.DeleteEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L349}
  ///
  Status DeleteEntityType(std::string const& name, Options opts = {});

  ///
  /// Deletes the specified entity type.
  ///
  /// Note: You should always train a flow prior to sending it queries. See the
  /// [training
  /// documentation](https://cloud.google.com/dialogflow/cx/docs/concept/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::DeleteEntityTypeRequest,google/cloud/dialogflow/cx/v3/entity_type.proto#L349}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.cx.v3.DeleteEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/entity_type.proto#L349}
  ///
  Status DeleteEntityType(
      google::cloud::dialogflow::cx::v3::DeleteEntityTypeRequest const& request,
      Options opts = {});

 private:
  std::shared_ptr<EntityTypesConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_cx
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_ENTITY_TYPES_CLIENT_H
