// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/cx/v3/environment.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_ENVIRONMENTS_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_ENVIRONMENTS_CLIENT_H

#include "google/cloud/dialogflow_cx/environments_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace dialogflow_cx {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Service for managing
/// [Environments][google.cloud.dialogflow.cx.v3.Environment].
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class EnvironmentsClient {
 public:
  explicit EnvironmentsClient(
      std::shared_ptr<EnvironmentsConnection> connection, Options opts = {});
  ~EnvironmentsClient();

  ///@{
  // @name Copy and move support
  EnvironmentsClient(EnvironmentsClient const&) = default;
  EnvironmentsClient& operator=(EnvironmentsClient const&) = default;
  EnvironmentsClient(EnvironmentsClient&&) = default;
  EnvironmentsClient& operator=(EnvironmentsClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(EnvironmentsClient const& a,
                         EnvironmentsClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(EnvironmentsClient const& a,
                         EnvironmentsClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Returns the list of all environments in the specified
  /// [Agent][google.cloud.dialogflow.cx.v3.Agent].
  ///
  /// @param parent  Required. The [Agent][google.cloud.dialogflow.cx.v3.Agent]
  /// to list all environments for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.ListEnvironmentsRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L250}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::Environment> ListEnvironments(
      std::string const& parent, Options opts = {});

  ///
  /// Returns the list of all environments in the specified
  /// [Agent][google.cloud.dialogflow.cx.v3.Agent].
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::ListEnvironmentsRequest,google/cloud/dialogflow/cx/v3/environment.proto#L250}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.ListEnvironmentsRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L250}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::Environment> ListEnvironments(
      google::cloud::dialogflow::cx::v3::ListEnvironmentsRequest request,
      Options opts = {});

  ///
  /// Retrieves the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// @param name  Required. The name of the
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>/environments/<Environment ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.GetEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L282}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::Environment> GetEnvironment(
      std::string const& name, Options opts = {});

  ///
  /// Retrieves the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::GetEnvironmentRequest,google/cloud/dialogflow/cx/v3/environment.proto#L282}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.GetEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L282}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  StatusOr<google::cloud::dialogflow::cx::v3::Environment> GetEnvironment(
      google::cloud::dialogflow::cx::v3::GetEnvironmentRequest const& request,
      Options opts = {});

  ///
  /// Creates an [Environment][google.cloud.dialogflow.cx.v3.Environment] in the
  /// specified [Agent][google.cloud.dialogflow.cx.v3.Agent].
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/cx/docs/how/long-running-operation).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: [Environment][google.cloud.dialogflow.cx.v3.Environment]
  ///
  /// @param parent  Required. The [Agent][google.cloud.dialogflow.cx.v3.Agent]
  /// to create an [Environment][google.cloud.dialogflow.cx.v3.Environment] for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>`.
  /// @param environment  Required. The environment to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.CreateEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L295}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  future<StatusOr<google::cloud::dialogflow::cx::v3::Environment>>
  CreateEnvironment(
      std::string const& parent,
      google::cloud::dialogflow::cx::v3::Environment const& environment,
      Options opts = {});

  ///
  /// Creates an [Environment][google.cloud.dialogflow.cx.v3.Environment] in the
  /// specified [Agent][google.cloud.dialogflow.cx.v3.Agent].
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/cx/docs/how/long-running-operation).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: [Environment][google.cloud.dialogflow.cx.v3.Environment]
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::CreateEnvironmentRequest,google/cloud/dialogflow/cx/v3/environment.proto#L295}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.CreateEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L295}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  future<StatusOr<google::cloud::dialogflow::cx::v3::Environment>>
  CreateEnvironment(
      google::cloud::dialogflow::cx::v3::CreateEnvironmentRequest const&
          request,
      Options opts = {});

  ///
  /// Updates the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/cx/docs/how/long-running-operation).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: [Environment][google.cloud.dialogflow.cx.v3.Environment]
  ///
  /// @param environment  Required. The environment to update.
  /// @param update_mask  Required. The mask to control which fields get
  /// updated.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.UpdateEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L310}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  future<StatusOr<google::cloud::dialogflow::cx::v3::Environment>>
  UpdateEnvironment(
      google::cloud::dialogflow::cx::v3::Environment const& environment,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/cx/docs/how/long-running-operation).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: [Environment][google.cloud.dialogflow.cx.v3.Environment]
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::UpdateEnvironmentRequest,google/cloud/dialogflow/cx/v3/environment.proto#L310}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.UpdateEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L310}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  future<StatusOr<google::cloud::dialogflow::cx::v3::Environment>>
  UpdateEnvironment(
      google::cloud::dialogflow::cx::v3::UpdateEnvironmentRequest const&
          request,
      Options opts = {});

  ///
  /// Deletes the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// @param name  Required. The name of the
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment] to delete.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>/environments/<Environment ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.cx.v3.DeleteEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L319}
  ///
  Status DeleteEnvironment(std::string const& name, Options opts = {});

  ///
  /// Deletes the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::DeleteEnvironmentRequest,google/cloud/dialogflow/cx/v3/environment.proto#L319}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.cx.v3.DeleteEnvironmentRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L319}
  ///
  Status DeleteEnvironment(
      google::cloud::dialogflow::cx::v3::DeleteEnvironmentRequest const&
          request,
      Options opts = {});

  ///
  /// Looks up the history of the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// @param name  Required. Resource name of the environment to look up the
  /// history for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent
  ///  ID>/environments/<Environment ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.LookupEnvironmentHistoryRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L332}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::Environment>
  LookupEnvironmentHistory(std::string const& name, Options opts = {});

  ///
  /// Looks up the history of the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::LookupEnvironmentHistoryRequest,google/cloud/dialogflow/cx/v3/environment.proto#L332}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::Environment,google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  /// [google.cloud.dialogflow.cx.v3.LookupEnvironmentHistoryRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L332}
  /// [google.cloud.dialogflow.cx.v3.Environment]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L175}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::Environment>
  LookupEnvironmentHistory(
      google::cloud::dialogflow::cx::v3::LookupEnvironmentHistoryRequest
          request,
      Options opts = {});

  ///
  /// Kicks off a continuous test under the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/cx/docs/how/long-running-operation).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [RunContinuousTestMetadata][google.cloud.dialogflow.cx.v3.RunContinuousTestMetadata]
  /// - `response`:
  /// [RunContinuousTestResponse][google.cloud.dialogflow.cx.v3.RunContinuousTestResponse]
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::RunContinuousTestRequest,google/cloud/dialogflow/cx/v3/environment.proto#L401}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::RunContinuousTestResponse,google/cloud/dialogflow/cx/v3/environment.proto#L413}
  ///
  /// [google.cloud.dialogflow.cx.v3.RunContinuousTestRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L401}
  /// [google.cloud.dialogflow.cx.v3.RunContinuousTestResponse]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L413}
  ///
  future<StatusOr<google::cloud::dialogflow::cx::v3::RunContinuousTestResponse>>
  RunContinuousTest(
      google::cloud::dialogflow::cx::v3::RunContinuousTestRequest const&
          request,
      Options opts = {});

  ///
  /// Fetches a list of continuous test results for a given environment.
  ///
  /// @param parent  Required. The environment to list results for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/
  ///  environments/<Environment ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::ContinuousTestResult,google/cloud/dialogflow/cx/v3/environment.proto#L363}
  ///
  /// [google.cloud.dialogflow.cx.v3.ListContinuousTestResultsRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L426}
  /// [google.cloud.dialogflow.cx.v3.ContinuousTestResult]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L363}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::ContinuousTestResult>
  ListContinuousTestResults(std::string const& parent, Options opts = {});

  ///
  /// Fetches a list of continuous test results for a given environment.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::ListContinuousTestResultsRequest,google/cloud/dialogflow/cx/v3/environment.proto#L426}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::ContinuousTestResult,google/cloud/dialogflow/cx/v3/environment.proto#L363}
  ///
  /// [google.cloud.dialogflow.cx.v3.ListContinuousTestResultsRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L426}
  /// [google.cloud.dialogflow.cx.v3.ContinuousTestResult]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L363}
  ///
  StreamRange<google::cloud::dialogflow::cx::v3::ContinuousTestResult>
  ListContinuousTestResults(
      google::cloud::dialogflow::cx::v3::ListContinuousTestResultsRequest
          request,
      Options opts = {});

  ///
  /// Deploys a flow to the specified
  /// [Environment][google.cloud.dialogflow.cx.v3.Environment].
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/cx/docs/how/long-running-operation).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [DeployFlowMetadata][google.cloud.dialogflow.cx.v3.DeployFlowMetadata]
  /// - `response`:
  /// [DeployFlowResponse][google.cloud.dialogflow.cx.v3.DeployFlowResponse]
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::DeployFlowRequest,google/cloud/dialogflow/cx/v3/environment.proto#L456}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::cx::v3::DeployFlowResponse,google/cloud/dialogflow/cx/v3/environment.proto#L479}
  ///
  /// [google.cloud.dialogflow.cx.v3.DeployFlowRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L456}
  /// [google.cloud.dialogflow.cx.v3.DeployFlowResponse]:
  /// @googleapis_reference_link{google/cloud/dialogflow/cx/v3/environment.proto#L479}
  ///
  future<StatusOr<google::cloud::dialogflow::cx::v3::DeployFlowResponse>>
  DeployFlow(
      google::cloud::dialogflow::cx::v3::DeployFlowRequest const& request,
      Options opts = {});

 private:
  std::shared_ptr<EnvironmentsConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_cx
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_CX_ENVIRONMENTS_CLIENT_H
