// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/v2/conversation_dataset.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_CONVERSATION_DATASETS_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_CONVERSATION_DATASETS_CLIENT_H

#include "google/cloud/dialogflow_es/conversation_datasets_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace dialogflow_es {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Conversation datasets.
///
/// Conversation datasets contain raw conversation files and their
/// customizable metadata that can be used for model training.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class ConversationDatasetsClient {
 public:
  explicit ConversationDatasetsClient(
      std::shared_ptr<ConversationDatasetsConnection> connection,
      Options opts = {});
  ~ConversationDatasetsClient();

  ///@{
  // @name Copy and move support
  ConversationDatasetsClient(ConversationDatasetsClient const&) = default;
  ConversationDatasetsClient& operator=(ConversationDatasetsClient const&) =
      default;
  ConversationDatasetsClient(ConversationDatasetsClient&&) = default;
  ConversationDatasetsClient& operator=(ConversationDatasetsClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(ConversationDatasetsClient const& a,
                         ConversationDatasetsClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(ConversationDatasetsClient const& a,
                         ConversationDatasetsClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Creates a new conversation dataset.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [CreateConversationDatasetOperationMetadata][google.cloud.dialogflow.v2.CreateConversationDatasetOperationMetadata]
  /// - `response`:
  /// [ConversationDataset][google.cloud.dialogflow.v2.ConversationDataset]
  ///
  /// @param parent  Required. The project to create conversation dataset for.
  /// Format:
  ///  `projects/<Project ID>/locations/<Location ID>`
  /// @param conversation_dataset  Required. The conversation dataset to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationDataset,google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  /// [google.cloud.dialogflow.v2.CreateConversationDatasetRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L191}
  /// [google.cloud.dialogflow.v2.ConversationDataset]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  future<StatusOr<google::cloud::dialogflow::v2::ConversationDataset>>
  CreateConversationDataset(
      std::string const& parent,
      google::cloud::dialogflow::v2::ConversationDataset const&
          conversation_dataset,
      Options opts = {});

  ///
  /// Creates a new conversation dataset.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [CreateConversationDatasetOperationMetadata][google.cloud.dialogflow.v2.CreateConversationDatasetOperationMetadata]
  /// - `response`:
  /// [ConversationDataset][google.cloud.dialogflow.v2.ConversationDataset]
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::CreateConversationDatasetRequest,google/cloud/dialogflow/v2/conversation_dataset.proto#L191}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationDataset,google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  /// [google.cloud.dialogflow.v2.CreateConversationDatasetRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L191}
  /// [google.cloud.dialogflow.v2.ConversationDataset]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  future<StatusOr<google::cloud::dialogflow::v2::ConversationDataset>>
  CreateConversationDataset(
      google::cloud::dialogflow::v2::CreateConversationDatasetRequest const&
          request,
      Options opts = {});

  ///
  /// Retrieves the specified conversation dataset.
  ///
  /// @param name  Required. The conversation dataset to retrieve. Format:
  ///  `projects/<Project ID>/locations/<Location
  ///  ID>/conversationDatasets/<Conversation Dataset ID>`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationDataset,google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  /// [google.cloud.dialogflow.v2.GetConversationDatasetRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L202}
  /// [google.cloud.dialogflow.v2.ConversationDataset]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  StatusOr<google::cloud::dialogflow::v2::ConversationDataset>
  GetConversationDataset(std::string const& name, Options opts = {});

  ///
  /// Retrieves the specified conversation dataset.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::GetConversationDatasetRequest,google/cloud/dialogflow/v2/conversation_dataset.proto#L202}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationDataset,google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  /// [google.cloud.dialogflow.v2.GetConversationDatasetRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L202}
  /// [google.cloud.dialogflow.v2.ConversationDataset]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  StatusOr<google::cloud::dialogflow::v2::ConversationDataset>
  GetConversationDataset(
      google::cloud::dialogflow::v2::GetConversationDatasetRequest const&
          request,
      Options opts = {});

  ///
  /// Returns the list of all conversation datasets in the specified
  /// project and location.
  ///
  /// @param parent  Required. The project and location name to list all
  /// conversation datasets for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationDataset,google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  /// [google.cloud.dialogflow.v2.ListConversationDatasetsRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L216}
  /// [google.cloud.dialogflow.v2.ConversationDataset]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  StreamRange<google::cloud::dialogflow::v2::ConversationDataset>
  ListConversationDatasets(std::string const& parent, Options opts = {});

  ///
  /// Returns the list of all conversation datasets in the specified
  /// project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::ListConversationDatasetsRequest,google/cloud/dialogflow/v2/conversation_dataset.proto#L216}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationDataset,google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  /// [google.cloud.dialogflow.v2.ListConversationDatasetsRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L216}
  /// [google.cloud.dialogflow.v2.ConversationDataset]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L159}
  ///
  StreamRange<google::cloud::dialogflow::v2::ConversationDataset>
  ListConversationDatasets(
      google::cloud::dialogflow::v2::ListConversationDatasetsRequest request,
      Options opts = {});

  ///
  /// Deletes the specified conversation dataset.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [DeleteConversationDatasetOperationMetadata][google.cloud.dialogflow.v2.DeleteConversationDatasetOperationMetadata]
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// @param name  Required. The conversation dataset to delete. Format:
  ///  `projects/<Project ID>/locations/<Location
  ///  ID>/conversationDatasets/<Conversation Dataset ID>`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::DeleteConversationDatasetOperationMetadata,google/cloud/dialogflow/v2/conversation_dataset.proto#L318}
  ///
  /// [google.cloud.dialogflow.v2.DeleteConversationDatasetRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L247}
  /// [google.cloud.dialogflow.v2.DeleteConversationDatasetOperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L318}
  ///
  future<StatusOr<google::cloud::dialogflow::v2::
                      DeleteConversationDatasetOperationMetadata>>
  DeleteConversationDataset(std::string const& name, Options opts = {});

  ///
  /// Deletes the specified conversation dataset.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [DeleteConversationDatasetOperationMetadata][google.cloud.dialogflow.v2.DeleteConversationDatasetOperationMetadata]
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::DeleteConversationDatasetRequest,google/cloud/dialogflow/v2/conversation_dataset.proto#L247}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::DeleteConversationDatasetOperationMetadata,google/cloud/dialogflow/v2/conversation_dataset.proto#L318}
  ///
  /// [google.cloud.dialogflow.v2.DeleteConversationDatasetRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L247}
  /// [google.cloud.dialogflow.v2.DeleteConversationDatasetOperationMetadata]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L318}
  ///
  future<StatusOr<google::cloud::dialogflow::v2::
                      DeleteConversationDatasetOperationMetadata>>
  DeleteConversationDataset(
      google::cloud::dialogflow::v2::DeleteConversationDatasetRequest const&
          request,
      Options opts = {});

  ///
  /// Import data into the specified conversation dataset. Note that it
  /// is not allowed to import data to a conversation dataset that
  /// already has data in it.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [ImportConversationDataOperationMetadata][google.cloud.dialogflow.v2.ImportConversationDataOperationMetadata]
  /// - `response`:
  /// [ImportConversationDataOperationResponse][google.cloud.dialogflow.v2.ImportConversationDataOperationResponse]
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::ImportConversationDataRequest,google/cloud/dialogflow/v2/conversation_dataset.proto#L260}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ImportConversationDataOperationResponse,google/cloud/dialogflow/v2/conversation_dataset.proto#L295}
  ///
  /// [google.cloud.dialogflow.v2.ImportConversationDataRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L260}
  /// [google.cloud.dialogflow.v2.ImportConversationDataOperationResponse]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_dataset.proto#L295}
  ///
  future<StatusOr<
      google::cloud::dialogflow::v2::ImportConversationDataOperationResponse>>
  ImportConversationData(
      google::cloud::dialogflow::v2::ImportConversationDataRequest const&
          request,
      Options opts = {});

 private:
  std::shared_ptr<ConversationDatasetsConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_es
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_CONVERSATION_DATASETS_CLIENT_H
