// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/v2/conversation_profile.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_CONVERSATION_PROFILES_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_CONVERSATION_PROFILES_CLIENT_H

#include "google/cloud/dialogflow_es/conversation_profiles_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace dialogflow_es {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Service for managing
/// [ConversationProfiles][google.cloud.dialogflow.v2.ConversationProfile].
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class ConversationProfilesClient {
 public:
  explicit ConversationProfilesClient(
      std::shared_ptr<ConversationProfilesConnection> connection,
      Options opts = {});
  ~ConversationProfilesClient();

  ///@{
  // @name Copy and move support
  ConversationProfilesClient(ConversationProfilesClient const&) = default;
  ConversationProfilesClient& operator=(ConversationProfilesClient const&) =
      default;
  ConversationProfilesClient(ConversationProfilesClient&&) = default;
  ConversationProfilesClient& operator=(ConversationProfilesClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(ConversationProfilesClient const& a,
                         ConversationProfilesClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(ConversationProfilesClient const& a,
                         ConversationProfilesClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Returns the list of all conversation profiles in the specified project.
  ///
  /// @param parent  Required. The project to list all conversation profiles
  /// from.
  ///  Format: `projects/<Project ID>/locations/<Location ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.ListConversationProfilesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L242}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  StreamRange<google::cloud::dialogflow::v2::ConversationProfile>
  ListConversationProfiles(std::string const& parent, Options opts = {});

  ///
  /// Returns the list of all conversation profiles in the specified project.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::ListConversationProfilesRequest,google/cloud/dialogflow/v2/conversation_profile.proto#L242}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.ListConversationProfilesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L242}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  StreamRange<google::cloud::dialogflow::v2::ConversationProfile>
  ListConversationProfiles(
      google::cloud::dialogflow::v2::ListConversationProfilesRequest request,
      Options opts = {});

  ///
  /// Retrieves the specified conversation profile.
  ///
  /// @param name  Required. The resource name of the conversation profile.
  ///  Format: `projects/<Project ID>/locations/<Location
  ///  ID>/conversationProfiles/<Conversation Profile ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.GetConversationProfileRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L272}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  StatusOr<google::cloud::dialogflow::v2::ConversationProfile>
  GetConversationProfile(std::string const& name, Options opts = {});

  ///
  /// Retrieves the specified conversation profile.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::GetConversationProfileRequest,google/cloud/dialogflow/v2/conversation_profile.proto#L272}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.GetConversationProfileRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L272}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  StatusOr<google::cloud::dialogflow::v2::ConversationProfile>
  GetConversationProfile(
      google::cloud::dialogflow::v2::GetConversationProfileRequest const&
          request,
      Options opts = {});

  ///
  /// Creates a conversation profile in the specified project.
  ///
  /// [ConversationProfile.CreateTime][] and [ConversationProfile.UpdateTime][]
  /// aren't populated in the response. You can retrieve them via
  /// [GetConversationProfile][google.cloud.dialogflow.v2.ConversationProfiles.GetConversationProfile]
  /// API.
  ///
  /// @param parent  Required. The project to create a conversation profile for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>`.
  /// @param conversation_profile  Required. The conversation profile to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.CreateConversationProfileRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L285}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  StatusOr<google::cloud::dialogflow::v2::ConversationProfile>
  CreateConversationProfile(
      std::string const& parent,
      google::cloud::dialogflow::v2::ConversationProfile const&
          conversation_profile,
      Options opts = {});

  ///
  /// Creates a conversation profile in the specified project.
  ///
  /// [ConversationProfile.CreateTime][] and [ConversationProfile.UpdateTime][]
  /// aren't populated in the response. You can retrieve them via
  /// [GetConversationProfile][google.cloud.dialogflow.v2.ConversationProfiles.GetConversationProfile]
  /// API.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::CreateConversationProfileRequest,google/cloud/dialogflow/v2/conversation_profile.proto#L285}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.CreateConversationProfileRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L285}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  StatusOr<google::cloud::dialogflow::v2::ConversationProfile>
  CreateConversationProfile(
      google::cloud::dialogflow::v2::CreateConversationProfileRequest const&
          request,
      Options opts = {});

  ///
  /// Updates the specified conversation profile.
  ///
  /// [ConversationProfile.CreateTime][] and [ConversationProfile.UpdateTime][]
  /// aren't populated in the response. You can retrieve them via
  /// [GetConversationProfile][google.cloud.dialogflow.v2.ConversationProfiles.GetConversationProfile]
  /// API.
  ///
  /// @param conversation_profile  Required. The conversation profile to update.
  /// @param update_mask  Required. The mask to control which fields to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.UpdateConversationProfileRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L300}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  StatusOr<google::cloud::dialogflow::v2::ConversationProfile>
  UpdateConversationProfile(
      google::cloud::dialogflow::v2::ConversationProfile const&
          conversation_profile,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates the specified conversation profile.
  ///
  /// [ConversationProfile.CreateTime][] and [ConversationProfile.UpdateTime][]
  /// aren't populated in the response. You can retrieve them via
  /// [GetConversationProfile][google.cloud.dialogflow.v2.ConversationProfiles.GetConversationProfile]
  /// API.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::UpdateConversationProfileRequest,google/cloud/dialogflow/v2/conversation_profile.proto#L300}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.UpdateConversationProfileRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L300}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  StatusOr<google::cloud::dialogflow::v2::ConversationProfile>
  UpdateConversationProfile(
      google::cloud::dialogflow::v2::UpdateConversationProfileRequest const&
          request,
      Options opts = {});

  ///
  /// Deletes the specified conversation profile.
  ///
  /// @param name  Required. The name of the conversation profile to delete.
  ///  Format: `projects/<Project ID>/locations/<Location
  ///  ID>/conversationProfiles/<Conversation Profile ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.v2.DeleteConversationProfileRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L312}
  ///
  Status DeleteConversationProfile(std::string const& name, Options opts = {});

  ///
  /// Deletes the specified conversation profile.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::DeleteConversationProfileRequest,google/cloud/dialogflow/v2/conversation_profile.proto#L312}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.v2.DeleteConversationProfileRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L312}
  ///
  Status DeleteConversationProfile(
      google::cloud::dialogflow::v2::DeleteConversationProfileRequest const&
          request,
      Options opts = {});

  ///
  /// Adds or updates a suggestion feature in a conversation profile.
  /// If the conversation profile contains the type of suggestion feature for
  /// the participant role, it will update it. Otherwise it will insert the
  /// suggestion feature.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [SetSuggestionFeatureConfigOperationMetadata][google.cloud.dialogflow.v2.SetSuggestionFeatureConfigOperationMetadata]
  /// - `response`:
  /// [ConversationProfile][google.cloud.dialogflow.v2.ConversationProfile]
  ///
  /// If a long running operation to add or update suggestion feature
  /// config for the same conversation profile, participant role and suggestion
  /// feature type exists, please cancel the existing long running operation
  /// before sending such request, otherwise the request will be rejected.
  ///
  /// @param conversation_profile  Required. The Conversation Profile to add or
  /// update the suggestion feature
  ///  config. Format: `projects/<Project ID>/locations/<Location
  ///  ID>/conversationProfiles/<Conversation Profile ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.SetSuggestionFeatureConfigRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L688}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  future<StatusOr<google::cloud::dialogflow::v2::ConversationProfile>>
  SetSuggestionFeatureConfig(std::string const& conversation_profile,
                             Options opts = {});

  ///
  /// Adds or updates a suggestion feature in a conversation profile.
  /// If the conversation profile contains the type of suggestion feature for
  /// the participant role, it will update it. Otherwise it will insert the
  /// suggestion feature.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [SetSuggestionFeatureConfigOperationMetadata][google.cloud.dialogflow.v2.SetSuggestionFeatureConfigOperationMetadata]
  /// - `response`:
  /// [ConversationProfile][google.cloud.dialogflow.v2.ConversationProfile]
  ///
  /// If a long running operation to add or update suggestion feature
  /// config for the same conversation profile, participant role and suggestion
  /// feature type exists, please cancel the existing long running operation
  /// before sending such request, otherwise the request will be rejected.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::SetSuggestionFeatureConfigRequest,google/cloud/dialogflow/v2/conversation_profile.proto#L688}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.SetSuggestionFeatureConfigRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L688}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  future<StatusOr<google::cloud::dialogflow::v2::ConversationProfile>>
  SetSuggestionFeatureConfig(
      google::cloud::dialogflow::v2::SetSuggestionFeatureConfigRequest const&
          request,
      Options opts = {});

  ///
  /// Clears a suggestion feature from a conversation profile for the given
  /// participant role.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [ClearSuggestionFeatureConfigOperationMetadata][google.cloud.dialogflow.v2.ClearSuggestionFeatureConfigOperationMetadata]
  /// - `response`:
  /// [ConversationProfile][google.cloud.dialogflow.v2.ConversationProfile]
  ///
  /// @param conversation_profile  Required. The Conversation Profile to add or
  /// update the suggestion feature
  ///  config. Format: `projects/<Project ID>/locations/<Location
  ///  ID>/conversationProfiles/<Conversation Profile ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.ClearSuggestionFeatureConfigRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L703}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  future<StatusOr<google::cloud::dialogflow::v2::ConversationProfile>>
  ClearSuggestionFeatureConfig(std::string const& conversation_profile,
                               Options opts = {});

  ///
  /// Clears a suggestion feature from a conversation profile for the given
  /// participant role.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`:
  /// [ClearSuggestionFeatureConfigOperationMetadata][google.cloud.dialogflow.v2.ClearSuggestionFeatureConfigOperationMetadata]
  /// - `response`:
  /// [ConversationProfile][google.cloud.dialogflow.v2.ConversationProfile]
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::ClearSuggestionFeatureConfigRequest,google/cloud/dialogflow/v2/conversation_profile.proto#L703}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::ConversationProfile,google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  /// [google.cloud.dialogflow.v2.ClearSuggestionFeatureConfigRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L703}
  /// [google.cloud.dialogflow.v2.ConversationProfile]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/conversation_profile.proto#L175}
  ///
  future<StatusOr<google::cloud::dialogflow::v2::ConversationProfile>>
  ClearSuggestionFeatureConfig(
      google::cloud::dialogflow::v2::ClearSuggestionFeatureConfigRequest const&
          request,
      Options opts = {});

 private:
  std::shared_ptr<ConversationProfilesConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_es
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_CONVERSATION_PROFILES_CLIENT_H
