// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/v2/entity_type.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_ENTITY_TYPES_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_ENTITY_TYPES_CLIENT_H

#include "google/cloud/dialogflow_es/entity_types_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace dialogflow_es {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Service for managing [EntityTypes][google.cloud.dialogflow.v2.EntityType].
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class EntityTypesClient {
 public:
  explicit EntityTypesClient(std::shared_ptr<EntityTypesConnection> connection,
                             Options opts = {});
  ~EntityTypesClient();

  ///@{
  // @name Copy and move support
  EntityTypesClient(EntityTypesClient const&) = default;
  EntityTypesClient& operator=(EntityTypesClient const&) = default;
  EntityTypesClient(EntityTypesClient&&) = default;
  EntityTypesClient& operator=(EntityTypesClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(EntityTypesClient const& a,
                         EntityTypesClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(EntityTypesClient const& a,
                         EntityTypesClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Returns the list of all entity types in the specified agent.
  ///
  /// @param parent  Required. The agent to list all entity types from.
  ///  Format: `projects/<Project ID>/agent`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.ListEntityTypesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L372}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StreamRange<google::cloud::dialogflow::v2::EntityType> ListEntityTypes(
      std::string const& parent, Options opts = {});

  ///
  /// Returns the list of all entity types in the specified agent.
  ///
  /// @param parent  Required. The agent to list all entity types from.
  ///  Format: `projects/<Project ID>/agent`.
  /// @param language_code  Optional. The language used to access
  /// language-specific data.
  ///  If not specified, the agent's default language is used.
  ///  For more information, see
  ///  [Multilingual intent and entity
  ///  data](https://cloud.google.com/dialogflow/docs/agents-multilingual#intent-entity).
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.ListEntityTypesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L372}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StreamRange<google::cloud::dialogflow::v2::EntityType> ListEntityTypes(
      std::string const& parent, std::string const& language_code,
      Options opts = {});

  ///
  /// Returns the list of all entity types in the specified agent.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::ListEntityTypesRequest,google/cloud/dialogflow/v2/entity_type.proto#L372}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.ListEntityTypesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L372}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StreamRange<google::cloud::dialogflow::v2::EntityType> ListEntityTypes(
      google::cloud::dialogflow::v2::ListEntityTypesRequest request,
      Options opts = {});

  ///
  /// Retrieves the specified entity type.
  ///
  /// @param name  Required. The name of the entity type.
  ///  Format: `projects/<Project ID>/agent/entityTypes/<EntityType ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.GetEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L409}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> GetEntityType(
      std::string const& name, Options opts = {});

  ///
  /// Retrieves the specified entity type.
  ///
  /// @param name  Required. The name of the entity type.
  ///  Format: `projects/<Project ID>/agent/entityTypes/<EntityType ID>`.
  /// @param language_code  Optional. The language used to access
  /// language-specific data.
  ///  If not specified, the agent's default language is used.
  ///  For more information, see
  ///  [Multilingual intent and entity
  ///  data](https://cloud.google.com/dialogflow/docs/agents-multilingual#intent-entity).
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.GetEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L409}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> GetEntityType(
      std::string const& name, std::string const& language_code,
      Options opts = {});

  ///
  /// Retrieves the specified entity type.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::GetEntityTypeRequest,google/cloud/dialogflow/v2/entity_type.proto#L409}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.GetEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L409}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> GetEntityType(
      google::cloud::dialogflow::v2::GetEntityTypeRequest const& request,
      Options opts = {});

  ///
  /// Creates an entity type in the specified agent.
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param parent  Required. The agent to create a entity type for.
  ///  Format: `projects/<Project ID>/agent`.
  /// @param entity_type  Required. The entity type to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.CreateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L428}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> CreateEntityType(
      std::string const& parent,
      google::cloud::dialogflow::v2::EntityType const& entity_type,
      Options opts = {});

  ///
  /// Creates an entity type in the specified agent.
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param parent  Required. The agent to create a entity type for.
  ///  Format: `projects/<Project ID>/agent`.
  /// @param entity_type  Required. The entity type to create.
  /// @param language_code  Optional. The language used to access
  /// language-specific data.
  ///  If not specified, the agent's default language is used.
  ///  For more information, see
  ///  [Multilingual intent and entity
  ///  data](https://cloud.google.com/dialogflow/docs/agents-multilingual#intent-entity).
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.CreateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L428}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> CreateEntityType(
      std::string const& parent,
      google::cloud::dialogflow::v2::EntityType const& entity_type,
      std::string const& language_code, Options opts = {});

  ///
  /// Creates an entity type in the specified agent.
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::CreateEntityTypeRequest,google/cloud/dialogflow/v2/entity_type.proto#L428}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.CreateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L428}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> CreateEntityType(
      google::cloud::dialogflow::v2::CreateEntityTypeRequest const& request,
      Options opts = {});

  ///
  /// Updates the specified entity type.
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param entity_type  Required. The entity type to update.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.UpdateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L450}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> UpdateEntityType(
      google::cloud::dialogflow::v2::EntityType const& entity_type,
      Options opts = {});

  ///
  /// Updates the specified entity type.
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param entity_type  Required. The entity type to update.
  /// @param language_code  Optional. The language used to access
  /// language-specific data.
  ///  If not specified, the agent's default language is used.
  ///  For more information, see
  ///  [Multilingual intent and entity
  ///  data](https://cloud.google.com/dialogflow/docs/agents-multilingual#intent-entity).
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.UpdateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L450}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> UpdateEntityType(
      google::cloud::dialogflow::v2::EntityType const& entity_type,
      std::string const& language_code, Options opts = {});

  ///
  /// Updates the specified entity type.
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::UpdateEntityTypeRequest,google/cloud/dialogflow/v2/entity_type.proto#L450}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::EntityType,google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  /// [google.cloud.dialogflow.v2.UpdateEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L450}
  /// [google.cloud.dialogflow.v2.EntityType]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L284}
  ///
  StatusOr<google::cloud::dialogflow::v2::EntityType> UpdateEntityType(
      google::cloud::dialogflow::v2::UpdateEntityTypeRequest const& request,
      Options opts = {});

  ///
  /// Deletes the specified entity type.
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param name  Required. The name of the entity type to delete.
  ///  Format: `projects/<Project ID>/agent/entityTypes/<EntityType ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.v2.DeleteEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L466}
  ///
  Status DeleteEntityType(std::string const& name, Options opts = {});

  ///
  /// Deletes the specified entity type.
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::DeleteEntityTypeRequest,google/cloud/dialogflow/v2/entity_type.proto#L466}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.v2.DeleteEntityTypeRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L466}
  ///
  Status DeleteEntityType(
      google::cloud::dialogflow::v2::DeleteEntityTypeRequest const& request,
      Options opts = {});

  ///
  /// Updates/Creates multiple entity types in the specified agent.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`:
  /// [BatchUpdateEntityTypesResponse][google.cloud.dialogflow.v2.BatchUpdateEntityTypesResponse]
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::BatchUpdateEntityTypesRequest,google/cloud/dialogflow/v2/entity_type.proto#L478}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::BatchUpdateEntityTypesResponse,google/cloud/dialogflow/v2/entity_type.proto#L517}
  ///
  /// [google.cloud.dialogflow.v2.BatchUpdateEntityTypesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L478}
  /// [google.cloud.dialogflow.v2.BatchUpdateEntityTypesResponse]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L517}
  ///
  future<
      StatusOr<google::cloud::dialogflow::v2::BatchUpdateEntityTypesResponse>>
  BatchUpdateEntityTypes(
      google::cloud::dialogflow::v2::BatchUpdateEntityTypesRequest const&
          request,
      Options opts = {});

  ///
  /// Deletes entity types in the specified agent.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param parent  Required. The name of the agent to delete all entities
  /// types for. Format:
  ///  `projects/<Project ID>/agent`.
  /// @param entity_type_names  Required. The names entity types to delete. All
  /// names must point to the
  ///  same agent as `parent`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchDeleteEntityTypesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L523}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchDeleteEntityTypes(
      std::string const& parent,
      std::vector<std::string> const& entity_type_names, Options opts = {});

  ///
  /// Deletes entity types in the specified agent.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::BatchDeleteEntityTypesRequest,google/cloud/dialogflow/v2/entity_type.proto#L523}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchDeleteEntityTypesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L523}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchDeleteEntityTypes(
      google::cloud::dialogflow::v2::BatchDeleteEntityTypesRequest const&
          request,
      Options opts = {});

  ///
  /// Creates multiple new entities in the specified entity type.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param parent  Required. The name of the entity type to create entities
  /// in. Format:
  ///  `projects/<Project ID>/agent/entityTypes/<Entity Type ID>`.
  /// @param entities  Required. The entities to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchCreateEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L539}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchCreateEntities(
      std::string const& parent,
      std::vector<google::cloud::dialogflow::v2::EntityType::Entity> const&
          entities,
      Options opts = {});

  ///
  /// Creates multiple new entities in the specified entity type.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param parent  Required. The name of the entity type to create entities
  /// in. Format:
  ///  `projects/<Project ID>/agent/entityTypes/<Entity Type ID>`.
  /// @param entities  Required. The entities to create.
  /// @param language_code  Optional. The language used to access
  /// language-specific data.
  ///  If not specified, the agent's default language is used.
  ///  For more information, see
  ///  [Multilingual intent and entity
  ///  data](https://cloud.google.com/dialogflow/docs/agents-multilingual#intent-entity).
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchCreateEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L539}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchCreateEntities(
      std::string const& parent,
      std::vector<google::cloud::dialogflow::v2::EntityType::Entity> const&
          entities,
      std::string const& language_code, Options opts = {});

  ///
  /// Creates multiple new entities in the specified entity type.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::BatchCreateEntitiesRequest,google/cloud/dialogflow/v2/entity_type.proto#L539}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchCreateEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L539}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchCreateEntities(
      google::cloud::dialogflow::v2::BatchCreateEntitiesRequest const& request,
      Options opts = {});

  ///
  /// Updates or creates multiple entities in the specified entity type. This
  /// method does not affect entities in the entity type that aren't explicitly
  /// specified in the request.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  ///
  /// @param parent  Required. The name of the entity type to update or create
  /// entities in.
  ///  Format: `projects/<Project ID>/agent/entityTypes/<Entity Type ID>`.
  /// @param entities  Required. The entities to update or create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchUpdateEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L561}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchUpdateEntities(
      std::string const& parent,
      std::vector<google::cloud::dialogflow::v2::EntityType::Entity> const&
          entities,
      Options opts = {});

  ///
  /// Updates or creates multiple entities in the specified entity type. This
  /// method does not affect entities in the entity type that aren't explicitly
  /// specified in the request.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  ///
  /// @param parent  Required. The name of the entity type to update or create
  /// entities in.
  ///  Format: `projects/<Project ID>/agent/entityTypes/<Entity Type ID>`.
  /// @param entities  Required. The entities to update or create.
  /// @param language_code  Optional. The language used to access
  /// language-specific data.
  ///  If not specified, the agent's default language is used.
  ///  For more information, see
  ///  [Multilingual intent and entity
  ///  data](https://cloud.google.com/dialogflow/docs/agents-multilingual#intent-entity).
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchUpdateEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L561}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchUpdateEntities(
      std::string const& parent,
      std::vector<google::cloud::dialogflow::v2::EntityType::Entity> const&
          entities,
      std::string const& language_code, Options opts = {});

  ///
  /// Updates or creates multiple entities in the specified entity type. This
  /// method does not affect entities in the entity type that aren't explicitly
  /// specified in the request.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::BatchUpdateEntitiesRequest,google/cloud/dialogflow/v2/entity_type.proto#L561}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchUpdateEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L561}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchUpdateEntities(
      google::cloud::dialogflow::v2::BatchUpdateEntitiesRequest const& request,
      Options opts = {});

  ///
  /// Deletes entities in the specified entity type.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param parent  Required. The name of the entity type to delete entries
  /// for. Format:
  ///  `projects/<Project ID>/agent/entityTypes/<Entity Type ID>`.
  /// @param entity_values  Required. The reference `values` of the entities to
  /// delete. Note that
  ///  these are not fully-qualified names, i.e. they don't start with
  ///  `projects/<Project ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchDeleteEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L586}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchDeleteEntities(
      std::string const& parent, std::vector<std::string> const& entity_values,
      Options opts = {});

  ///
  /// Deletes entities in the specified entity type.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param parent  Required. The name of the entity type to delete entries
  /// for. Format:
  ///  `projects/<Project ID>/agent/entityTypes/<Entity Type ID>`.
  /// @param entity_values  Required. The reference `values` of the entities to
  /// delete. Note that
  ///  these are not fully-qualified names, i.e. they don't start with
  ///  `projects/<Project ID>`.
  /// @param language_code  Optional. The language used to access
  /// language-specific data.
  ///  If not specified, the agent's default language is used.
  ///  For more information, see
  ///  [Multilingual intent and entity
  ///  data](https://cloud.google.com/dialogflow/docs/agents-multilingual#intent-entity).
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchDeleteEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L586}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchDeleteEntities(
      std::string const& parent, std::vector<std::string> const& entity_values,
      std::string const& language_code, Options opts = {});

  ///
  /// Deletes entities in the specified entity type.
  ///
  /// This method is a [long-running
  /// operation](https://cloud.google.com/dialogflow/es/docs/how/long-running-operations).
  /// The returned `Operation` type has the following method-specific fields:
  ///
  /// - `metadata`: An empty [Struct
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#struct)
  /// - `response`: An [Empty
  ///   message](https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#empty)
  ///
  /// Note: You should always train an agent prior to sending it queries. See
  /// the [training
  /// documentation](https://cloud.google.com/dialogflow/es/docs/training).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::BatchDeleteEntitiesRequest,google/cloud/dialogflow/v2/entity_type.proto#L586}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::protobuf::Struct,google/protobuf/struct.proto#L51}
  ///
  /// [google.cloud.dialogflow.v2.BatchDeleteEntitiesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/entity_type.proto#L586}
  /// [google.protobuf.Struct]:
  /// @googleapis_reference_link{google/protobuf/struct.proto#L51}
  ///
  future<StatusOr<google::protobuf::Struct>> BatchDeleteEntities(
      google::cloud::dialogflow::v2::BatchDeleteEntitiesRequest const& request,
      Options opts = {});

 private:
  std::shared_ptr<EntityTypesConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_es
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_ENTITY_TYPES_CLIENT_H
