// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/v2/knowledge_base.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_KNOWLEDGE_BASES_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_KNOWLEDGE_BASES_CLIENT_H

#include "google/cloud/dialogflow_es/knowledge_bases_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <memory>

namespace google {
namespace cloud {
namespace dialogflow_es {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Service for managing
/// [KnowledgeBases][google.cloud.dialogflow.v2.KnowledgeBase].
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class KnowledgeBasesClient {
 public:
  explicit KnowledgeBasesClient(
      std::shared_ptr<KnowledgeBasesConnection> connection, Options opts = {});
  ~KnowledgeBasesClient();

  ///@{
  // @name Copy and move support
  KnowledgeBasesClient(KnowledgeBasesClient const&) = default;
  KnowledgeBasesClient& operator=(KnowledgeBasesClient const&) = default;
  KnowledgeBasesClient(KnowledgeBasesClient&&) = default;
  KnowledgeBasesClient& operator=(KnowledgeBasesClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(KnowledgeBasesClient const& a,
                         KnowledgeBasesClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(KnowledgeBasesClient const& a,
                         KnowledgeBasesClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Returns the list of all knowledge bases of the specified agent.
  ///
  /// @param parent  Required. The project to list of knowledge bases for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::KnowledgeBase,google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  /// [google.cloud.dialogflow.v2.ListKnowledgeBasesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L152}
  /// [google.cloud.dialogflow.v2.KnowledgeBase]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  StreamRange<google::cloud::dialogflow::v2::KnowledgeBase> ListKnowledgeBases(
      std::string const& parent, Options opts = {});

  ///
  /// Returns the list of all knowledge bases of the specified agent.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::ListKnowledgeBasesRequest,google/cloud/dialogflow/v2/knowledge_base.proto#L152}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::KnowledgeBase,google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  /// [google.cloud.dialogflow.v2.ListKnowledgeBasesRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L152}
  /// [google.cloud.dialogflow.v2.KnowledgeBase]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  StreamRange<google::cloud::dialogflow::v2::KnowledgeBase> ListKnowledgeBases(
      google::cloud::dialogflow::v2::ListKnowledgeBasesRequest request,
      Options opts = {});

  ///
  /// Retrieves the specified knowledge base.
  ///
  /// @param name  Required. The name of the knowledge base to retrieve.
  ///  Format `projects/<Project ID>/locations/<Location
  ///  ID>/knowledgeBases/<Knowledge Base ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::KnowledgeBase,google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  /// [google.cloud.dialogflow.v2.GetKnowledgeBaseRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L209}
  /// [google.cloud.dialogflow.v2.KnowledgeBase]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  StatusOr<google::cloud::dialogflow::v2::KnowledgeBase> GetKnowledgeBase(
      std::string const& name, Options opts = {});

  ///
  /// Retrieves the specified knowledge base.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::GetKnowledgeBaseRequest,google/cloud/dialogflow/v2/knowledge_base.proto#L209}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::KnowledgeBase,google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  /// [google.cloud.dialogflow.v2.GetKnowledgeBaseRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L209}
  /// [google.cloud.dialogflow.v2.KnowledgeBase]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  StatusOr<google::cloud::dialogflow::v2::KnowledgeBase> GetKnowledgeBase(
      google::cloud::dialogflow::v2::GetKnowledgeBaseRequest const& request,
      Options opts = {});

  ///
  /// Creates a knowledge base.
  ///
  /// @param parent  Required. The project to create a knowledge base for.
  ///  Format: `projects/<Project ID>/locations/<Location ID>`.
  /// @param knowledge_base  Required. The knowledge base to create.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::KnowledgeBase,google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  /// [google.cloud.dialogflow.v2.CreateKnowledgeBaseRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L222}
  /// [google.cloud.dialogflow.v2.KnowledgeBase]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  StatusOr<google::cloud::dialogflow::v2::KnowledgeBase> CreateKnowledgeBase(
      std::string const& parent,
      google::cloud::dialogflow::v2::KnowledgeBase const& knowledge_base,
      Options opts = {});

  ///
  /// Creates a knowledge base.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::CreateKnowledgeBaseRequest,google/cloud/dialogflow/v2/knowledge_base.proto#L222}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::KnowledgeBase,google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  /// [google.cloud.dialogflow.v2.CreateKnowledgeBaseRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L222}
  /// [google.cloud.dialogflow.v2.KnowledgeBase]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  StatusOr<google::cloud::dialogflow::v2::KnowledgeBase> CreateKnowledgeBase(
      google::cloud::dialogflow::v2::CreateKnowledgeBaseRequest const& request,
      Options opts = {});

  ///
  /// Deletes the specified knowledge base.
  ///
  /// @param name  Required. The name of the knowledge base to delete.
  ///  Format: `projects/<Project ID>/locations/<Location
  ///  ID>/knowledgeBases/<Knowledge Base ID>`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.v2.DeleteKnowledgeBaseRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L237}
  ///
  Status DeleteKnowledgeBase(std::string const& name, Options opts = {});

  ///
  /// Deletes the specified knowledge base.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::DeleteKnowledgeBaseRequest,google/cloud/dialogflow/v2/knowledge_base.proto#L237}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.dialogflow.v2.DeleteKnowledgeBaseRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L237}
  ///
  Status DeleteKnowledgeBase(
      google::cloud::dialogflow::v2::DeleteKnowledgeBaseRequest const& request,
      Options opts = {});

  ///
  /// Updates the specified knowledge base.
  ///
  /// @param knowledge_base  Required. The knowledge base to update.
  /// @param update_mask  Optional. Not specified means `update all`.
  ///  Currently, only `display_name` can be updated, an InvalidArgument will be
  ///  returned for attempting to update other fields.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::KnowledgeBase,google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  /// [google.cloud.dialogflow.v2.UpdateKnowledgeBaseRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L254}
  /// [google.cloud.dialogflow.v2.KnowledgeBase]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  StatusOr<google::cloud::dialogflow::v2::KnowledgeBase> UpdateKnowledgeBase(
      google::cloud::dialogflow::v2::KnowledgeBase const& knowledge_base,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Updates the specified knowledge base.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::dialogflow::v2::UpdateKnowledgeBaseRequest,google/cloud/dialogflow/v2/knowledge_base.proto#L254}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::dialogflow::v2::KnowledgeBase,google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  /// [google.cloud.dialogflow.v2.UpdateKnowledgeBaseRequest]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L254}
  /// [google.cloud.dialogflow.v2.KnowledgeBase]:
  /// @googleapis_reference_link{google/cloud/dialogflow/v2/knowledge_base.proto#L128}
  ///
  StatusOr<google::cloud::dialogflow::v2::KnowledgeBase> UpdateKnowledgeBase(
      google::cloud::dialogflow::v2::UpdateKnowledgeBaseRequest const& request,
      Options opts = {});

 private:
  std::shared_ptr<KnowledgeBasesConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_es
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_DIALOGFLOW_ES_KNOWLEDGE_BASES_CLIENT_H
