// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/eventarc/v1/eventarc.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_EVENTARC_EVENTARC_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_EVENTARC_EVENTARC_CLIENT_H

#include "google/cloud/eventarc/eventarc_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace eventarc {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Eventarc allows users to subscribe to various events that are provided by
/// Google Cloud services and forward them to supported destinations.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class EventarcClient {
 public:
  explicit EventarcClient(std::shared_ptr<EventarcConnection> connection,
                          Options opts = {});
  ~EventarcClient();

  ///@{
  // @name Copy and move support
  EventarcClient(EventarcClient const&) = default;
  EventarcClient& operator=(EventarcClient const&) = default;
  EventarcClient(EventarcClient&&) = default;
  EventarcClient& operator=(EventarcClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(EventarcClient const& a, EventarcClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(EventarcClient const& a, EventarcClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Get a single trigger.
  ///
  /// @param name  Required. The name of the trigger to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.GetTriggerRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L230}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  StatusOr<google::cloud::eventarc::v1::Trigger> GetTrigger(
      std::string const& name, Options opts = {});

  ///
  /// Get a single trigger.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::GetTriggerRequest,google/cloud/eventarc/v1/eventarc.proto#L230}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.GetTriggerRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L230}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  StatusOr<google::cloud::eventarc::v1::Trigger> GetTrigger(
      google::cloud::eventarc::v1::GetTriggerRequest const& request,
      Options opts = {});

  ///
  /// List triggers.
  ///
  /// @param parent  Required. The parent collection to list triggers on.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.ListTriggersRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L241}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  StreamRange<google::cloud::eventarc::v1::Trigger> ListTriggers(
      std::string const& parent, Options opts = {});

  ///
  /// List triggers.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::ListTriggersRequest,google/cloud/eventarc/v1/eventarc.proto#L241}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.ListTriggersRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L241}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  StreamRange<google::cloud::eventarc::v1::Trigger> ListTriggers(
      google::cloud::eventarc::v1::ListTriggersRequest request,
      Options opts = {});

  ///
  /// Create a new trigger in a particular project and location.
  ///
  /// @param parent  Required. The parent collection in which to add this
  /// trigger.
  /// @param trigger  Required. The trigger to create.
  /// @param trigger_id  Required. The user-provided ID to be assigned to the
  /// trigger.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.CreateTriggerRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L288}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Trigger>> CreateTrigger(
      std::string const& parent,
      google::cloud::eventarc::v1::Trigger const& trigger,
      std::string const& trigger_id, Options opts = {});

  ///
  /// Create a new trigger in a particular project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::CreateTriggerRequest,google/cloud/eventarc/v1/eventarc.proto#L288}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.CreateTriggerRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L288}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Trigger>> CreateTrigger(
      google::cloud::eventarc::v1::CreateTriggerRequest const& request,
      Options opts = {});

  ///
  /// Update a single trigger.
  ///
  /// @param trigger  The trigger to be updated.
  /// @param update_mask  The fields to be updated; only fields explicitly
  /// provided are updated.
  ///  If no field mask is provided, all provided fields in the request are
  ///  updated. To update all fields, provide a field mask of "*".
  /// @param allow_missing  If set to true, and the trigger is not found, a new
  /// trigger will be
  ///  created. In this situation, `update_mask` is ignored.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.UpdateTriggerRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L309}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Trigger>> UpdateTrigger(
      google::cloud::eventarc::v1::Trigger const& trigger,
      google::protobuf::FieldMask const& update_mask, bool allow_missing,
      Options opts = {});

  ///
  /// Update a single trigger.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::UpdateTriggerRequest,google/cloud/eventarc/v1/eventarc.proto#L309}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.UpdateTriggerRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L309}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Trigger>> UpdateTrigger(
      google::cloud::eventarc::v1::UpdateTriggerRequest const& request,
      Options opts = {});

  ///
  /// Delete a single trigger.
  ///
  /// @param name  Required. The name of the trigger to be deleted.
  /// @param allow_missing  If set to true, and the trigger is not found, the
  /// request will succeed
  ///  but no action will be taken on the server.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.DeleteTriggerRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L328}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Trigger>> DeleteTrigger(
      std::string const& name, bool allow_missing, Options opts = {});

  ///
  /// Delete a single trigger.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::DeleteTriggerRequest,google/cloud/eventarc/v1/eventarc.proto#L328}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Trigger,google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  /// [google.cloud.eventarc.v1.DeleteTriggerRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L328}
  /// [google.cloud.eventarc.v1.Trigger]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/trigger.proto#L47}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Trigger>> DeleteTrigger(
      google::cloud::eventarc::v1::DeleteTriggerRequest const& request,
      Options opts = {});

  ///
  /// Get a single Channel.
  ///
  /// @param name  Required. The name of the channel to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.GetChannelRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L351}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  StatusOr<google::cloud::eventarc::v1::Channel> GetChannel(
      std::string const& name, Options opts = {});

  ///
  /// Get a single Channel.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::GetChannelRequest,google/cloud/eventarc/v1/eventarc.proto#L351}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.GetChannelRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L351}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  StatusOr<google::cloud::eventarc::v1::Channel> GetChannel(
      google::cloud::eventarc::v1::GetChannelRequest const& request,
      Options opts = {});

  ///
  /// List channels.
  ///
  /// @param parent  Required. The parent collection to list channels on.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.ListChannelsRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L362}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  StreamRange<google::cloud::eventarc::v1::Channel> ListChannels(
      std::string const& parent, Options opts = {});

  ///
  /// List channels.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::ListChannelsRequest,google/cloud/eventarc/v1/eventarc.proto#L362}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.ListChannelsRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L362}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  StreamRange<google::cloud::eventarc::v1::Channel> ListChannels(
      google::cloud::eventarc::v1::ListChannelsRequest request,
      Options opts = {});

  ///
  /// Create a new channel in a particular project and location.
  ///
  /// @param parent  Required. The parent collection in which to add this
  /// channel.
  /// @param channel  Required. The channel to create.
  /// @param channel_id  Required. The user-provided ID to be assigned to the
  /// channel.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.CreateChannelRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L404}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Channel>> CreateChannel(
      std::string const& parent,
      google::cloud::eventarc::v1::Channel const& channel,
      std::string const& channel_id, Options opts = {});

  ///
  /// Create a new channel in a particular project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::CreateChannelRequest,google/cloud/eventarc/v1/eventarc.proto#L404}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.CreateChannelRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L404}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Channel>> CreateChannel(
      google::cloud::eventarc::v1::CreateChannelRequest const& request,
      Options opts = {});

  ///
  /// Update a single channel.
  ///
  /// @param channel  The channel to be updated.
  /// @param update_mask  The fields to be updated; only fields explicitly
  /// provided are updated.
  ///  If no field mask is provided, all provided fields in the request are
  ///  updated. To update all fields, provide a field mask of "*".
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.UpdateChannelRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L425}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Channel>> UpdateChannel(
      google::cloud::eventarc::v1::Channel const& channel,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Update a single channel.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::UpdateChannelRequest,google/cloud/eventarc/v1/eventarc.proto#L425}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.UpdateChannelRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L425}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Channel>> UpdateChannel(
      google::cloud::eventarc::v1::UpdateChannelRequest const& request,
      Options opts = {});

  ///
  /// Delete a single channel.
  ///
  /// @param name  Required. The name of the channel to be deleted.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.DeleteChannelRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L440}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Channel>> DeleteChannel(
      std::string const& name, Options opts = {});

  ///
  /// Delete a single channel.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::DeleteChannelRequest,google/cloud/eventarc/v1/eventarc.proto#L440}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Channel,google/cloud/eventarc/v1/channel.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.DeleteChannelRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L440}
  /// [google.cloud.eventarc.v1.Channel]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::Channel>> DeleteChannel(
      google::cloud::eventarc::v1::DeleteChannelRequest const& request,
      Options opts = {});

  ///
  /// Get a single Provider.
  ///
  /// @param name  Required. The name of the provider to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Provider,google/cloud/eventarc/v1/discovery.proto#L28}
  ///
  /// [google.cloud.eventarc.v1.GetProviderRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L455}
  /// [google.cloud.eventarc.v1.Provider]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/discovery.proto#L28}
  ///
  StatusOr<google::cloud::eventarc::v1::Provider> GetProvider(
      std::string const& name, Options opts = {});

  ///
  /// Get a single Provider.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::GetProviderRequest,google/cloud/eventarc/v1/eventarc.proto#L455}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Provider,google/cloud/eventarc/v1/discovery.proto#L28}
  ///
  /// [google.cloud.eventarc.v1.GetProviderRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L455}
  /// [google.cloud.eventarc.v1.Provider]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/discovery.proto#L28}
  ///
  StatusOr<google::cloud::eventarc::v1::Provider> GetProvider(
      google::cloud::eventarc::v1::GetProviderRequest const& request,
      Options opts = {});

  ///
  /// List providers.
  ///
  /// @param parent  Required. The parent of the provider to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Provider,google/cloud/eventarc/v1/discovery.proto#L28}
  ///
  /// [google.cloud.eventarc.v1.ListProvidersRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L466}
  /// [google.cloud.eventarc.v1.Provider]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/discovery.proto#L28}
  ///
  StreamRange<google::cloud::eventarc::v1::Provider> ListProviders(
      std::string const& parent, Options opts = {});

  ///
  /// List providers.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::ListProvidersRequest,google/cloud/eventarc/v1/eventarc.proto#L466}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::Provider,google/cloud/eventarc/v1/discovery.proto#L28}
  ///
  /// [google.cloud.eventarc.v1.ListProvidersRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L466}
  /// [google.cloud.eventarc.v1.Provider]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/discovery.proto#L28}
  ///
  StreamRange<google::cloud::eventarc::v1::Provider> ListProviders(
      google::cloud::eventarc::v1::ListProvidersRequest request,
      Options opts = {});

  ///
  /// Get a single ChannelConnection.
  ///
  /// @param name  Required. The name of the channel connection to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::ChannelConnection,google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.GetChannelConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L509}
  /// [google.cloud.eventarc.v1.ChannelConnection]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  StatusOr<google::cloud::eventarc::v1::ChannelConnection> GetChannelConnection(
      std::string const& name, Options opts = {});

  ///
  /// Get a single ChannelConnection.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::GetChannelConnectionRequest,google/cloud/eventarc/v1/eventarc.proto#L509}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::ChannelConnection,google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.GetChannelConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L509}
  /// [google.cloud.eventarc.v1.ChannelConnection]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  StatusOr<google::cloud::eventarc::v1::ChannelConnection> GetChannelConnection(
      google::cloud::eventarc::v1::GetChannelConnectionRequest const& request,
      Options opts = {});

  ///
  /// List channel connections.
  ///
  /// @param parent  Required. The parent collection from which to list channel
  /// connections.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::ChannelConnection,google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.ListChannelConnectionsRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L520}
  /// [google.cloud.eventarc.v1.ChannelConnection]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  StreamRange<google::cloud::eventarc::v1::ChannelConnection>
  ListChannelConnections(std::string const& parent, Options opts = {});

  ///
  /// List channel connections.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::ListChannelConnectionsRequest,google/cloud/eventarc/v1/eventarc.proto#L520}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::ChannelConnection,google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.ListChannelConnectionsRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L520}
  /// [google.cloud.eventarc.v1.ChannelConnection]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  StreamRange<google::cloud::eventarc::v1::ChannelConnection>
  ListChannelConnections(
      google::cloud::eventarc::v1::ListChannelConnectionsRequest request,
      Options opts = {});

  ///
  /// Create a new ChannelConnection in a particular project and location.
  ///
  /// @param parent  Required. The parent collection in which to add this
  /// channel connection.
  /// @param channel_connection  Required. Channel connection to create.
  /// @param channel_connection_id  Required. The user-provided ID to be
  /// assigned to the channel connection.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::ChannelConnection,google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.CreateChannelConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L558}
  /// [google.cloud.eventarc.v1.ChannelConnection]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::ChannelConnection>>
  CreateChannelConnection(
      std::string const& parent,
      google::cloud::eventarc::v1::ChannelConnection const& channel_connection,
      std::string const& channel_connection_id, Options opts = {});

  ///
  /// Create a new ChannelConnection in a particular project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::CreateChannelConnectionRequest,google/cloud/eventarc/v1/eventarc.proto#L558}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::ChannelConnection,google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.CreateChannelConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L558}
  /// [google.cloud.eventarc.v1.ChannelConnection]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::ChannelConnection>>
  CreateChannelConnection(
      google::cloud::eventarc::v1::CreateChannelConnectionRequest const&
          request,
      Options opts = {});

  ///
  /// Delete a single ChannelConnection.
  ///
  /// @param name  Required. The name of the channel connection to delete.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::ChannelConnection,google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.DeleteChannelConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L575}
  /// [google.cloud.eventarc.v1.ChannelConnection]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::ChannelConnection>>
  DeleteChannelConnection(std::string const& name, Options opts = {});

  ///
  /// Delete a single ChannelConnection.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::DeleteChannelConnectionRequest,google/cloud/eventarc/v1/eventarc.proto#L575}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::ChannelConnection,google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  /// [google.cloud.eventarc.v1.DeleteChannelConnectionRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L575}
  /// [google.cloud.eventarc.v1.ChannelConnection]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/channel_connection.proto#L35}
  ///
  future<StatusOr<google::cloud::eventarc::v1::ChannelConnection>>
  DeleteChannelConnection(
      google::cloud::eventarc::v1::DeleteChannelConnectionRequest const&
          request,
      Options opts = {});

  ///
  /// Get a GoogleChannelConfig
  ///
  /// @param name  Required. The name of the config to get.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::GoogleChannelConfig,google/cloud/eventarc/v1/google_channel_config.proto#L40}
  ///
  /// [google.cloud.eventarc.v1.GetGoogleChannelConfigRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L597}
  /// [google.cloud.eventarc.v1.GoogleChannelConfig]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/google_channel_config.proto#L40}
  ///
  StatusOr<google::cloud::eventarc::v1::GoogleChannelConfig>
  GetGoogleChannelConfig(std::string const& name, Options opts = {});

  ///
  /// Get a GoogleChannelConfig
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::GetGoogleChannelConfigRequest,google/cloud/eventarc/v1/eventarc.proto#L597}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::GoogleChannelConfig,google/cloud/eventarc/v1/google_channel_config.proto#L40}
  ///
  /// [google.cloud.eventarc.v1.GetGoogleChannelConfigRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L597}
  /// [google.cloud.eventarc.v1.GoogleChannelConfig]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/google_channel_config.proto#L40}
  ///
  StatusOr<google::cloud::eventarc::v1::GoogleChannelConfig>
  GetGoogleChannelConfig(
      google::cloud::eventarc::v1::GetGoogleChannelConfigRequest const& request,
      Options opts = {});

  ///
  /// Update a single GoogleChannelConfig
  ///
  /// @param google_channel_config  Required. The config to be updated.
  /// @param update_mask  The fields to be updated; only fields explicitly
  /// provided are updated.
  ///  If no field mask is provided, all provided fields in the request are
  ///  updated. To update all fields, provide a field mask of "*".
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::GoogleChannelConfig,google/cloud/eventarc/v1/google_channel_config.proto#L40}
  ///
  /// [google.cloud.eventarc.v1.UpdateGoogleChannelConfigRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L586}
  /// [google.cloud.eventarc.v1.GoogleChannelConfig]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/google_channel_config.proto#L40}
  ///
  StatusOr<google::cloud::eventarc::v1::GoogleChannelConfig>
  UpdateGoogleChannelConfig(
      google::cloud::eventarc::v1::GoogleChannelConfig const&
          google_channel_config,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Update a single GoogleChannelConfig
  ///
  /// @param request
  /// @googleapis_link{google::cloud::eventarc::v1::UpdateGoogleChannelConfigRequest,google/cloud/eventarc/v1/eventarc.proto#L586}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::eventarc::v1::GoogleChannelConfig,google/cloud/eventarc/v1/google_channel_config.proto#L40}
  ///
  /// [google.cloud.eventarc.v1.UpdateGoogleChannelConfigRequest]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/eventarc.proto#L586}
  /// [google.cloud.eventarc.v1.GoogleChannelConfig]:
  /// @googleapis_reference_link{google/cloud/eventarc/v1/google_channel_config.proto#L40}
  ///
  StatusOr<google::cloud::eventarc::v1::GoogleChannelConfig>
  UpdateGoogleChannelConfig(
      google::cloud::eventarc::v1::UpdateGoogleChannelConfigRequest const&
          request,
      Options opts = {});

 private:
  std::shared_ptr<EventarcConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace eventarc
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_EVENTARC_EVENTARC_CLIENT_H
