// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/video/transcoder/v1/services.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VIDEO_TRANSCODER_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VIDEO_TRANSCODER_CLIENT_H

#include "google/cloud/video/transcoder_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace video {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// Using the Transcoder API, you can queue asynchronous jobs for transcoding
/// media into various output formats. Output formats may include different
/// streaming standards such as HTTP Live Streaming (HLS) and Dynamic Adaptive
/// Streaming over HTTP (DASH). You can also customize jobs using advanced
/// features such as Digital Rights Management (DRM), audio equalization,
/// content concatenation, and digital ad-stitch ready content generation.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class TranscoderServiceClient {
 public:
  explicit TranscoderServiceClient(
      std::shared_ptr<TranscoderServiceConnection> connection,
      Options opts = {});
  ~TranscoderServiceClient();

  ///@{
  // @name Copy and move support
  TranscoderServiceClient(TranscoderServiceClient const&) = default;
  TranscoderServiceClient& operator=(TranscoderServiceClient const&) = default;
  TranscoderServiceClient(TranscoderServiceClient&&) = default;
  TranscoderServiceClient& operator=(TranscoderServiceClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(TranscoderServiceClient const& a,
                         TranscoderServiceClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(TranscoderServiceClient const& a,
                         TranscoderServiceClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Creates a job in the specified region.
  ///
  /// @param parent  Required. The parent location to create and process this
  /// job.
  ///  Format: `projects/{project}/locations/{location}`
  /// @param job  Required. Parameters for creating transcoding job.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::Job,google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  /// [google.cloud.video.transcoder.v1.CreateJobRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L112}
  /// [google.cloud.video.transcoder.v1.Job]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  StatusOr<google::cloud::video::transcoder::v1::Job> CreateJob(
      std::string const& parent,
      google::cloud::video::transcoder::v1::Job const& job, Options opts = {});

  ///
  /// Creates a job in the specified region.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::video::transcoder::v1::CreateJobRequest,google/cloud/video/transcoder/v1/services.proto#L112}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::Job,google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  /// [google.cloud.video.transcoder.v1.CreateJobRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L112}
  /// [google.cloud.video.transcoder.v1.Job]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  StatusOr<google::cloud::video::transcoder::v1::Job> CreateJob(
      google::cloud::video::transcoder::v1::CreateJobRequest const& request,
      Options opts = {});

  ///
  /// Lists jobs in the specified region.
  ///
  /// @param parent  Required. Format: `projects/{project}/locations/{location}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::Job,google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  /// [google.cloud.video.transcoder.v1.ListJobsRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L128}
  /// [google.cloud.video.transcoder.v1.Job]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  StreamRange<google::cloud::video::transcoder::v1::Job> ListJobs(
      std::string const& parent, Options opts = {});

  ///
  /// Lists jobs in the specified region.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::video::transcoder::v1::ListJobsRequest,google/cloud/video/transcoder/v1/services.proto#L128}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::Job,google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  /// [google.cloud.video.transcoder.v1.ListJobsRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L128}
  /// [google.cloud.video.transcoder.v1.Job]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  StreamRange<google::cloud::video::transcoder::v1::Job> ListJobs(
      google::cloud::video::transcoder::v1::ListJobsRequest request,
      Options opts = {});

  ///
  /// Returns the job data.
  ///
  /// @param name  Required. The name of the job to retrieve.
  ///  Format: `projects/{project}/locations/{location}/jobs/{job}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::Job,google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  /// [google.cloud.video.transcoder.v1.GetJobRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L154}
  /// [google.cloud.video.transcoder.v1.Job]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  StatusOr<google::cloud::video::transcoder::v1::Job> GetJob(
      std::string const& name, Options opts = {});

  ///
  /// Returns the job data.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::video::transcoder::v1::GetJobRequest,google/cloud/video/transcoder/v1/services.proto#L154}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::Job,google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  /// [google.cloud.video.transcoder.v1.GetJobRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L154}
  /// [google.cloud.video.transcoder.v1.Job]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L31}
  ///
  StatusOr<google::cloud::video::transcoder::v1::Job> GetJob(
      google::cloud::video::transcoder::v1::GetJobRequest const& request,
      Options opts = {});

  ///
  /// Deletes a job.
  ///
  /// @param name  Required. The name of the job to delete.
  ///  Format: `projects/{project}/locations/{location}/jobs/{job}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.video.transcoder.v1.DeleteJobRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L166}
  ///
  Status DeleteJob(std::string const& name, Options opts = {});

  ///
  /// Deletes a job.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::video::transcoder::v1::DeleteJobRequest,google/cloud/video/transcoder/v1/services.proto#L166}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.video.transcoder.v1.DeleteJobRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L166}
  ///
  Status DeleteJob(
      google::cloud::video::transcoder::v1::DeleteJobRequest const& request,
      Options opts = {});

  ///
  /// Creates a job template in the specified region.
  ///
  /// @param parent  Required. The parent location to create this job template.
  ///  Format: `projects/{project}/locations/{location}`
  /// @param job_template  Required. Parameters for creating job template.
  /// @param job_template_id  Required. The ID to use for the job template,
  /// which will become the final component
  ///  of the job template's resource name.
  ///  This value should be 4-63 characters, and valid characters must match the
  ///  regular expression `[a-zA-Z][a-zA-Z0-9_-]*`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::JobTemplate,google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  /// [google.cloud.video.transcoder.v1.CreateJobTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L194}
  /// [google.cloud.video.transcoder.v1.JobTemplate]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  StatusOr<google::cloud::video::transcoder::v1::JobTemplate> CreateJobTemplate(
      std::string const& parent,
      google::cloud::video::transcoder::v1::JobTemplate const& job_template,
      std::string const& job_template_id, Options opts = {});

  ///
  /// Creates a job template in the specified region.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::video::transcoder::v1::CreateJobTemplateRequest,google/cloud/video/transcoder/v1/services.proto#L194}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::JobTemplate,google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  /// [google.cloud.video.transcoder.v1.CreateJobTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L194}
  /// [google.cloud.video.transcoder.v1.JobTemplate]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  StatusOr<google::cloud::video::transcoder::v1::JobTemplate> CreateJobTemplate(
      google::cloud::video::transcoder::v1::CreateJobTemplateRequest const&
          request,
      Options opts = {});

  ///
  /// Lists job templates in the specified region.
  ///
  /// @param parent  Required. The parent location from which to retrieve the
  /// collection of job templates.
  ///  Format: `projects/{project}/locations/{location}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::JobTemplate,google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  /// [google.cloud.video.transcoder.v1.ListJobTemplatesRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L216}
  /// [google.cloud.video.transcoder.v1.JobTemplate]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  StreamRange<google::cloud::video::transcoder::v1::JobTemplate>
  ListJobTemplates(std::string const& parent, Options opts = {});

  ///
  /// Lists job templates in the specified region.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::video::transcoder::v1::ListJobTemplatesRequest,google/cloud/video/transcoder/v1/services.proto#L216}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::JobTemplate,google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  /// [google.cloud.video.transcoder.v1.ListJobTemplatesRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L216}
  /// [google.cloud.video.transcoder.v1.JobTemplate]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  StreamRange<google::cloud::video::transcoder::v1::JobTemplate>
  ListJobTemplates(
      google::cloud::video::transcoder::v1::ListJobTemplatesRequest request,
      Options opts = {});

  ///
  /// Returns the job template data.
  ///
  /// @param name  Required. The name of the job template to retrieve.
  ///  Format:
  ///  `projects/{project}/locations/{location}/jobTemplates/{job_template}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::JobTemplate,google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  /// [google.cloud.video.transcoder.v1.GetJobTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L243}
  /// [google.cloud.video.transcoder.v1.JobTemplate]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  StatusOr<google::cloud::video::transcoder::v1::JobTemplate> GetJobTemplate(
      std::string const& name, Options opts = {});

  ///
  /// Returns the job template data.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::video::transcoder::v1::GetJobTemplateRequest,google/cloud/video/transcoder/v1/services.proto#L243}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::video::transcoder::v1::JobTemplate,google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  /// [google.cloud.video.transcoder.v1.GetJobTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L243}
  /// [google.cloud.video.transcoder.v1.JobTemplate]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/resources.proto#L121}
  ///
  StatusOr<google::cloud::video::transcoder::v1::JobTemplate> GetJobTemplate(
      google::cloud::video::transcoder::v1::GetJobTemplateRequest const&
          request,
      Options opts = {});

  ///
  /// Deletes a job template.
  ///
  /// @param name  Required. The name of the job template to delete.
  ///  `projects/{project}/locations/{location}/jobTemplates/{job_template}`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.video.transcoder.v1.DeleteJobTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L256}
  ///
  Status DeleteJobTemplate(std::string const& name, Options opts = {});

  ///
  /// Deletes a job template.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::video::transcoder::v1::DeleteJobTemplateRequest,google/cloud/video/transcoder/v1/services.proto#L256}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  ///
  /// [google.cloud.video.transcoder.v1.DeleteJobTemplateRequest]:
  /// @googleapis_reference_link{google/cloud/video/transcoder/v1/services.proto#L256}
  ///
  Status DeleteJobTemplate(
      google::cloud::video::transcoder::v1::DeleteJobTemplateRequest const&
          request,
      Options opts = {});

 private:
  std::shared_ptr<TranscoderServiceConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace video
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VIDEO_TRANSCODER_CLIENT_H
