// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/vmwareengine/v1/vmwareengine.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VMWAREENGINE_V1_VMWARE_ENGINE_CLIENT_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VMWAREENGINE_V1_VMWARE_ENGINE_CLIENT_H

#include "google/cloud/vmwareengine/v1/vmware_engine_connection.h"
#include "google/cloud/future.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/longrunning/operations.grpc.pb.h>
#include <map>
#include <memory>

namespace google {
namespace cloud {
namespace vmwareengine_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

///
/// VMwareEngine manages VMware's private clusters in the Cloud.
///
/// @par Equality
///
/// Instances of this class created via copy-construction or copy-assignment
/// always compare equal. Instances created with equal
/// `std::shared_ptr<*Connection>` objects compare equal. Objects that compare
/// equal share the same underlying resources.
///
/// @par Performance
///
/// Creating a new instance of this class is a relatively expensive operation,
/// new objects establish new connections to the service. In contrast,
/// copy-construction, move-construction, and the corresponding assignment
/// operations are relatively efficient as the copies share all underlying
/// resources.
///
/// @par Thread Safety
///
/// Concurrent access to different instances of this class, even if they compare
/// equal, is guaranteed to work. Two or more threads operating on the same
/// instance of this class is not guaranteed to work. Since copy-construction
/// and move-construction is a relatively efficient operation, consider using
/// such a copy when using this class from multiple threads.
///
class VmwareEngineClient {
 public:
  explicit VmwareEngineClient(
      std::shared_ptr<VmwareEngineConnection> connection, Options opts = {});
  ~VmwareEngineClient();

  ///@{
  // @name Copy and move support
  VmwareEngineClient(VmwareEngineClient const&) = default;
  VmwareEngineClient& operator=(VmwareEngineClient const&) = default;
  VmwareEngineClient(VmwareEngineClient&&) = default;
  VmwareEngineClient& operator=(VmwareEngineClient&&) = default;
  ///@}

  ///@{
  // @name Equality
  friend bool operator==(VmwareEngineClient const& a,
                         VmwareEngineClient const& b) {
    return a.connection_ == b.connection_;
  }
  friend bool operator!=(VmwareEngineClient const& a,
                         VmwareEngineClient const& b) {
    return !(a == b);
  }
  ///@}

  ///
  /// Lists `PrivateCloud` resources in a given project and location.
  ///
  /// @param parent  Required. The resource name of the private cloud to be
  /// queried for
  ///  clusters. Resource names are schemeless URIs that follow the conventions
  ///  in https://cloud.google.com/apis/design/resource_names. For example:
  ///  `projects/my-project/locations/us-west1-a`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.ListPrivateCloudsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L583}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  StreamRange<google::cloud::vmwareengine::v1::PrivateCloud> ListPrivateClouds(
      std::string const& parent, Options opts = {});

  ///
  /// Lists `PrivateCloud` resources in a given project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ListPrivateCloudsRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L583}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.ListPrivateCloudsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L583}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  StreamRange<google::cloud::vmwareengine::v1::PrivateCloud> ListPrivateClouds(
      google::cloud::vmwareengine::v1::ListPrivateCloudsRequest request,
      Options opts = {});

  ///
  /// Retrieves a `PrivateCloud` resource by its resource name.
  ///
  /// @param name  Required. The resource name of the private cloud to retrieve.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.GetPrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L660}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  StatusOr<google::cloud::vmwareengine::v1::PrivateCloud> GetPrivateCloud(
      std::string const& name, Options opts = {});

  ///
  /// Retrieves a `PrivateCloud` resource by its resource name.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::GetPrivateCloudRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L660}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.GetPrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L660}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  StatusOr<google::cloud::vmwareengine::v1::PrivateCloud> GetPrivateCloud(
      google::cloud::vmwareengine::v1::GetPrivateCloudRequest const& request,
      Options opts = {});

  ///
  /// Creates a new `PrivateCloud` resource in a given project and location.
  /// Private clouds can only be created in zones, regional private clouds are
  /// not supported.
  ///
  /// Creating a private cloud also creates a [management
  /// cluster](https://cloud.google.com/vmware-engine/docs/concepts-vmware-components)
  /// for that private cloud.
  ///
  /// @param parent  Required. The resource name of the location to create the
  /// new
  ///  private cloud in. Resource names are schemeless URIs that follow the
  ///  conventions in https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a`
  /// @param private_cloud  Required. The initial description of the new private
  /// cloud.
  /// @param private_cloud_id  Required. The user-provided identifier of the
  /// private cloud to be created.
  ///  This identifier must be unique among each `PrivateCloud` within the
  ///  parent and becomes the final token in the name URI. The identifier must
  ///  meet the following requirements:
  ///  * Only contains 1-63 alphanumeric characters and hyphens
  ///  * Begins with an alphabetical character
  ///  * Ends with a non-hyphen character
  ///  * Not formatted as a UUID
  ///  * Complies with [RFC 1034](https://datatracker.ietf.org/doc/html/rfc1034)
  ///  (section 3.5)
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.CreatePrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L675}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  CreatePrivateCloud(
      std::string const& parent,
      google::cloud::vmwareengine::v1::PrivateCloud const& private_cloud,
      std::string const& private_cloud_id, Options opts = {});

  ///
  /// Creates a new `PrivateCloud` resource in a given project and location.
  /// Private clouds can only be created in zones, regional private clouds are
  /// not supported.
  ///
  /// Creating a private cloud also creates a [management
  /// cluster](https://cloud.google.com/vmware-engine/docs/concepts-vmware-components)
  /// for that private cloud.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::CreatePrivateCloudRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L675}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.CreatePrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L675}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  CreatePrivateCloud(
      google::cloud::vmwareengine::v1::CreatePrivateCloudRequest const& request,
      Options opts = {});

  ///
  /// Modifies a `PrivateCloud` resource. Only the following fields can be
  /// updated: `description`.
  /// Only fields specified in `updateMask` are applied.
  ///
  /// During operation processing, the resource is temporarily in the `ACTIVE`
  /// state before the operation fully completes. For that period of time, you
  /// can't update the resource. Use the operation status to determine when the
  /// processing fully completes.
  ///
  /// @param private_cloud  Required. Private cloud description.
  /// @param update_mask  Required. Field mask is used to specify the fields to
  /// be overwritten in the
  ///  `PrivateCloud` resource by the update. The fields specified in
  ///  `updateMask` are relative to the resource, not the full request. A field
  ///  will be overwritten if it is in the mask. If the user does not provide a
  ///  mask then all fields will be overwritten.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.UpdatePrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L714}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  UpdatePrivateCloud(
      google::cloud::vmwareengine::v1::PrivateCloud const& private_cloud,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Modifies a `PrivateCloud` resource. Only the following fields can be
  /// updated: `description`.
  /// Only fields specified in `updateMask` are applied.
  ///
  /// During operation processing, the resource is temporarily in the `ACTIVE`
  /// state before the operation fully completes. For that period of time, you
  /// can't update the resource. Use the operation status to determine when the
  /// processing fully completes.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::UpdatePrivateCloudRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L714}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.UpdatePrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L714}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  UpdatePrivateCloud(
      google::cloud::vmwareengine::v1::UpdatePrivateCloudRequest const& request,
      Options opts = {});

  ///
  /// Schedules a `PrivateCloud` resource for deletion.
  ///
  /// A `PrivateCloud` resource scheduled for deletion has `PrivateCloud.state`
  /// set to `DELETED` and `expireTime` set to the time when deletion is final
  /// and can no longer be reversed. The delete operation is marked as done
  /// as soon as the `PrivateCloud` is successfully scheduled for deletion
  /// (this also applies when `delayHours` is set to zero), and the operation is
  /// not kept in pending state until `PrivateCloud` is purged.
  /// `PrivateCloud` can be restored using `UndeletePrivateCloud` method before
  /// the `expireTime` elapses. When `expireTime` is reached, deletion is final
  /// and all private cloud resources are irreversibly removed and billing
  /// stops. During the final removal process, `PrivateCloud.state` is set to
  /// `PURGING`. `PrivateCloud` can be polled using standard `GET` method for
  /// the whole period of deletion and purging. It will not be returned only
  /// when it is completely purged.
  ///
  /// @param name  Required. The resource name of the private cloud to delete.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.DeletePrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L731}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  DeletePrivateCloud(std::string const& name, Options opts = {});

  ///
  /// Schedules a `PrivateCloud` resource for deletion.
  ///
  /// A `PrivateCloud` resource scheduled for deletion has `PrivateCloud.state`
  /// set to `DELETED` and `expireTime` set to the time when deletion is final
  /// and can no longer be reversed. The delete operation is marked as done
  /// as soon as the `PrivateCloud` is successfully scheduled for deletion
  /// (this also applies when `delayHours` is set to zero), and the operation is
  /// not kept in pending state until `PrivateCloud` is purged.
  /// `PrivateCloud` can be restored using `UndeletePrivateCloud` method before
  /// the `expireTime` elapses. When `expireTime` is reached, deletion is final
  /// and all private cloud resources are irreversibly removed and billing
  /// stops. During the final removal process, `PrivateCloud.state` is set to
  /// `PURGING`. `PrivateCloud` can be polled using standard `GET` method for
  /// the whole period of deletion and purging. It will not be returned only
  /// when it is completely purged.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::DeletePrivateCloudRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L731}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.DeletePrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L731}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  DeletePrivateCloud(
      google::cloud::vmwareengine::v1::DeletePrivateCloudRequest const& request,
      Options opts = {});

  ///
  /// Restores a private cloud that was previously scheduled for deletion by
  /// `DeletePrivateCloud`. A `PrivateCloud` resource scheduled for deletion has
  /// `PrivateCloud.state` set to `DELETED` and `PrivateCloud.expireTime` set to
  /// the time when deletion can no longer be reversed.
  ///
  /// @param name  Required. The resource name of the private cloud scheduled
  /// for deletion.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.UndeletePrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L766}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  UndeletePrivateCloud(std::string const& name, Options opts = {});

  ///
  /// Restores a private cloud that was previously scheduled for deletion by
  /// `DeletePrivateCloud`. A `PrivateCloud` resource scheduled for deletion has
  /// `PrivateCloud.state` set to `DELETED` and `PrivateCloud.expireTime` set to
  /// the time when deletion can no longer be reversed.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::UndeletePrivateCloudRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L766}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.UndeletePrivateCloudRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L766}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  UndeletePrivateCloud(
      google::cloud::vmwareengine::v1::UndeletePrivateCloudRequest const&
          request,
      Options opts = {});

  ///
  /// Lists `Cluster` resources in a given private cloud.
  ///
  /// @param parent  Required. The resource name of the private cloud to query
  /// for clusters.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Cluster,google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  /// [google.cloud.vmwareengine.v1.ListClustersRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L844}
  /// [google.cloud.vmwareengine.v1.Cluster]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  StreamRange<google::cloud::vmwareengine::v1::Cluster> ListClusters(
      std::string const& parent, Options opts = {});

  ///
  /// Lists `Cluster` resources in a given private cloud.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ListClustersRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L844}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Cluster,google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  /// [google.cloud.vmwareengine.v1.ListClustersRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L844}
  /// [google.cloud.vmwareengine.v1.Cluster]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  StreamRange<google::cloud::vmwareengine::v1::Cluster> ListClusters(
      google::cloud::vmwareengine::v1::ListClustersRequest request,
      Options opts = {});

  ///
  /// Retrieves a `Cluster` resource by its resource name.
  ///
  /// @param name  Required. The cluster resource name to retrieve.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud/clusters/my-cluster`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Cluster,google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  /// [google.cloud.vmwareengine.v1.GetClusterRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L909}
  /// [google.cloud.vmwareengine.v1.Cluster]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  StatusOr<google::cloud::vmwareengine::v1::Cluster> GetCluster(
      std::string const& name, Options opts = {});

  ///
  /// Retrieves a `Cluster` resource by its resource name.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::GetClusterRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L909}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Cluster,google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  /// [google.cloud.vmwareengine.v1.GetClusterRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L909}
  /// [google.cloud.vmwareengine.v1.Cluster]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  StatusOr<google::cloud::vmwareengine::v1::Cluster> GetCluster(
      google::cloud::vmwareengine::v1::GetClusterRequest const& request,
      Options opts = {});

  ///
  /// Creates a new cluster in a given private cloud.
  /// Creating a new cluster provides additional nodes for
  /// use in the parent private cloud and requires sufficient [node
  /// quota](https://cloud.google.com/vmware-engine/quotas).
  ///
  /// @param parent  Required. The resource name of the private cloud to create
  /// a new cluster
  ///  in. Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param cluster  Required. The initial description of the new cluster.
  /// @param cluster_id  Required. The user-provided identifier of the new
  /// `Cluster`.
  ///  This identifier must be unique among clusters within the parent and
  ///  becomes the final token in the name URI. The identifier must meet the
  ///  following requirements:
  ///  * Only contains 1-63 alphanumeric characters and hyphens
  ///  * Begins with an alphabetical character
  ///  * Ends with a non-hyphen character
  ///  * Not formatted as a UUID
  ///  * Complies with [RFC 1034](https://datatracker.ietf.org/doc/html/rfc1034)
  ///  (section 3.5)
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Cluster,google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  /// [google.cloud.vmwareengine.v1.CreateClusterRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L924}
  /// [google.cloud.vmwareengine.v1.Cluster]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::Cluster>> CreateCluster(
      std::string const& parent,
      google::cloud::vmwareengine::v1::Cluster const& cluster,
      std::string const& cluster_id, Options opts = {});

  ///
  /// Creates a new cluster in a given private cloud.
  /// Creating a new cluster provides additional nodes for
  /// use in the parent private cloud and requires sufficient [node
  /// quota](https://cloud.google.com/vmware-engine/quotas).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::CreateClusterRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L924}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Cluster,google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  /// [google.cloud.vmwareengine.v1.CreateClusterRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L924}
  /// [google.cloud.vmwareengine.v1.Cluster]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::Cluster>> CreateCluster(
      google::cloud::vmwareengine::v1::CreateClusterRequest const& request,
      Options opts = {});

  ///
  /// Modifies a `Cluster` resource. Only the following fields can be updated:
  /// `node_type_configs.*.node_count`. Only fields specified in `updateMask`
  /// are applied.
  ///
  /// During operation processing, the resource is temporarily in the `ACTIVE`
  /// state before the operation fully completes. For that period of time, you
  /// can't update the resource. Use the operation status to determine when the
  /// processing fully completes.
  ///
  /// @param cluster  Required. The description of the cluster.
  /// @param update_mask  Required. Field mask is used to specify the fields to
  /// be overwritten in the
  ///  `Cluster` resource by the update. The fields specified in the
  ///  `updateMask` are relative to the resource, not the full request. A field
  ///  will be overwritten if it is in the mask. If the user does not provide a
  ///  mask then all fields will be overwritten.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Cluster,google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  /// [google.cloud.vmwareengine.v1.UpdateClusterRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L963}
  /// [google.cloud.vmwareengine.v1.Cluster]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::Cluster>> UpdateCluster(
      google::cloud::vmwareengine::v1::Cluster const& cluster,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Modifies a `Cluster` resource. Only the following fields can be updated:
  /// `node_type_configs.*.node_count`. Only fields specified in `updateMask`
  /// are applied.
  ///
  /// During operation processing, the resource is temporarily in the `ACTIVE`
  /// state before the operation fully completes. For that period of time, you
  /// can't update the resource. Use the operation status to determine when the
  /// processing fully completes.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::UpdateClusterRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L963}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Cluster,google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  /// [google.cloud.vmwareengine.v1.UpdateClusterRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L963}
  /// [google.cloud.vmwareengine.v1.Cluster]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L785}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::Cluster>> UpdateCluster(
      google::cloud::vmwareengine::v1::UpdateClusterRequest const& request,
      Options opts = {});

  ///
  /// Deletes a `Cluster` resource. To avoid unintended data loss, migrate or
  /// gracefully shut down any workloads running on the cluster before deletion.
  /// You cannot delete the management cluster of a private cloud using this
  /// method.
  ///
  /// @param name  Required. The resource name of the cluster to delete.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud/clusters/my-cluster`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::OperationMetadata,google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  /// [google.cloud.vmwareengine.v1.DeleteClusterRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L984}
  /// [google.cloud.vmwareengine.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::OperationMetadata>>
  DeleteCluster(std::string const& name, Options opts = {});

  ///
  /// Deletes a `Cluster` resource. To avoid unintended data loss, migrate or
  /// gracefully shut down any workloads running on the cluster before deletion.
  /// You cannot delete the management cluster of a private cloud using this
  /// method.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::DeleteClusterRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L984}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::OperationMetadata,google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  /// [google.cloud.vmwareengine.v1.DeleteClusterRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L984}
  /// [google.cloud.vmwareengine.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::OperationMetadata>>
  DeleteCluster(
      google::cloud::vmwareengine::v1::DeleteClusterRequest const& request,
      Options opts = {});

  ///
  /// Lists subnets in a given private cloud.
  ///
  /// @param parent  Required. The resource name of the private cloud to be
  /// queried for
  ///  subnets.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Subnet,google/cloud/vmwareengine/v1/vmwareengine.proto#L1004}
  ///
  /// [google.cloud.vmwareengine.v1.ListSubnetsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1050}
  /// [google.cloud.vmwareengine.v1.Subnet]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1004}
  ///
  StreamRange<google::cloud::vmwareengine::v1::Subnet> ListSubnets(
      std::string const& parent, Options opts = {});

  ///
  /// Lists subnets in a given private cloud.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ListSubnetsRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1050}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Subnet,google/cloud/vmwareengine/v1/vmwareengine.proto#L1004}
  ///
  /// [google.cloud.vmwareengine.v1.ListSubnetsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1050}
  /// [google.cloud.vmwareengine.v1.Subnet]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1004}
  ///
  StreamRange<google::cloud::vmwareengine::v1::Subnet> ListSubnets(
      google::cloud::vmwareengine::v1::ListSubnetsRequest request,
      Options opts = {});

  ///
  /// Lists node types
  ///
  /// @param parent  Required. The resource name of the location to be queried
  /// for node types.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NodeType,google/cloud/vmwareengine/v1/vmwareengine.proto#L1117}
  ///
  /// [google.cloud.vmwareengine.v1.ListNodeTypesRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1155}
  /// [google.cloud.vmwareengine.v1.NodeType]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1117}
  ///
  StreamRange<google::cloud::vmwareengine::v1::NodeType> ListNodeTypes(
      std::string const& parent, Options opts = {});

  ///
  /// Lists node types
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ListNodeTypesRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1155}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NodeType,google/cloud/vmwareengine/v1/vmwareengine.proto#L1117}
  ///
  /// [google.cloud.vmwareengine.v1.ListNodeTypesRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1155}
  /// [google.cloud.vmwareengine.v1.NodeType]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1117}
  ///
  StreamRange<google::cloud::vmwareengine::v1::NodeType> ListNodeTypes(
      google::cloud::vmwareengine::v1::ListNodeTypesRequest request,
      Options opts = {});

  ///
  /// Gets details of a single `NodeType`.
  ///
  /// @param name  Required. The resource name of the node type to retrieve.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-proj/locations/us-west1-a/nodeTypes/standard-72`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NodeType,google/cloud/vmwareengine/v1/vmwareengine.proto#L1117}
  ///
  /// [google.cloud.vmwareengine.v1.GetNodeTypeRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1224}
  /// [google.cloud.vmwareengine.v1.NodeType]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1117}
  ///
  StatusOr<google::cloud::vmwareengine::v1::NodeType> GetNodeType(
      std::string const& name, Options opts = {});

  ///
  /// Gets details of a single `NodeType`.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::GetNodeTypeRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1224}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NodeType,google/cloud/vmwareengine/v1/vmwareengine.proto#L1117}
  ///
  /// [google.cloud.vmwareengine.v1.GetNodeTypeRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1224}
  /// [google.cloud.vmwareengine.v1.NodeType]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1117}
  ///
  StatusOr<google::cloud::vmwareengine::v1::NodeType> GetNodeType(
      google::cloud::vmwareengine::v1::GetNodeTypeRequest const& request,
      Options opts = {});

  ///
  /// Gets details of credentials for NSX appliance.
  ///
  /// @param private_cloud  Required. The resource name of the private cloud
  ///  to be queried for credentials.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Credentials,google/cloud/vmwareengine/v1/vmwareengine.proto#L1239}
  ///
  /// [google.cloud.vmwareengine.v1.ShowNsxCredentialsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1248}
  /// [google.cloud.vmwareengine.v1.Credentials]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1239}
  ///
  StatusOr<google::cloud::vmwareengine::v1::Credentials> ShowNsxCredentials(
      std::string const& private_cloud, Options opts = {});

  ///
  /// Gets details of credentials for NSX appliance.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ShowNsxCredentialsRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1248}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Credentials,google/cloud/vmwareengine/v1/vmwareengine.proto#L1239}
  ///
  /// [google.cloud.vmwareengine.v1.ShowNsxCredentialsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1248}
  /// [google.cloud.vmwareengine.v1.Credentials]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1239}
  ///
  StatusOr<google::cloud::vmwareengine::v1::Credentials> ShowNsxCredentials(
      google::cloud::vmwareengine::v1::ShowNsxCredentialsRequest const& request,
      Options opts = {});

  ///
  /// Gets details of credentials for Vcenter appliance.
  ///
  /// @param private_cloud  Required. The resource name of the private cloud
  ///  to be queried for credentials.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Credentials,google/cloud/vmwareengine/v1/vmwareengine.proto#L1239}
  ///
  /// [google.cloud.vmwareengine.v1.ShowVcenterCredentialsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1264}
  /// [google.cloud.vmwareengine.v1.Credentials]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1239}
  ///
  StatusOr<google::cloud::vmwareengine::v1::Credentials> ShowVcenterCredentials(
      std::string const& private_cloud, Options opts = {});

  ///
  /// Gets details of credentials for Vcenter appliance.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ShowVcenterCredentialsRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1264}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::Credentials,google/cloud/vmwareengine/v1/vmwareengine.proto#L1239}
  ///
  /// [google.cloud.vmwareengine.v1.ShowVcenterCredentialsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1264}
  /// [google.cloud.vmwareengine.v1.Credentials]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1239}
  ///
  StatusOr<google::cloud::vmwareengine::v1::Credentials> ShowVcenterCredentials(
      google::cloud::vmwareengine::v1::ShowVcenterCredentialsRequest const&
          request,
      Options opts = {});

  ///
  /// Resets credentials of the NSX appliance.
  ///
  /// @param private_cloud  Required. The resource name of the private cloud
  ///  to reset credentials for.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.ResetNsxCredentialsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1280}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  ResetNsxCredentials(std::string const& private_cloud, Options opts = {});

  ///
  /// Resets credentials of the NSX appliance.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ResetNsxCredentialsRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1280}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.ResetNsxCredentialsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1280}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  ResetNsxCredentials(
      google::cloud::vmwareengine::v1::ResetNsxCredentialsRequest const&
          request,
      Options opts = {});

  ///
  /// Resets credentials of the Vcenter appliance.
  ///
  /// @param private_cloud  Required. The resource name of the private cloud
  ///  to reset credentials for.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1-a/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.ResetVcenterCredentialsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1312}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  ResetVcenterCredentials(std::string const& private_cloud, Options opts = {});

  ///
  /// Resets credentials of the Vcenter appliance.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ResetVcenterCredentialsRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1312}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::PrivateCloud,google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  /// [google.cloud.vmwareengine.v1.ResetVcenterCredentialsRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1312}
  /// [google.cloud.vmwareengine.v1.PrivateCloud]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L482}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::PrivateCloud>>
  ResetVcenterCredentials(
      google::cloud::vmwareengine::v1::ResetVcenterCredentialsRequest const&
          request,
      Options opts = {});

  ///
  /// Creates a new HCX activation key in a given private cloud.
  ///
  /// @param parent  Required. The resource name of the private cloud to create
  /// the key for.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-central1/privateClouds/my-cloud`
  /// @param hcx_activation_key  Required. The initial description of a new HCX
  /// activation key. When
  ///  creating a new key, this field must be an empty object.
  /// @param hcx_activation_key_id  Required. The user-provided identifier of
  /// the `HcxActivationKey` to be
  ///  created. This identifier must be unique among `HcxActivationKey`
  ///  resources within the parent and becomes the final token in the name URI.
  ///  The identifier must meet the following requirements:
  ///  * Only contains 1-63 alphanumeric characters and hyphens
  ///  * Begins with an alphabetical character
  ///  * Ends with a non-hyphen character
  ///  * Not formatted as a UUID
  ///  * Complies with [RFC 1034](https://datatracker.ietf.org/doc/html/rfc1034)
  ///  (section 3.5)
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::HcxActivationKey,google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  /// [google.cloud.vmwareengine.v1.CreateHcxActivationKeyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1449}
  /// [google.cloud.vmwareengine.v1.HcxActivationKey]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::HcxActivationKey>>
  CreateHcxActivationKey(
      std::string const& parent,
      google::cloud::vmwareengine::v1::HcxActivationKey const&
          hcx_activation_key,
      std::string const& hcx_activation_key_id, Options opts = {});

  ///
  /// Creates a new HCX activation key in a given private cloud.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::CreateHcxActivationKeyRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1449}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::HcxActivationKey,google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  /// [google.cloud.vmwareengine.v1.CreateHcxActivationKeyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1449}
  /// [google.cloud.vmwareengine.v1.HcxActivationKey]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::HcxActivationKey>>
  CreateHcxActivationKey(
      google::cloud::vmwareengine::v1::CreateHcxActivationKeyRequest const&
          request,
      Options opts = {});

  ///
  /// Lists `HcxActivationKey` resources in a given private cloud.
  ///
  /// @param parent  Required. The resource name of the private cloud
  ///  to be queried for HCX activation keys.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-central1/privateClouds/my-cloud`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::HcxActivationKey,google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  /// [google.cloud.vmwareengine.v1.ListHcxActivationKeysRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1404}
  /// [google.cloud.vmwareengine.v1.HcxActivationKey]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  StreamRange<google::cloud::vmwareengine::v1::HcxActivationKey>
  ListHcxActivationKeys(std::string const& parent, Options opts = {});

  ///
  /// Lists `HcxActivationKey` resources in a given private cloud.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ListHcxActivationKeysRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1404}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::HcxActivationKey,google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  /// [google.cloud.vmwareengine.v1.ListHcxActivationKeysRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1404}
  /// [google.cloud.vmwareengine.v1.HcxActivationKey]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  StreamRange<google::cloud::vmwareengine::v1::HcxActivationKey>
  ListHcxActivationKeys(
      google::cloud::vmwareengine::v1::ListHcxActivationKeysRequest request,
      Options opts = {});

  ///
  /// Retrieves a `HcxActivationKey` resource by its resource name.
  ///
  /// @param name  Required. The resource name of the HCX activation key to
  /// retrieve.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-west1/privateClouds/my-cloud/hcxActivationKeys/my-key`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::HcxActivationKey,google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  /// [google.cloud.vmwareengine.v1.GetHcxActivationKeyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1434}
  /// [google.cloud.vmwareengine.v1.HcxActivationKey]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  StatusOr<google::cloud::vmwareengine::v1::HcxActivationKey>
  GetHcxActivationKey(std::string const& name, Options opts = {});

  ///
  /// Retrieves a `HcxActivationKey` resource by its resource name.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::GetHcxActivationKeyRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1434}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::HcxActivationKey,google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  /// [google.cloud.vmwareengine.v1.GetHcxActivationKeyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1434}
  /// [google.cloud.vmwareengine.v1.HcxActivationKey]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1362}
  ///
  StatusOr<google::cloud::vmwareengine::v1::HcxActivationKey>
  GetHcxActivationKey(
      google::cloud::vmwareengine::v1::GetHcxActivationKeyRequest const&
          request,
      Options opts = {});

  ///
  /// Retrieves a `NetworkPolicy` resource by its resource name.
  ///
  /// @param name  Required. The resource name of the network policy to
  /// retrieve.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-central1/networkPolicies/my-network-policy`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NetworkPolicy,google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  /// [google.cloud.vmwareengine.v1.GetNetworkPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1750}
  /// [google.cloud.vmwareengine.v1.NetworkPolicy]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  StatusOr<google::cloud::vmwareengine::v1::NetworkPolicy> GetNetworkPolicy(
      std::string const& name, Options opts = {});

  ///
  /// Retrieves a `NetworkPolicy` resource by its resource name.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::GetNetworkPolicyRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1750}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NetworkPolicy,google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  /// [google.cloud.vmwareengine.v1.GetNetworkPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1750}
  /// [google.cloud.vmwareengine.v1.NetworkPolicy]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  StatusOr<google::cloud::vmwareengine::v1::NetworkPolicy> GetNetworkPolicy(
      google::cloud::vmwareengine::v1::GetNetworkPolicyRequest const& request,
      Options opts = {});

  ///
  /// Lists `NetworkPolicy` resources in a specified project and location.
  ///
  /// @param parent  Required. The resource name of the location (region) to
  /// query for
  ///  network policies. Resource names are schemeless URIs that follow the
  ///  conventions in https://cloud.google.com/apis/design/resource_names. For
  ///  example: `projects/my-project/locations/us-central1`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NetworkPolicy,google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  /// [google.cloud.vmwareengine.v1.ListNetworkPoliciesRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1674}
  /// [google.cloud.vmwareengine.v1.NetworkPolicy]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  StreamRange<google::cloud::vmwareengine::v1::NetworkPolicy>
  ListNetworkPolicies(std::string const& parent, Options opts = {});

  ///
  /// Lists `NetworkPolicy` resources in a specified project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ListNetworkPoliciesRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1674}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NetworkPolicy,google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  /// [google.cloud.vmwareengine.v1.ListNetworkPoliciesRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1674}
  /// [google.cloud.vmwareengine.v1.NetworkPolicy]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  StreamRange<google::cloud::vmwareengine::v1::NetworkPolicy>
  ListNetworkPolicies(
      google::cloud::vmwareengine::v1::ListNetworkPoliciesRequest request,
      Options opts = {});

  ///
  /// Creates a new network policy in a given VMware Engine network of a
  /// project and location (region). A new network policy cannot be created if
  /// another network policy already exists in the same scope.
  ///
  /// @param parent  Required. The resource name of the location (region)
  ///  to create the new network policy in.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///   `projects/my-project/locations/us-central1`
  /// @param network_policy  Required. The network policy configuration to use
  /// in the request.
  /// @param network_policy_id  Required. The user-provided identifier of the
  /// network policy to be created.
  ///  This identifier must be unique within parent
  ///  `projects/{my-project}/locations/{us-central1}/networkPolicies` and
  ///  becomes the final token in the name URI. The identifier must meet the
  ///  following requirements:
  ///  * Only contains 1-63 alphanumeric characters and hyphens
  ///  * Begins with an alphabetical character
  ///  * Ends with a non-hyphen character
  ///  * Not formatted as a UUID
  ///  * Complies with [RFC 1034](https://datatracker.ietf.org/doc/html/rfc1034)
  ///  (section 3.5)
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NetworkPolicy,google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  /// [google.cloud.vmwareengine.v1.CreateNetworkPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1794}
  /// [google.cloud.vmwareengine.v1.NetworkPolicy]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::NetworkPolicy>>
  CreateNetworkPolicy(
      std::string const& parent,
      google::cloud::vmwareengine::v1::NetworkPolicy const& network_policy,
      std::string const& network_policy_id, Options opts = {});

  ///
  /// Creates a new network policy in a given VMware Engine network of a
  /// project and location (region). A new network policy cannot be created if
  /// another network policy already exists in the same scope.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::CreateNetworkPolicyRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1794}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NetworkPolicy,google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  /// [google.cloud.vmwareengine.v1.CreateNetworkPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1794}
  /// [google.cloud.vmwareengine.v1.NetworkPolicy]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::NetworkPolicy>>
  CreateNetworkPolicy(
      google::cloud::vmwareengine::v1::CreateNetworkPolicyRequest const&
          request,
      Options opts = {});

  ///
  /// Modifies a `NetworkPolicy` resource. Only the following fields can be
  /// updated: `internet_access`, `external_ip`, `edge_services_cidr`.
  /// Only fields specified in `updateMask` are applied. When updating a network
  /// policy, the external IP network service can only be disabled if there are
  /// no external IP addresses present in the scope of the policy. Also, a
  /// `NetworkService` cannot be updated when `NetworkService.state` is set
  /// to `RECONCILING`.
  ///
  /// During operation processing, the resource is temporarily in the `ACTIVE`
  /// state before the operation fully completes. For that period of time, you
  /// can't update the resource. Use the operation status to determine when the
  /// processing fully completes.
  ///
  /// @param network_policy  Required. Network policy description.
  /// @param update_mask  Required. Field mask is used to specify the fields to
  /// be overwritten in the
  ///  `NetworkPolicy` resource by the update.
  ///  The fields specified in the `update_mask` are relative to the resource,
  ///  not the full request. A field will be overwritten if it is in the mask.
  ///  If the user does not provide a mask then all fields will be overwritten.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NetworkPolicy,google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  /// [google.cloud.vmwareengine.v1.UpdateNetworkPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1765}
  /// [google.cloud.vmwareengine.v1.NetworkPolicy]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::NetworkPolicy>>
  UpdateNetworkPolicy(
      google::cloud::vmwareengine::v1::NetworkPolicy const& network_policy,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Modifies a `NetworkPolicy` resource. Only the following fields can be
  /// updated: `internet_access`, `external_ip`, `edge_services_cidr`.
  /// Only fields specified in `updateMask` are applied. When updating a network
  /// policy, the external IP network service can only be disabled if there are
  /// no external IP addresses present in the scope of the policy. Also, a
  /// `NetworkService` cannot be updated when `NetworkService.state` is set
  /// to `RECONCILING`.
  ///
  /// During operation processing, the resource is temporarily in the `ACTIVE`
  /// state before the operation fully completes. For that period of time, you
  /// can't update the resource. Use the operation status to determine when the
  /// processing fully completes.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::UpdateNetworkPolicyRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1765}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::NetworkPolicy,google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  /// [google.cloud.vmwareengine.v1.UpdateNetworkPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1765}
  /// [google.cloud.vmwareengine.v1.NetworkPolicy]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1583}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::NetworkPolicy>>
  UpdateNetworkPolicy(
      google::cloud::vmwareengine::v1::UpdateNetworkPolicyRequest const&
          request,
      Options opts = {});

  ///
  /// Deletes a `NetworkPolicy` resource. A network policy cannot be deleted
  /// when `NetworkService.state` is set to `RECONCILING` for either its
  /// external IP or internet access service.
  ///
  /// @param name  Required. The resource name of the network policy to delete.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/us-central1/networkPolicies/my-network-policy`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::OperationMetadata,google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  /// [google.cloud.vmwareengine.v1.DeleteNetworkPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1843}
  /// [google.cloud.vmwareengine.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::OperationMetadata>>
  DeleteNetworkPolicy(std::string const& name, Options opts = {});

  ///
  /// Deletes a `NetworkPolicy` resource. A network policy cannot be deleted
  /// when `NetworkService.state` is set to `RECONCILING` for either its
  /// external IP or internet access service.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::DeleteNetworkPolicyRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1843}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::OperationMetadata,google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  /// [google.cloud.vmwareengine.v1.DeleteNetworkPolicyRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1843}
  /// [google.cloud.vmwareengine.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::OperationMetadata>>
  DeleteNetworkPolicy(
      google::cloud::vmwareengine::v1::DeleteNetworkPolicyRequest const&
          request,
      Options opts = {});

  ///
  /// Creates a new VMware Engine network that can be used by a private cloud.
  ///
  /// @param parent  Required. The resource name of the location to create the
  /// new VMware Engine
  ///  network in. A VMware Engine network of type
  ///  `LEGACY` is a regional resource, and a VMware
  ///  Engine network of type `STANDARD` is a global resource.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names. For example:
  ///  `projects/my-project/locations/global`
  /// @param vmware_engine_network  Required. The initial description of the new
  /// VMware Engine network.
  /// @param vmware_engine_network_id  Required. The user-provided identifier of
  /// the new VMware Engine network.
  ///  This identifier must be unique among VMware Engine network resources
  ///  within the parent and becomes the final token in the name URI. The
  ///  identifier must meet the following requirements:
  ///  * For networks of type LEGACY, adheres to the format:
  ///  `{region-id}-default`. Replace `{region-id}` with the region where you
  ///  want to create the VMware Engine network. For example,
  ///  "us-west1-default".
  ///  * Only contains 1-63 alphanumeric characters and hyphens
  ///  * Begins with an alphabetical character
  ///  * Ends with a non-hyphen character
  ///  * Not formatted as a UUID
  ///  * Complies with [RFC 1034](https://datatracker.ietf.org/doc/html/rfc1034)
  ///  (section 3.5)
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::VmwareEngineNetwork,google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  /// [google.cloud.vmwareengine.v1.CreateVmwareEngineNetworkRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1986}
  /// [google.cloud.vmwareengine.v1.VmwareEngineNetwork]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::VmwareEngineNetwork>>
  CreateVmwareEngineNetwork(
      std::string const& parent,
      google::cloud::vmwareengine::v1::VmwareEngineNetwork const&
          vmware_engine_network,
      std::string const& vmware_engine_network_id, Options opts = {});

  ///
  /// Creates a new VMware Engine network that can be used by a private cloud.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::CreateVmwareEngineNetworkRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L1986}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::VmwareEngineNetwork,google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  /// [google.cloud.vmwareengine.v1.CreateVmwareEngineNetworkRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1986}
  /// [google.cloud.vmwareengine.v1.VmwareEngineNetwork]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::VmwareEngineNetwork>>
  CreateVmwareEngineNetwork(
      google::cloud::vmwareengine::v1::CreateVmwareEngineNetworkRequest const&
          request,
      Options opts = {});

  ///
  /// Modifies a VMware Engine network resource. Only the following fields can
  /// be updated: `description`. Only fields specified in `updateMask` are
  /// applied.
  ///
  /// @param vmware_engine_network  Required. VMware Engine network description.
  /// @param update_mask  Required. Field mask is used to specify the fields to
  /// be overwritten in the
  ///  VMware Engine network resource by the update.
  ///  The fields specified in the `update_mask` are relative to the resource,
  ///  not the full request. A field will be overwritten if it is in the mask.
  ///  If the user does not provide a mask then all fields will be overwritten.
  ///  Only the following fields can be updated: `description`.
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::VmwareEngineNetwork,google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  /// [google.cloud.vmwareengine.v1.UpdateVmwareEngineNetworkRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L2038}
  /// [google.cloud.vmwareengine.v1.VmwareEngineNetwork]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::VmwareEngineNetwork>>
  UpdateVmwareEngineNetwork(
      google::cloud::vmwareengine::v1::VmwareEngineNetwork const&
          vmware_engine_network,
      google::protobuf::FieldMask const& update_mask, Options opts = {});

  ///
  /// Modifies a VMware Engine network resource. Only the following fields can
  /// be updated: `description`. Only fields specified in `updateMask` are
  /// applied.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::UpdateVmwareEngineNetworkRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L2038}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::VmwareEngineNetwork,google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  /// [google.cloud.vmwareengine.v1.UpdateVmwareEngineNetworkRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L2038}
  /// [google.cloud.vmwareengine.v1.VmwareEngineNetwork]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::VmwareEngineNetwork>>
  UpdateVmwareEngineNetwork(
      google::cloud::vmwareengine::v1::UpdateVmwareEngineNetworkRequest const&
          request,
      Options opts = {});

  ///
  /// Deletes a `VmwareEngineNetwork` resource. You can only delete a VMware
  /// Engine network after all resources that refer to it are deleted. For
  /// example, a private cloud, a network peering, and a network policy can all
  /// refer to the same VMware Engine network.
  ///
  /// @param name  Required. The resource name of the VMware Engine network to
  /// be deleted.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/global/vmwareEngineNetworks/my-network`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::OperationMetadata,google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  /// [google.cloud.vmwareengine.v1.DeleteVmwareEngineNetworkRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L2068}
  /// [google.cloud.vmwareengine.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::OperationMetadata>>
  DeleteVmwareEngineNetwork(std::string const& name, Options opts = {});

  ///
  /// Deletes a `VmwareEngineNetwork` resource. You can only delete a VMware
  /// Engine network after all resources that refer to it are deleted. For
  /// example, a private cloud, a network peering, and a network policy can all
  /// refer to the same VMware Engine network.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::DeleteVmwareEngineNetworkRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L2068}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::OperationMetadata,google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  /// [google.cloud.vmwareengine.v1.DeleteVmwareEngineNetworkRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L2068}
  /// [google.cloud.vmwareengine.v1.OperationMetadata]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1089}
  ///
  future<StatusOr<google::cloud::vmwareengine::v1::OperationMetadata>>
  DeleteVmwareEngineNetwork(
      google::cloud::vmwareengine::v1::DeleteVmwareEngineNetworkRequest const&
          request,
      Options opts = {});

  ///
  /// Retrieves a `VmwareEngineNetwork` resource by its resource name. The
  /// resource contains details of the VMware Engine network, such as its VMware
  /// Engine network type, peered networks in a service project, and state
  /// (for example, `CREATING`, `ACTIVE`, `DELETING`).
  ///
  /// @param name  Required. The resource name of the VMware Engine network to
  /// retrieve.
  ///  Resource names are schemeless URIs that follow the conventions in
  ///  https://cloud.google.com/apis/design/resource_names.
  ///  For example:
  ///  `projects/my-project/locations/global/vmwareEngineNetworks/my-network`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::VmwareEngineNetwork,google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  /// [google.cloud.vmwareengine.v1.GetVmwareEngineNetworkRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L2105}
  /// [google.cloud.vmwareengine.v1.VmwareEngineNetwork]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  StatusOr<google::cloud::vmwareengine::v1::VmwareEngineNetwork>
  GetVmwareEngineNetwork(std::string const& name, Options opts = {});

  ///
  /// Retrieves a `VmwareEngineNetwork` resource by its resource name. The
  /// resource contains details of the VMware Engine network, such as its VMware
  /// Engine network type, peered networks in a service project, and state
  /// (for example, `CREATING`, `ACTIVE`, `DELETING`).
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::GetVmwareEngineNetworkRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L2105}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::VmwareEngineNetwork,google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  /// [google.cloud.vmwareengine.v1.GetVmwareEngineNetworkRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L2105}
  /// [google.cloud.vmwareengine.v1.VmwareEngineNetwork]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  StatusOr<google::cloud::vmwareengine::v1::VmwareEngineNetwork>
  GetVmwareEngineNetwork(
      google::cloud::vmwareengine::v1::GetVmwareEngineNetworkRequest const&
          request,
      Options opts = {});

  ///
  /// Lists `VmwareEngineNetwork` resources in a given project and location.
  ///
  /// @param parent  Required. The resource name of the location to query for
  ///  VMware Engine networks. Resource names are schemeless URIs that follow
  ///  the conventions in https://cloud.google.com/apis/design/resource_names.
  ///  For example: `projects/my-project/locations/global`
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::VmwareEngineNetwork,google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  /// [google.cloud.vmwareengine.v1.ListVmwareEngineNetworksRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L2120}
  /// [google.cloud.vmwareengine.v1.VmwareEngineNetwork]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  StreamRange<google::cloud::vmwareengine::v1::VmwareEngineNetwork>
  ListVmwareEngineNetworks(std::string const& parent, Options opts = {});

  ///
  /// Lists `VmwareEngineNetwork` resources in a given project and location.
  ///
  /// @param request
  /// @googleapis_link{google::cloud::vmwareengine::v1::ListVmwareEngineNetworksRequest,google/cloud/vmwareengine/v1/vmwareengine.proto#L2120}
  /// @param opts Optional. Override the class-level options, such as retry and
  ///     backoff policies.
  /// @return
  /// @googleapis_link{google::cloud::vmwareengine::v1::VmwareEngineNetwork,google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  /// [google.cloud.vmwareengine.v1.ListVmwareEngineNetworksRequest]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L2120}
  /// [google.cloud.vmwareengine.v1.VmwareEngineNetwork]:
  /// @googleapis_reference_link{google/cloud/vmwareengine/v1/vmwareengine.proto#L1875}
  ///
  StreamRange<google::cloud::vmwareengine::v1::VmwareEngineNetwork>
  ListVmwareEngineNetworks(
      google::cloud::vmwareengine::v1::ListVmwareEngineNetworksRequest request,
      Options opts = {});

 private:
  std::shared_ptr<VmwareEngineConnection> connection_;
  Options options_;
};

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace vmwareengine_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_VMWAREENGINE_V1_VMWARE_ENGINE_CLIENT_H
